@@ -75,7 +75,8 @@
  */
 #pragma ident	"@(#) $Id: dynodump.c,v 1.8 1996/05/23 18:39:21 georgn Exp $ - SMI"
 
-#define __EXTENSIONS__
+#define __EXTENSIONS__ 1
+
 #include	<sys/param.h>
 #include	<sys/procfs.h>
 #include	<fcntl.h>
@@ -214,7 +215,7 @@ dynodump(const char * file)
 	return (elferr("elf_getscn"));
     if ((data = elf_getdata(scn, NULL)) == NULL)
 	return (elferr("elf_getdata"));
-    istrs = data->d_buf;
+    istrs = (char *) data->d_buf;
 
     /*
      * Construct a cache to maintain the input files section information.
@@ -449,7 +450,7 @@ dynodump(const char * file)
 	return (elferr("elf_getscn"));
     if ((data = elf_getdata(scn, NULL)) == NULL)
 	return (elferr("elf_getdata"));
-    ostrs = _ostrs = data->d_buf;
+    ostrs = _ostrs = (char *) data->d_buf;
     *_ostrs++ = '\0';
 
     /*
