@@ -13,7 +13,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+the Free the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+Boston, MA 02111-1307, USA.  */
 
 /* Synched up with: FSF 19.28. */
 
@@ -40,16 +41,16 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include <string.h>
 
 #if __STDC__ || defined(STDC_HEADERS)
-#include <stdlib.h>
+# include <stdlib.h>
 #else
 char *malloc ();
+char *getenv ();
 #endif
 static void *xmalloc (unsigned int);
 static void *xrealloc (char *ptr, unsigned int);
 static void skip_to_lf (FILE *stream);
-static void fatal (char *s1, char *s2);
-static void error (char *s1, char *s2);
-char *getenv ();
+static void fatal (CONST char *s1, CONST char *s2);
+static void error (CONST char *s1, CONST char *s2);
 
 int
 main (argc, argv)
@@ -149,16 +150,14 @@ xrealloc (ptr, size)
 /* Print error message and exit.  */
 
 static void
-fatal (s1, s2)
-     char *s1, *s2;
+fatal (CONST char *s1, CONST char *s2)
 {
   error (s1, s2);
   exit (1);
 }
 
 static void
-error (s1, s2)
-     char *s1, *s2;
+error (CONST char *s1, CONST char *s2)
 {
   fprintf (stderr, "cvtmail: ");
   fprintf (stderr, s1, s2);
