@@ -106,10 +106,8 @@ static char header_rcsid [] = "!Header: gnuserv.h,v 2.4 95/02/16 11:58:11 arup a
 #include <sys/param.h>
 #include <sys/stat.h>
 #include <stdio.h>
-#ifdef STDC_HEADERS
 #include <stdlib.h>
 #include <string.h>
-#endif /* STDC_HEADERS */
 #include <signal.h>
 #include <errno.h>
 
@@ -208,14 +206,15 @@ extern char *progname;
 #define CONN_IPC      2
 
 /* function declarations */
-extern int make_connection (char *hostarg, int portarg, int *s);
+int make_connection (char *hostarg, int portarg, int *s);
 #ifdef SYSV_IPC
-extern void disconnect_from_ipc_server();
+void disconnect_from_ipc_server();
 #endif
 #if defined(INTERNET_DOMAIN_SOCKETS) || defined(UNIX_DOMAIN_SOCKETS)
-extern void send_string (int s, CONST char *msg);
-extern void disconnect_from_server (int s, int echo);
+void send_string (int s, CONST char *msg);
+void disconnect_from_server (int s, int echo);
+int read_line (int s, char *dest);
 #endif
 #ifdef INTERNET_DOMAIN_SOCKETS
-extern int internet_addr (char *host);
+int internet_addr (char *host);
 #endif
