@@ -33,8 +33,8 @@
 static char rcsid [] = "!Header: gnuslib.c,v 2.4 95/02/16 11:57:37 arup alpha !";
 #endif
 
-#include <errno.h>
 #include "gnuserv.h"
+#include <errno.h>
 
 #ifdef SYSV_IPC
 static int connect_to_ipc_server (void);
@@ -63,8 +63,12 @@ static int connect_to_internet_server (char *serverhost, u_short port);
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif /* HAVE_UNISTD_H */
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif /* HAVE_STRING_H */
 
 #include <arpa/inet.h>
 
@@ -76,7 +80,6 @@ int make_connection(hostarg, portarg, s)
      int *s;
 {
 #ifdef INTERNET_DOMAIN_SOCKETS
-  char localhost[HOSTNAMSZ];
   char *ptr;
   if (hostarg == NULL)
     hostarg = getenv("GNU_HOST");
@@ -103,19 +106,20 @@ int make_connection(hostarg, portarg, s)
   } else {
     /* no hostname given.  Use unix-domain/sysv-ipc, or
      * internet-domain connection to local host if they're not available. */
-#ifdef UNIX_DOMAIN_SOCKETS
+#if   defined(UNIX_DOMAIN_SOCKETS)
     *s = connect_to_unix_server();
     return (int) CONN_UNIX;
-#endif
-#ifdef SYSV_IPC
+#elif defined(SYSV_IPC)
     *s = connect_to_ipc_server();
     return (int) CONN_IPC;
-#endif
-#ifdef INTERNET_DOMAIN_SOCKETS
-    gethostname(localhost,HOSTNAMSZ);	  /* use this host by default */    
-    *s = connect_to_internet_server(localhost, portarg);
-    return (int) CONN_INTERNET;
-#endif      
+#elif defined(INTERNET_DOMAIN_SOCKETS)
+    {
+      char localhost[HOSTNAMSZ];
+      gethostname(localhost,HOSTNAMSZ);	  /* use this host by default */    
+      *s = connect_to_internet_server(localhost, portarg);
+      return (int) CONN_INTERNET;
+    }
+#endif /* IPC type */
   }
 }
 
@@ -221,11 +225,8 @@ void send_string(s,msg)
 /*
   read_line -- read a \n terminated line from a socket
 */
-int read_line(s,dest)
-     int s;
-     char *dest;
+int read_line(int s, char *dest)
 {
-  char *index = NULL;
   int length;
   int offset=0;
   char buffer[GSERV_BUFSZ+1];
