@@ -67,7 +67,7 @@ echo "          with $REAL..."
 
 $EMACS -batch -vanilla -l `pwd`/lisp/prim/cleantree -f batch-remove-old-elc lisp
 
-prune_vc="( -name SCCS -o -name RCS -o -name CVS ) -prune -o"
+prune_vc="( -name '.*' -o -name SCCS -o -name RCS -o -name CVS ) -prune -o"
 
 # $els  is a list of all .el  files
 # $elcs is a list of all .elc files
@@ -137,11 +137,9 @@ fi
 make_special efs x20
 make_special eos -k		# not strictly necessary...
 make_special gnus  some
-make_special hyperbole elc
 # We're not ready for the following, yet.
 #make_special ilisp XEmacsELC=custom-load.elc elc
 make_special ilisp elc
-make_special oobr HYPB_ELC='' elc
 make_special w3 xemacs-w3
 
 for dir in $ignore_dirs ; do
