@@ -214,6 +214,29 @@ window for repective \\[gdb-display-frame] commands."
 			     (not gdb-display-mode)
 			   (> (prefix-numeric-value arg) 0))))
 
+;; Using cc-mode's syntax table is broken.
+(defvar gdb-mode-syntax-table nil
+  "Syntax table for GDB mode.")
+
+;; This is adapted from CC Mode 5.11.
+(unless gdb-mode-syntax-table
+  (setq gdb-mode-syntax-table (make-syntax-table))
+  ;; DO NOT TRY TO SET _ (UNDERSCORE) TO WORD CLASS!
+  (modify-syntax-entry ?_  "_" gdb-mode-syntax-table)
+  (modify-syntax-entry ?\\ "\\" gdb-mode-syntax-table)
+  (modify-syntax-entry ?+  "." gdb-mode-syntax-table)
+  (modify-syntax-entry ?-  "." gdb-mode-syntax-table)
+  (modify-syntax-entry ?=  "." gdb-mode-syntax-table)
+  (modify-syntax-entry ?%  "." gdb-mode-syntax-table)
+  (modify-syntax-entry ?<  "." gdb-mode-syntax-table)
+  (modify-syntax-entry ?>  "." gdb-mode-syntax-table)
+  (modify-syntax-entry ?&  "." gdb-mode-syntax-table)
+  (modify-syntax-entry ?|  "." gdb-mode-syntax-table)
+  (modify-syntax-entry ?\' "\"" gdb-mode-syntax-table)
+  ;; add extra comment syntax
+  (modify-syntax-entry ?/  ". 14"  gdb-mode-syntax-table)
+  (modify-syntax-entry ?*  ". 23"  gdb-mode-syntax-table))
+
 
 (defun gdb-mode ()
   "Major mode for interacting with an inferior Gdb process.
@@ -243,9 +266,7 @@ C-x SPACE sets break point at current line."
   (interactive)
   (comint-mode)
   (use-local-map gdb-mode-map)
-  (when (not (boundp 'c-mode-syntax-table))
-    (require 'cc-mode))
-  (set-syntax-table c-mode-syntax-table)
+  (set-syntax-table gdb-mode-syntax-table)
   (make-local-variable 'gdb-last-frame-displayed-p)
   (make-local-variable 'gdb-last-frame)
   (make-local-variable 'gdb-delete-prompt-marker)
@@ -267,7 +288,7 @@ C-x SPACE sets break point at current line."
   ;; XEmacs change:
   (make-local-hook 'kill-buffer-hook)
   (add-hook 'kill-buffer-hook 'gdb-delete-arrow-extent nil t)
-  (setq comint-input-sentinel 'shell-directory-tracker)
+  (add-hook 'comint-input-filter-functions 'shell-directory-tracker nil t)
   (run-hooks 'gdb-mode-hook))
 
 (defun gdb-delete-arrow-extent ()
@@ -310,7 +331,7 @@ the GDB commands `cd DIR' and `directory'."
     (gdb-mode)
     (gdb-set-buffer)))
 
-;;;####autoload
+;;;###autoload
 (defun gdb-with-core (file corefile)
   "Debug a program using a corefile."
   (interactive "fProgram to debug: \nfCore file to use: ")
@@ -652,11 +673,13 @@ It is for customization by you.")
     (goto-char (point-max))
     (insert comm)))
 
-(defun gdb-control-c-subjob ()
-  "Send a Control-C to the subprocess."
-  (interactive)
-  (process-send-string (get-buffer-process (current-buffer))
-		       "\C-c"))
+(fset 'gdb-control-c-subjob 'comint-interrupt-subjob)
+
+;(defun gdb-control-c-subjob ()
+;  "Send a Control-C to the subprocess."
+;  (interactive)
+;  (process-send-string (get-buffer-process (current-buffer))
+;		       "\C-c"))
 
 (defun gdb-toolbar-break ()
   (interactive)
