@@ -54,7 +54,7 @@
 (require 'comint)
 
 (defgroup telnet nil
-  "Telnet/rsh stuff"
+  "Run a telnet session from within an Emacs buffer."
   :group 'comint)
 
 (defvar telnet-new-line "\r")
@@ -76,6 +76,7 @@ while looking for the initial password.")
 
 (defcustom telnet-program "telnet"
   "*Program to run to open a telnet connection."
+  :type 'string
   :group 'telnet)
 
 (defcustom rsh-eat-password-string nil
@@ -298,7 +299,7 @@ See also `\\[telnet]'."
   (require 'shell)
   (let ((name (concat "rsh-" host)))
     (pop-to-buffer (make-comint name remote-shell-program nil host))
-    (setq telnet-count telnet-initial-count)
+    (set (make-local-variable 'telnet-count) telnet-initial-count)
     ;;
     ;; SunOS doesn't print "unix" in its rsh login banner, so let's get a
     ;; reasonable default here.  There do exist non-Unix machines which
