@@ -86,7 +86,7 @@ This shouldn't match socket or symbolic link lines (which aren't editable).")
      ["Find in Other Window" dired-find-file-other-window t]
      ["Find in Other Frame" dired-find-file-other-frame t]
      ["View File" dired-view-file t]
-     ["Display in Other Window" dired-find-file-other-window t]))
+     ["Display in Other Window" dired-display-file t]))
 
   (setq
    dired-do-popup-menu
@@ -165,6 +165,7 @@ This shouldn't match socket or symbolic link lines (which aren't editable).")
      ["Change Mode..." dired-do-interactive-chmod t]
      ["Change Owner..." dired-do-chown t]
      ["Change Group..." dired-do-chgrp t]
+     ["Load" dired-do-load t]
      ["Byte-compile" dired-do-byte-compile t]
      ["Hardlink to..." dired-do-hardlink t]
      ["Symlink to..." dired-do-symlink t]
@@ -277,7 +278,6 @@ This shouldn't match socket or symbolic link lines (which aren't editable).")
   (if (and (featurep 'menubar) current-menubar)
       (progn
 	(let ((buffer-menubar (copy-sequence current-menubar)))
-	  (delete (assoc "Edit" buffer-menubar) buffer-menubar)
 	  (set-buffer-menubar buffer-menubar)
 	  (mapcar
 	   (function
@@ -293,20 +293,39 @@ This shouldn't match socket or symbolic link lines (which aren't editable).")
 
 ;;; Mouse functions
 
-(defun dired-mouse-find-file (event)
-  "In dired, visit the file or directory name you click on."
-  (interactive "e")
+(defun dired-mouse-file-action (event fun)
+  "In dired, apply function FUN to the file or directory name you click on."
   (save-excursion
     (set-buffer (window-buffer (event-window event)))
     (if dired-subdir-alist
 	(save-excursion
 	  (goto-char (event-point event))
-	  (dired-find-file))
+	  (funcall fun))
       (error
        (concat "dired-subdir-alist seems to be mangled.  "
 	       (substitute-command-keys
 		"\\<dired-mode-map>Try dired-revert (\\[dired-revert])."))))))
 
+(defun dired-mouse-find-file (event)
+  "In dired, visit the file or directory name you click on."
+  (interactive "e")
+  (dired-mouse-file-action event 'dired-find-file))
+
+(defun dired-mouse-display-file (event)
+  "In dired, display the file or directory name you click on."
+  (interactive "e")
+  (dired-mouse-file-action event 'dired-display-file))
+
+(defun dired-mouse-find-file-other-window (event)
+  "In dired, visit the file or directory name you click on in another window."
+  (interactive "e")
+  (dired-mouse-file-action event 'dired-find-file-other-window))
+
+(defun dired-mouse-find-file-other-frame (event)
+  "In dired, visit the file or directory name you click on in another frame."
+  (interactive "e")
+  (dired-mouse-file-action event 'dired-find-file-other-frame))
+
 (defun dired-mouse-mark (event)
   "In dired, mark the file name that you click on.
 If the file name is already marked, this unmarks it."
@@ -335,7 +354,7 @@ If the file name is already flag, this unflags it."
 	(save-excursion
 	  (goto-char (event-point event))
 	  (beginning-of-line)
-	  (if (char-equal (following-char) dired-del-marker)
+	  (if (char-equal (char-after (point)) dired-del-marker)
 	      (dired-unflag 1)
 	    (dired-flag-file-deletion 1)))
       (error
@@ -404,11 +423,13 @@ If the file name is already flag, this unflags it."
 (defvar dired-filename-local-map
   (let ((map (make-sparse-keymap)))
     (set-keymap-name map 'dired-filename-local-map)
-    (define-key map 'button2 'dired-mouse-find-file)
-    (define-key map 'button3 'dired-visit-popup-menu)
-    (define-key map '(control button2) 'dired-do-popup-menu)
-    (define-key map '(shift button1) 'dired-mouse-mark)
-    (define-key map '(control shift button1) 'dired-mouse-flag)
+    (define-key map [button2]           'dired-mouse-find-file)
+    (define-key map [(shift button2)]   'dired-mouse-display-file)
+    (define-key map [(meta button2)]    'dired-mouse-find-file-other-frame)
+    (define-key map [button3]           'dired-visit-popup-menu)
+    (define-key map [(control button2)] 'dired-do-popup-menu)
+    (define-key map [(shift button1)]   'dired-mouse-mark)
+    (define-key map [(control shift button1)] 'dired-mouse-flag)
     map)
   "Keymap used to activate actions on files in dired.")
 
@@ -465,7 +486,9 @@ If the file name is already flag, this unflags it."
 		  ((looking-at dired-re-raw-socket) 'dired-face-socket)
 		  ((save-excursion
 		     (goto-char start)
-		     (re-search-forward dired-re-raw-boring eol t))
+		     (save-restriction
+		       (narrow-to-region (point) eol)
+		       (re-search-forward dired-re-raw-boring eol t)))
 		   'dired-face-boring))))))
 
         (setq bol (1+ eol))
@@ -485,7 +508,7 @@ If the file name is already flag, this unflags it."
   (let ((mark
 	 (save-excursion
 	   (skip-chars-backward "^\n\r")
-	   (following-char)))
+	   (char-after (point))))
 	(face (extent-face extent)))
     (if (char-equal mark ?\ )
 	(if (consp face)
@@ -566,10 +589,11 @@ If the file name is already flag, this unflags it."
 	 (failure (apply (function dired-check-process)
 			 operation
 			 "chmod" new-attribute (list file))))
-    (dired-do-redisplay) 
+    (dired-do-redisplay)
     (if failure
 	(dired-log-summary (buffer-name (current-buffer))
-			   (format "%s: error" operation) nil))))
+			   (format "%s: error" operation) nil)
+      (forward-char 1))))
 
 (defun dired-chmod-popup-menu (event menu)
   (save-excursion
