@@ -1,4 +1,4 @@
-;; mailcrypt.el v3.2, mail encryption with PGP
+;; mailcrypt.el v3.4, mail encryption with PGP
 ;; Copyright (C) 1995  Jin Choi <jin@atype.com>
 ;;                     Patrick LoPresti <patl@lcs.mit.edu>
 ;; Any comments or suggestions welcome.
@@ -41,18 +41,20 @@
   (if (not (fboundp 'buffer-substring-no-properties))
       (fset 'buffer-substring-no-properties 'buffer-substring)))
 
-(defconst mc-xemacs-p (string-match "Xemacs" emacs-version))
+(defconst mc-xemacs-p (string-match "XEmacs" emacs-version))
 
-(autoload 'mc-decrypt "mc-toplev" nil t)
-(autoload 'mc-verify "mc-toplev" nil t)
-(autoload 'mc-snarf "mc-toplev" nil t)
-(autoload 'mc-pgp-fetch-key "mc-pgp" nil t)
-(autoload 'mc-encrypt "mc-toplev" nil t)
-(autoload 'mc-sign "mc-toplev" nil t)
-(autoload 'mc-insert-public-key "mc-toplev" nil t)
-(autoload 'mc-remailer-encrypt-for-chain "mc-remail" nil t)
-(autoload 'mc-remailer-insert-response-block "mc-remail" nil t)
-(autoload 'mc-remailer-insert-pseudonym "mc-remail" nil t)
+(if (not mc-xemacs-p)
+    (progn
+      (autoload 'mc-decrypt "mc-toplev" nil t)
+      (autoload 'mc-verify "mc-toplev" nil t)
+      (autoload 'mc-snarf "mc-toplev" nil t)
+      (autoload 'mc-pgp-fetch-key "mc-pgp" nil t)
+      (autoload 'mc-encrypt "mc-toplev" nil t)
+      (autoload 'mc-sign "mc-toplev" nil t)
+      (autoload 'mc-insert-public-key "mc-toplev" nil t)
+      (autoload 'mc-remailer-encrypt-for-chain "mc-remail" nil t)
+      (autoload 'mc-remailer-insert-response-block "mc-remail" nil t)
+      (autoload 'mc-remailer-insert-pseudonym "mc-remail" nil t)))
 
 ;;}}}
 
@@ -146,7 +148,7 @@
 
 (defun mc-read-mode (&optional arg)
   "\nMinor mode for interfacing with cryptographic functions.
-
+\\<mc-read-mode-map>
 \\[mc-decrypt]\t\tDecrypt an encrypted message
 \\[mc-verify]\t\tVerify signature on a clearsigned message
 \\[mc-snarf]\t\tAdd public key(s) to keyring
@@ -163,7 +165,7 @@
 	
 (defun mc-write-mode (&optional arg)
   "\nMinor mode for interfacing with cryptographic functions.
-
+\\<mc-write-mode-map>
 \\[mc-encrypt]\t\tEncrypt (and optionally sign) message
 \\[mc-sign]\t\tClearsign message
 \\[mc-insert-public-key]\t\tExtract public key from keyring and insert into message
@@ -181,10 +183,12 @@
       (easy-menu-add mc-write-mode-menu)
     (easy-menu-remove mc-write-mode-menu)))
 
+;;;###autoload
 (defun mc-install-read-mode ()
   (interactive)
   (mc-read-mode 1))
 
+;;;###autoload
 (defun mc-install-write-mode ()
   (interactive)
   (mc-write-mode 1))
@@ -197,7 +201,7 @@
 ;;}}}
 
 ;;{{{ User variables.
-(defconst mc-version "3.3")
+(defconst mc-version "3.4")
 (defvar mc-default-scheme 'mc-scheme-pgp "*Default encryption scheme to use.")
 (defvar mc-passwd-timeout 60
   "*Time to deactivate password in seconds after a use.
@@ -243,15 +247,24 @@ If 'never, always use a viewer instead of replacing.")
     (vm-mode (decrypt . mc-vm-decrypt-message)
 	     (verify . mc-vm-verify-signature)
 	     (snarf . mc-vm-snarf-keys))
+    (vm-virtual-mode (decrypt . mc-vm-decrypt-message)
+		     (verify . mc-vm-verify-signature)
+		     (snarf . mc-vm-snarf-keys))
     (vm-summary-mode (decrypt . mc-vm-decrypt-message)
 		     (verify . mc-vm-verify-signature)
 		     (snarf . mc-vm-snarf-keys))
     (mh-folder-mode (decrypt . mc-mh-decrypt-message)
 		    (verify . mc-mh-verify-signature)
 		    (snarf . mc-mh-snarf-keys))
-    (gnus-summary-mode (decrypt . mc-gnus-summary-decrypt-message)
-		       (verify . mc-gnus-summary-verify-signature)
-		       (snarf . mc-gnus-summary-snarf-keys))
+    ;; September Gnus (5.2) has a new message editing mode
+    (message-mode (encrypt . mc-encrypt-message)
+		  (sign . mc-sign-message))
+    (gnus-summary-mode (decrypt . mc-gnus-decrypt-message)
+		       (verify . mc-gnus-verify-signature)
+		       (snarf . mc-gnus-snarf-keys))
+    (gnus-article-mode (decrypt . mc-gnus-decrypt-message)
+		       (verify . mc-gnus-verify-signature)
+		       (snarf . mc-gnus-snarf-keys))
     (mail-mode (encrypt . mc-encrypt-message)
 	       (sign . mc-sign-message))
     (vm-mail-mode (encrypt . mc-encrypt-message)
@@ -313,20 +326,76 @@ stripping initial and trailing whitespace."
       (store-match-data data))))
 
 ;;; FIXME - Function never called?
-(defun mc-temp-display (beg end &optional name)
-  (let (tmp)
-    (if (not name)
-	(setq name mc-buffer-name))
-    (if (string-match name "*ERROR*")
-	(progn
-	  (message "mailcrypt: An error occured!  See *ERROR* buffer.")
-	  (beep)))
-    (setq tmp (buffer-substring beg end))
-    (delete-region beg end)
-    (save-excursion
-      (save-window-excursion
-	(with-output-to-temp-buffer name
-	  (princ tmp))))))
+;(defun mc-temp-display (beg end &optional name)
+;  (let (tmp)
+;    (if (not name)
+;	(setq name mc-buffer-name))
+;    (if (string-match name "*ERROR*")
+;	(progn
+;	  (message "mailcrypt: An error occured!  See *ERROR* buffer.")
+;	  (beep)))
+;    (setq tmp (buffer-substring beg end))
+;    (delete-region beg end)
+;    (save-excursion
+;      (save-window-excursion
+;	(with-output-to-temp-buffer name
+;	  (princ tmp))))))
+
+;; In case I ever decide to do this right.
+(defconst mc-field-name-regexp "^\\(.+\\)")
+(defconst mc-field-body-regexp "\\(.*\\(\n[ \t].*\\)*\n\\)")
+
+(defun mc-get-fields (&optional matching bounds nuke)
+  "Get all header fields within BOUNDS.  Return as an
+alist ((FIELD-NAME . FIELD-BODY) (FIELD-NAME . FIELD-BODY) ...).
+
+Argument MATCHING, if present, is a regexp which each FIELD-NAME
+must match exactly.  Matching is case-insensitive.
+
+Optional arg NUKE, if non-nil, means eliminate all fields returned."
+  (save-excursion
+    (save-restriction
+      (let ((case-fold-search t)
+	    (header-field-regexp
+	     (concat mc-field-name-regexp ":" mc-field-body-regexp))
+	    ret name body field-start field-end)
+	;; Ensure exact match
+	(if matching
+	    (setq matching (concat "^\\(" matching "\\)$")))
+
+	(if bounds
+	    (narrow-to-region (car bounds) (cdr bounds)))
+
+	(goto-char (point-max))
+ 
+	(while (re-search-backward header-field-regexp nil 'move)
+	  (setq field-start (match-beginning 0))
+	  (setq field-end (match-end 0))
+	  (setq name (buffer-substring-no-properties
+		      (match-beginning 1) (match-end 1)))
+	  (setq body (buffer-substring-no-properties
+		      (match-beginning 2) (match-end 2)))
+	  (if (or (null matching) (string-match matching name))
+	      (progn
+		(setq ret (cons (cons name body) ret))
+		(if nuke
+		    (delete-region field-start field-end)))))
+	ret))))
+
+(defsubst mc-strip-address (addr)
+  "Strip everything from ADDR except the basic Email address."
+  (car (cdr (mail-extract-address-components addr))))
+
+(defun mc-strip-addresses (addr-list)
+  "Strip everything from the addresses in ADDR-LIST except the basic
+Email address.  ADDR-LIST may be a single string or a list of strings."
+  (if (not (listp addr-list)) (setq addr-list (list addr-list)))
+  (setq addr-list
+	(mapcar
+	 (function (lambda (s) (mc-split "\\([ \t\n]*,[ \t\n]*\\)" s)))
+	 addr-list))
+  (setq addr-list (apply 'append addr-list))
+  (mapconcat 'mc-strip-address addr-list ", "))
 
 (defun mc-display-buffer (buffer)
   "Like display-buffer, but always display top of the buffer."
@@ -344,7 +413,8 @@ stripping initial and trailing whitespace."
 		(set-buffer buffer)
 		(goto-char (point-min))
 		(if (re-search-forward msg nil t)
-		    (buffer-substring (match-beginning 0) (match-end 0))
+		    (buffer-substring-no-properties
+		     (match-beginning 0) (match-end 0))
 		  (setq retval nil)
 		  default))))
     (if msg (message "%s" msg))
@@ -359,22 +429,24 @@ stripping initial and trailing whitespace."
 	  (setq mybuf (or buffer (generate-new-buffer " *mailcrypt temp")))
 	  (set-buffer mybuf)
 	  (erase-buffer)
+	  (set-buffer obuf)
 	  (buffer-disable-undo mybuf)
 	  (setq proc
 		(apply 'start-process "*PGP*" mybuf program args))
 	  (if passwd
 	      (progn
 		(process-send-string proc (concat passwd "\n"))
-		(or mc-passwd-timeout (mc-deactivate-passwd))))
-	  (set-buffer obuf)
+		(or mc-passwd-timeout (mc-deactivate-passwd t))))
 	  (process-send-region proc beg end)
 	  (process-send-eof proc)
 	  (while (eq 'run (process-status proc))
 	    (accept-process-output proc 5))
 	  (setq result (process-exit-status proc))
+	  ;; Hack to force a status_notify() in Emacs 19.29
+	  (delete-process proc)
 	  (set-buffer mybuf)
 	  (goto-char (point-max))
-	  (if (re-search-backward "\nProcess .* finished\n\\'" nil t)
+	  (if (re-search-backward "\nProcess \\*PGP.*\n\\'" nil t)
 	      (delete-region (match-beginning 0) (match-end 0)))
 	  (goto-char (point-min))
 	  ;; CRNL -> NL
@@ -433,6 +505,7 @@ Return the passphrase.  If PROMPT is nil, only return value if cached."
       (setq mc-passwd-cache (cons (cons id passwd) mc-passwd-cache)))
     passwd))
 
+;;;###autoload
 (defun mc-deactivate-passwd (&optional inhibit-message)
   "*Deactivate the passphrase cache."
   (interactive)
@@ -446,6 +519,7 @@ Return the passphrase.  If PROMPT is nil, only return value if cached."
       (setcdr cell nil)))
    mc-passwd-cache)
   (or inhibit-message
+      (not (interactive-p))
       (message "Passphrase%s deactivated"
 	       (if (> (length mc-passwd-cache) 1) "s" ""))))
 
