@@ -138,7 +138,7 @@
   (define-key hide-ifdef-mode-submap "\C-s" 'show-ifdef-block)
 
   (define-key hide-ifdef-mode-submap "\C-q" 'hide-ifdef-toggle-read-only)
-  (let ((where (where-is-internal 'toggle-read-only '(keymap) t)))
+  (let ((where (where-is-internal 'toggle-read-only nil t)))
     (if where
 	(define-key hide-ifdef-mode-submap
 	  where
@@ -169,7 +169,7 @@
 ;                minor-mode-alist)))
 ;; XEmacs: do it right.
 ;;;###autoload
-(add-minor-mode 'hide-ifdef-mode " Ifdef")
+(add-minor-mode 'hide-ifdef-mode " Ifdef" 'hide-ifdef-mode-map)
 
 ;; fix c-mode syntax table so we can recognize whole symbols.
 (defvar hide-ifdef-syntax-table
