@@ -252,7 +252,13 @@ Returns list of symbols and documentation found."
 			(if (setq doc (face-doc-string symbol))
 			    (substring doc 0
 				       (string-match "\n" doc))
-			  "(not documented)"))))
+			  "(not documented)"))
+		    (when (get symbol 'custom-group)
+		      (if (setq doc (documentation-property
+				     symbol 'group-documentation t))
+			  (substring doc 0
+				     (string-match "\n" doc))
+			"(not documented)"))))
 	 (setq p (cdr p)))))
    nil))
 
@@ -459,8 +465,10 @@ Will return nil instead."
       (setq function (cdr function)))
   ;; XEmacs change from: (setq function (if (byte-code-function-p function)
   (setq function (if (compiled-function-p function)
-		     (if (> (length function) 4)
-			 (aref function 4))
+		     (if (fboundp 'compiled-function-doc-string)
+			 (compiled-function-doc-string function)
+		       (if (> (length function) 4)
+			   (aref function 4)))
 		   (if (eq (car-safe function) 'autoload)
 		       (nth 2 function)
 		     (if (eq (car-safe function) 'lambda)
@@ -576,6 +584,7 @@ found."
 				 "User Option" do-keys)
 	    (apropos-print-doc 'describe-variable 2
 			       "Variable" do-keys))
+	  (apropos-print-doc 'customize-other-window 6 "Group" do-keys)
 	  (apropos-print-doc 'customize-face-other-window 5 "Face" do-keys)
 	  (apropos-print-doc 'widget-browse-other-window 4 "Widget" do-keys)
 	  (apropos-print-doc 'apropos-describe-plist 3
