@@ -34,8 +34,10 @@
 ;;; Code:
 
 (defgroup compilation nil
-  "Compilation buffer processing"
-  :group 'programming)
+  "Run compiler as inferior of Emacs, parse error messages."
+  :group 'programming
+  :group 'tools
+  :group 'processes)
 
 
 ;;;###autoload
@@ -125,32 +127,11 @@ buffer is not visible in its window."
 
 (defvar compilation-num-errors-found)
 
-(defcustom compilation-error-regexp-systems-list 'all
-  "*This is either the symbol `all', or a list of systems for which
-compilation error regexps should be included in
-`compilation-error-regexp-alist'.  
-
-The list of known systems is:
-  gnu:      but of course
-  lcc:      Lucid compilers
-  ada:      Ada compilers
-  of:       Using tool that says line xx of foo.c
-  comma:    Using tool that says \"foo.c\", line 12
-  4bsd:     Using 4bsd
-  msft:     Using microsoft
-  borland:  Using Borland
-  mips:     Using Mips
-  sgi:      Using SGI
-  cray:     Using Cray
-  ibm:      IBM C compilers
-  aix:      the operating system
-  ultrix:   the operating system
-
-See also the variable `compilation-error-regexp-alist-alist'."
-  :type '(choice (const all) (repeat symbol))
-  :group 'compilation)
-
 (defun compilation-build-compilation-error-regexp-alist ()
+  "Set the regular expressions used for parsing compiler
+errors based on the compilers listed in the variable
+`compilation-error-regexp-systems-list'.  Updates the
+variable `compilation-error-regexp-alist'."
   (interactive)
   (setq compilation-error-regexp-alist
         (apply 'append
@@ -349,6 +330,42 @@ Note that this now gets set by the function
 `compilation-build-compilation-error-regexp-alist' using the 
 value of the variable `compilation-error-regexp-alist-alist'")
 
+(defcustom compilation-error-regexp-systems-list 'all
+  "*This is either the symbol `all', or a list of systems for which
+compilation error regexps should be included in
+`compilation-error-regexp-alist'.  You must run the function
+`compilation-build-compilation-error-regexp-alist' after changing
+the value of this variable for the change to take effect.
+
+The list of known systems is:
+  gnu:      but of course
+  lcc:      Lucid compilers
+  ada:      Ada compilers
+  of:       Using tool that says line xx of foo.c
+  comma:    Using tool that says \"foo.c\", line 12
+  4bsd:     Using 4bsd
+  msft:     Using microsoft
+  borland:  Using Borland
+  mips:     Using Mips
+  sgi:      Using SGI
+  cray:     Using Cray
+  ibm:      IBM C compilers
+  aix:      the operating system
+  ultrix:   the operating system
+
+See also the variable `compilation-error-regexp-alist-alist'."
+  :type '(choice (const all)
+		 (set :menu-tag "Pick"
+		      (const gnu) (const lcc) (const ada)
+		      (const of) (const comma) (const 4bsd)
+		      (const msft) (const borland) (const mips)
+		      (const sgi) (const cray) (const ibm)
+		      (const aix) (const ultrix)))
+  :set (lambda (symbol value)
+         (set-default symbol value)
+         (compilation-build-compilation-error-regexp-alist))
+  :group 'compilation)
+
 (compilation-build-compilation-error-regexp-alist)
 
 (defcustom compilation-read-command t
@@ -641,14 +658,18 @@ Returns the compilation buffer created."
 	      (setq compilation-in-progress 
 		    (cons proc compilation-in-progress)))
 	  ;; No asynchronous processes available.
-	  (message "Executing `%s'..." command)
+	  (display-message
+	   'progress
+	   (format "Executing `%s'..." command))
 	  ; FSF
 	  ; (setq mode-line-process ":run")
 	  ; (force-mode-line-update)
 	  (sit-for 0)			;; Force redisplay
 	  (let ((status (call-process shell-file-name nil outbuf nil "-c"
 				      command))))
-	  (message "Executing `%s'...done" command)))
+	  (display-message
+	   'progress
+	   (format "Executing `%s'...done" command))))
        (set-buffer buffer-save)))
 
     ;; Make it so the next C-x ` will use this buffer.
@@ -766,6 +787,7 @@ or *grep* buffers."
 (defun grep-menu-filter (menu)
   (compile-menu-filter-1 menu grep-history "Grep History" 'grep))
 
+;;;###autoload
 (defun compilation-mode (&optional name-of-mode)
   "Major mode for compilation log buffers.
 \\<compilation-mode-map>To visit the source for a line-numbered error,
@@ -947,9 +969,11 @@ See `compilation-mode'.
 			       (pos-visible-in-window-p (point-max) window))
 			  nil		; assume that the user will see it...
 			(ding t 'ready)
-			(message "Compilation process completed%s."
+			(display-message
+			 'progress
+			 (format "Compilation process completed%s."
 				 (or estatus " successfully")
-				 ))
+				 )))
 		      ;; Since the buffer and mode line will show that the
 		      ;; process is dead, we can delete it now.  Otherwise it
 		      ;; will stay around until M-x list-processes.
@@ -1629,7 +1653,7 @@ Selects a window with point at SOURCE, with another window displaying ERROR."
   "Parse the current buffer as grep, cc or lint error messages.
 See variable `compilation-parse-errors-function' for the interface it uses."
   (setq compilation-error-list nil)
-  (message "Parsing error messages...")
+  (display-message 'progress "Parsing error messages...")
   (let (;;text-buffer -- unused
 	orig orig-expanded parent-expanded
 	regexp enter-group leave-group error-group
@@ -1876,11 +1900,13 @@ See variable `compilation-parse-errors-function' for the interface it uses."
 	     (error "compilation-parse-errors: known groups didn't match!")))
 
       (when (= (% compilation-num-errors-found message-freq) 0)
-        (message "Parsing error messages...%d (%.0f%% of buffer)"
+        (display-message
+	 'progress
+	 (format "Parsing error messages...%d (%.0f%% of buffer)"
                  compilation-num-errors-found
                  ;; Use floating-point because (* 100 (point)) frequently
                  ;; exceeds the range of Emacs Lisp integers.
-                 (/ (* 100.0 (point)) (point-max))))
+                 (/ (* 100.0 (point)) (point-max)))))
 
 ;;; This is broken - it foils the logic above which is supposed to ensure
 ;;; that all errors for the current file are found before we quit. 
@@ -1893,7 +1919,7 @@ See variable `compilation-parse-errors-function' for the interface it uses."
 				    ;; We have searched the whole buffer.
 				    (point-max))))
   (setq compilation-error-list (nreverse compilation-error-list))
-  (message "Parsing error messages...done"))
+  (display-message 'progress "Parsing error messages...done"))
 
 ;; If directory DIR is a subdir of ORIG or of ORIG's parent,
 ;; return a relative name for it starting from ORIG or its parent.
