@@ -1,6 +1,6 @@
 ;;; func-menu.el         --- Jump to a function within a buffer.
 ;;;
-;;; David Hughes <ukchugd@ukpmr.cs.philips.nl>
+;;; David Hughes <d.hughes@videonetworks.com>
 ;;; Last modified: David Hughes 13th January 1997
 ;;; Version: 2.45
 ;;; Keywords: tools, c, lisp
@@ -65,7 +65,7 @@
 ;;; Peter Pezaris <pez@dwwc.com>
 ;;;
 ;;; Made menu placement more flexible
-;;; Bob Weiner <weiner@infodock.com>
+;;; Bob Weiner <weiner@altrasoft.com>
 ;;;
 ;;; Fortran90 regexp
 ;;; John Turner <turner@xdiv.lanl.gov>
@@ -74,7 +74,7 @@
 ;;; Andy Piper <andyp@parallax.co.uk>
 ;;;
 ;;; Java support
-;;; Bob Weiner <weiner@infodock.com>
+;;; Bob Weiner <weiner@altrasoft.com>
 ;;; Heddy Boubaker <boubaker@dgac.fr>
 ;;;
 ;;; Patch for fume-rescan-buffer{-trigger}
@@ -173,7 +173,7 @@
 ;;; Philippe Queinnec <queinnec@cenatls.cena.dgac.fr>
 ;;;
 ;;; Assembly support
-;;; Bob Weiner <weiner@infodock.com>
+;;; Bob Weiner <weiner@altrasoft.com>
 ;;;
 ;;; Removal of cl dependencies
 ;;; Russell Ritchie <russell@gssec.bt.co.uk>
@@ -206,7 +206,6 @@
 (eval-when-compile
   (byte-compiler-options
     (optimize t)
-    (new-bytecodes t)
     (warnings (- free-vars unresolved))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -218,14 +217,17 @@
 (defgroup fume nil
   "Jump to a function within a buffer."
   :tag "Func Menu"
-  :group 'tools)
+  :group 'tools
+  :group 'c
+  :group 'lisp)
 
-(defconst fume-developer "David Hughes <ukchugd@ukpmr.cs.philips.nl>")
+(defconst fume-developer "David Hughes <d.hughes@videonetworks.com>")
 
 (defun fume-about ()
   (interactive)
   (sit-for 0)
-  (message "Func-Menu version %s,  1996 %s" fume-version fume-developer))
+  (display-message 'no-log
+     (format "Func-Menu version %s,  1996 %s" fume-version fume-developer)))
 
 (defconst fume-running-xemacs (string-match "XEmacs\\|Lucid" emacs-version))
 
@@ -421,7 +423,7 @@ Note, this is a buffer-local variable.")
   :type 'string
   :group 'fume)
 
-;;; Bob Weiner <weiner@infodock.com>
+;;; Bob Weiner <weiner@altrasoft.com>
 (defvar fume-menu-path nil
   "Menubar menu under which the function menu should be installed.
 Nil means install it on the menubar itself.  Otherwise, it should be a list
@@ -587,9 +589,10 @@ by func-menu, then do something along the lines of the following:
 ;;; C
 ;;;
 ;;; Danny Bar-Dov <danny@acet02.amil.co.il>
+;;; Bob Weiner    <weiner@altrasoft.com> added #define macro support.
 (defvar fume-function-name-regexp-c
   (concat
-   "^[a-zA-Z0-9_]+\\s-?"                ; type specs; there can be no
+   "^\\([a-zA-Z0-9]+\\|#define\\)\\s-?" ; type specs; there can be no
    "\\([a-zA-Z0-9_*]+\\s-+\\)?"         ; more than 3 tokens, right?
    "\\([a-zA-Z0-9_*]+\\s-+\\)?"
    "\\([*&]+\\s-*\\)?"                  ; pointer
@@ -605,16 +608,18 @@ by func-menu, then do something along the lines of the following:
 ;;; Mike Battaglia  <mbattagl@spd.dsccc.com>
 ;;; Oliver Schittko <schittko@fokus.gmd.de>
 ;;; Tom Murray      <tmurray@hpindck.cup.hp.com>
+;;; Bob Weiner      <weiner@altrasoft.com> added #define macro support.
 (defvar fume-function-name-regexp-c++
   (cons
    (concat
-    "^\\(template\\s +<[^>]+>\\s +\\)?"          ; template formals
+    "^\\(#define\\s-+\\|"
+    "\\(template\\s-+<[^>]+>\\s-+\\)?"           ; template formals
     "\\([a-zA-Z0-9_*&<,>:]+\\s-+\\)?"            ; type specs; there can be no
     "\\([a-zA-Z0-9_*&<,>\"]+\\s-+\\)?"           ; more than 3 tokens, right?
-    "\\([a-zA-Z0-9_*&<,>]+\\s-+\\)?"
+    "\\([a-zA-Z0-9_*&<,>]+\\s-+\\)?\\)"
     "\\(\\([a-zA-Z0-9_&~:<,>*]\\|\\(\\s +::\\s +\\)\\)+\\)"
     "\\(o?perator\\s *.[^(]*\\)?\\(\\s-\\|\n\\)*(" ; name
-    ) 5)
+    ) 6)
   "Expression to get C++ function names")
 
 ;;; FORTRAN
@@ -628,7 +633,7 @@ by func-menu, then do something along the lines of the following:
    ;; >= six spaces
    "^      \\s-*"
    ;; type specs
-   "+[a-zA-Z0-9*]*\\s-*"
+   "[a-zA-Z0-9*]*\\s-*"
    ;; continuation lines
    "\\(\n     [^ 0]\\s-*\\)*"
    ;; function or subroutine
@@ -642,7 +647,7 @@ by func-menu, then do something along the lines of the following:
 (defvar fume-function-name-regexp-fortran90
   (concat
    ;; type specs
-   "+[a-zA-Z0-9*]*\\s-*"
+   "[a-zA-Z0-9*]*\\s-*"
    ;; function or subroutine
    "\\(entry\\|ENTRY\\|function\\|FUNCTION\\|module\\|MODULE\\|subroutine\\|SUBROUTINE\\)\\s-*"
    )
@@ -833,7 +838,7 @@ by func-menu, then do something along the lines of the following:
 (defvar fume-function-name-regexp-make
   "^\\(\\(\\$\\s(\\)?\\(\\w\\|\\.\\)+\\(:sh\\)?\\(\\s)\\)?\\)\\s *\\(::?\\|\\+?=\\)"
   "Expression to get makefile target names")
-(add-hook 'makefile-mode-hook 'fume-add-menubar-entry)
+;;(add-hook 'makefile-mode-hook 'fume-add-menubar-entry)
 
 ;;; Directory Listings
 ;;;
@@ -1092,7 +1097,7 @@ by func-menu, then do something along the lines of the following:
 ;;; <jrm@odi.com>
 ;;; <ajp@eng.cam.ac.uk>
 ;;; <schittko@fokus.gmd.de>
-;;; <ukchugd@ukpmr.cs.philips.nl> - speedup, David Hughes 24th November 1996
+;;; <d.hughes@videonetworks.com> - speedup, David Hughes 24th November 1996
 ;;;
 (defun fume-match-find-next-function-name (buffer)
   ;; General next function name in BUFFER finder using match.
@@ -1125,7 +1130,7 @@ by func-menu, then do something along the lines of the following:
   (fume-find-next-sexp buffer))
 
 ;;; Specialised routine to find the next Java function
-;;; Bob Weiner <weiner@infodock.com>
+;;; Bob Weiner <weiner@altrasoft.com>
 ;;; Heddy Boubaker <boubaker@dgac.fr>
 ;;;
 (defun fume-find-next-java-function-name (buffer)
@@ -1140,7 +1145,7 @@ by func-menu, then do something along the lines of the following:
                  (not (fume-cc-inside-comment)))
             ;; This is a method definition and we're not in a comment
             (let ((str (buffer-substring beg end)))
-              ;; Bob Weiner <weiner@infodock.com> added exact match
+              ;; Bob Weiner <weiner@altrasoft.com> added exact match
               ;; delimiters so function names that happen to contain
               ;; any of these terms are not eliminated.  The old version
               ;; would ignore "notify()" since it contained "if".
@@ -1429,7 +1434,7 @@ by func-menu, then do something along the lines of the following:
 
 
 ;;; Assembly
-;;; Bob Weiner <weiner@infodock.com>
+;;; Bob Weiner <weiner@altrasoft.com>
 ;;;
 (defun fume-find-next-asm-function-name (buffer)
   "Searches for the next assembler function in BUFFER."
@@ -1883,9 +1888,9 @@ If optional arg POPMENU is non-nil, brings up the function-menu."
     (save-excursion
       (goto-char (point-min))
       (cond (fume-scanning-message
-             (message fume-scanning-message 0))
+             (display-message 'progress (format fume-scanning-message 0)))
             (fume-rescanning-message
-             (message fume-rescanning-message)))
+             (display-message 'progress fume-rescanning-message)))
       (while (setq fnam
                    (condition-case ()
                        (funcall find buffer-to-scan)
@@ -1904,11 +1909,11 @@ If optional arg POPMENU is non-nil, brings up the function-menu."
                (if fume-found-function-hook
                    (save-excursion (run-hooks 'fume-found-function-hook)))))
         (if fume-scanning-message
-            (message fume-scanning-message (fume-relative-position))))
+            (display-message 'progress (format fume-scanning-message (fume-relative-position)))))
       (cond (fume-scanning-message
-             (message "%s done" (format fume-scanning-message 100)))
+             (display-message 'progress (format "%s done" (format fume-scanning-message 100))))
             (fume-rescanning-message
-             (message "%s done" fume-rescanning-message)))
+             (display-message 'progress (format "%s done" fume-rescanning-message))))
       ;; make a copy of flst sorted by position in buffer
       (setq fume-modeline-funclist
             (nreverse
@@ -1943,7 +1948,7 @@ If optional arg POPMENU is non-nil, brings up the function-menu."
 
     (save-excursion
       (goto-char pos)
-      (or (looking-at match-fn)
+      (or (looking-at (regexp-quote match-fn))
           (let ((fume-scanning-message nil))
             (fume-rescan-buffer)
             (setq pos (cdr-safe (assoc fn fume-funclist))))))
@@ -2076,7 +2081,7 @@ the basic menu of functions."
                        fume-menubar-menu-location))
 
                      ((and fume-not-tty ; trap tty segmentation faults...
-                           (not (popup-menu-up-p)))
+                           (not (popup-up-p)))
                       (or (fume-update-menubar-entry)
                           (setq function-menu
                                 (cons
@@ -2114,7 +2119,7 @@ the basic menu of functions."
 (defvar fume-list-trampled-buffer nil)
 
 ;;; Espen Skoglund <espensk@stud.cs.uit.no>
-;;; David Hughes <ukchugd@ukpmr.cs.philips.nl>
+;;; David Hughes <d.hughes@videonetworks.com>
 ;;;
 (defun fume-prompt-function-goto (&optional other-window-p)
   "Goto function prompted for in minibuffer (with completion).
@@ -2192,14 +2197,15 @@ With prefix arg, jumps to function in a different window."
   (interactive)
   (fume-about)
   (sit-for 1)
-  (message "SPC=%s, p=%s, n=%s, o=%s, G=%s, RET=%s,    q=%s"
-           "this"
-           "previous"
-           "next"
-           "other win"
-           "one win"
-           "this win"
-           "quit"))
+  (display-message 'prompt
+		   (format "SPC=%s, p=%s, n=%s, o=%s, G=%s, RET=%s,    q=%s"
+			   "this"
+			   "previous"
+			   "next"
+			   "other win"
+			   "one win"
+			   "this win"
+			   "quit")))
 
 (defun fume-list-functions-quit ()
   (interactive)
