@@ -1,33 +1,34 @@
-;;; icomplete.el --- minibuffer completion with incremental feedback
+;;;_. icomplete.el - minibuffer completion incremental feedback
 
-;;; Copyright (C) 1992, 1993, 1994 Free Software Foundation, Inc.
+;; Copyright (C) 1992, 1993, 1994 Free Software Foundation, Inc.
 
-;;; Author: Ken Manheimer <klm@nist.gov>
-;;; Maintainer: Ken Manheimer <klm@nist.gov>
-;;; Version: $Id: icomplete.el,v 4.4 1995/08/03 21:42:15 klm Exp klm $
-;;; Created: Mar 1993 klm@nist.gov - first release to usenet
-;;; Keywords: help, abbrev
+;; Author: Ken Manheimer <klm@python.org>
+;; Maintainer: Ken Manheimer <klm@python.org>
+;; Version: $Id: icomplete.el,v 4.10 1997/05/30 22:01:13 klm Exp $
+;; Created: Mar 1993 klm@nist.gov - first release to usenet
+;; Keywords: help, abbrev
 
-;;; Hacked for XEmacs: David Hughes 7th September 1995
+;; This file is part of GNU Emacs.
 
-;; This file is part of XEmacs.
-
-;; XEmacs is free software; you can redistribute it and/or modify it
-;; under the terms of the GNU General Public License as published by
+;; GNU Emacs is free software; you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
-;; XEmacs is distributed in the hope that it will be useful, but
-;; WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;; General Public License for more details.
+;; GNU Emacs is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with XEmacs; see the file COPYING.  If not, write to the Free
-;; Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-;; 02111-1307, USA.
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
 
-;;; Synched up with: FSF 19.34.
+;; This file is also part of GNU XEmacs.
+;; Hacked for GNU XEmacs: David Hughes 7th September 1995.
+;; Icomplete keybindings display originally by Steve Bauer, with
+;; some integration and refinement by Ken Manheimer.
 
 ;;; Commentary:
 
@@ -43,26 +44,21 @@
 ;; customize icomplete setup for interoperation with other
 ;; minibuffer-oriented packages.
 
-;; To activate icomplete mode, simply load the package.  You can
-;; subsequently deactivate it by invoking the function icomplete-mode
-;; with a negative prefix-arg (C-U -1 ESC-x icomplete-mode).  Also,
-;; you can prevent activation of the mode during package load by
-;; first setting the variable `icomplete-mode' to nil.  Icompletion
-;; can be enabled any time after the package is loaded by invoking
-;; icomplete-mode without a prefix arg.
-
-;; This version of icomplete runs on Emacs 19.18 and later.  (It
-;; depends on the incorporation of minibuffer-setup-hook.)  The elisp
-;; archives, ftp://archive.cis.ohio-state.edu/pub/gnu/emacs/elisp-archive,
-;; probably still has a version that works in GNU Emacs v18.
+;; To activate icomplete mode, load the package and use the
+;; `icomplete-mode' function.  You can subsequently deactivate it by
+;; invoking the function icomplete-mode with a negative prefix-arg
+;; (C-U -1 ESC-x icomplete-mode).  Also, you can prevent activation of
+;; the mode during package load by first setting the variable
+;; `icomplete-mode' to nil.  Icompletion can be enabled any time after
+;; the package is loaded by invoking icomplete-mode without a prefix
+;; arg.
 
 ;; Thanks to everyone for their suggestions for refinements of this
 ;; package.  I particularly have to credit Michael Cook, who
 ;; implemented an incremental completion style in his 'iswitch'
 ;; functions that served as a model for icomplete.  Some other
 ;; contributors: Noah Freidman (restructuring as minor mode), Colin
-;; Rafferty (lemacs reconciliation), Lars Lindberg, RMS, and
-;; others.
+;; Rafferty (lemacs reconciliation), Lars Lindberg, RMS, and others.
 
 ;; klm.
 
@@ -71,11 +67,42 @@
 ;;;_* Provide
 (provide 'icomplete)
 
+(defgroup icomplete nil
+  "Minibuffer completion incremental feedback."
+  :group 'minibuffer)
+
+
+(defcustom icomplete-mode nil
+  "*Non-nil activates incremental minibuffer completion."
+  :type 'boolean
+  :set (lambda (symbol value)
+	 (icomplete-mode (if value 1 -1)))
+  :initialize 'custom-initialize-default
+  :require 'icomplete
+  :group 'icomplete)
+
 ;;;_* User Customization variables
+(defcustom icomplete-prospects-length 80
+  "*Length of string displaying the prospects."
+  :type 'integer
+  :group 'icomplete)
+(defcustom icomplete-compute-delay .3
+  "*Completions-computation stall, used only with large-number
+completions - see `icomplete-delay-completions-threshold'."
+  :type 'number
+  :group 'icomplete)
+(defcustom icomplete-delay-completions-threshold 400
+  "*Pending-completions number over which to apply icomplete-compute-delay."
+  :type 'integer
+  :group 'icomplete)
+(defcustom icomplete-max-delay-chars 3
+  "*Maximum number of initial chars to apply icomplete compute delay."
+  :type 'integer
+  :group 'icomplete)
 
 ;;;_* Initialization
 ;;;_  = icomplete-minibuffer-setup-hook
-(defvar icomplete-minibuffer-setup-hook nil
+(defcustom icomplete-minibuffer-setup-hook nil
   "*Icomplete-specific customization of minibuffer setup.
 
 This hook is run during minibuffer setup iff icomplete will be active.
@@ -89,16 +116,14 @@ with other packages.  For instance:
 	       \(setq resize-minibuffer-window-max-height 3))))
 
 will constrain rsz-mini to a maximum minibuffer height of 3 lines when
-icompletion is occurring.")
-
-(if (string-match "XEmacs\\|Lucid" emacs-version)
-    (add-hook 'icomplete-minibuffer-setup-hook 'icomplete-exhibit))
+icompletion is occurring."
+  :type 'hook
+  :group 'icomplete)
 
 ;;;_ + Internal Variables
 ;;;_  = icomplete-mode
-(defvar icomplete-mode t
-  "Non-nil enables incremental minibuffer completion, once
-`\\[icomplete-mode]' function has set things up.")
+;(defvar icomplete-mode t
+;  "*Nil inhibits activated incremental minibuffer completion.")
 ;;;_  = icomplete-eoinput 1
 (defvar icomplete-eoinput 1
   "Point where minibuffer input ends and completion info begins.")
@@ -120,35 +145,33 @@ Use `icomplete-mode' function to set it up properly for incremental
 minibuffer completion.")
 (add-hook 'icomplete-post-command-hook 'icomplete-exhibit)
 
-;; XEmacs addition
 (defvar icomplete-show-key-bindings t
-  "When non-nil show key bindings as well as completion when matching
-a command.")
+  "When non-nil, show key bindings as well as completion for sole matches.")
 
-;; XEmacs addition
 (defun icomplete-get-keys (func-name)
-  "Return the keys `func-name' is bound to as a string."
-  (when (commandp func-name)
+  "Return strings naming keys bound to `func-name', or nil if none.
+Examines the prior, not current, buffer, presuming that current buffer
+is minibuffer."
+  (if (commandp func-name)
     (save-excursion
-      ;; Depends on dynamic scope from read-from-minibuffer :-(
       (let* ((sym (intern func-name))
-	     (buf (set-buffer (window-buffer owindow)))
-	     (keys (where-is-internal sym (current-local-map buf))))
-	(concat "<"
-		(if keys
+	     (buf (other-buffer))
+	     (map (save-excursion (set-buffer buf) (current-local-map)))
+	     (keys (where-is-internal sym map)))
+	(if keys
+	    (concat "<"
 		    (mapconcat 'key-description
 			       (sort keys
 				     #'(lambda (x y)
 					 (< (length x) (length y))))
 			       ", ")
-		  "Unbound")
-		">")))))
+		    ">"))))))
 
 ;;;_ > icomplete-mode (&optional prefix)
 ;;;###autoload
 (defun icomplete-mode (&optional prefix)
-  "Activate incremental minibuffer completion for this emacs session,
-or deactivate with negative prefix arg."
+  "Activate incremental minibuffer completion for this emacs session.
+Deactivates with negative universal argument."
   (interactive "p")
   (or prefix (setq prefix 0))
   (cond ((>= prefix 0)
@@ -160,7 +183,6 @@ or deactivate with negative prefix arg."
 
 ;;;_ > icomplete-simple-completing-p ()
 (defun icomplete-simple-completing-p ()
-
   "Non-nil if current window is minibuffer that's doing simple completion.
 
 Conditions are:
@@ -231,7 +253,22 @@ and `minibuffer-setup-hook'."
 	      (make-local-variable 'icomplete-eoinput))
 	  (setq icomplete-eoinput (point))
                                         ; Insert the match-status information:
-	  (if (> (point-max) 1)
+	  (if (and (> (point-max) 1)
+		   (or
+		    ;; Don't bother with delay after certain number of chars:
+		    (> (point-max) icomplete-max-delay-chars)
+		    ;; Don't delay if alternatives number is small enough:
+		    (if minibuffer-completion-table
+			(cond ((numberp minibuffer-completion-table)
+			       (< minibuffer-completion-table
+				  icomplete-delay-completions-threshold))
+			      ((sequencep minibuffer-completion-table)
+			       (< (length minibuffer-completion-table)
+				  icomplete-delay-completions-threshold))
+			      ))
+		    ;; Delay - give some grace time for next keystroke, before
+		    ;; embarking on computing completions:
+		    (sit-for icomplete-compute-delay)))
 	      (insert-string
 	       (icomplete-completions contents
 				      minibuffer-completion-table
@@ -256,70 +293,65 @@ one of \(), \[], or \{} pairs.  The choice of brackets is as follows:
 
 The displays for unambiguous matches have ` [Matched]' appended
 \(whether complete or not), or ` \[No matches]', if no eligible
-matches exist."
+matches exist.  \(Keybindings for uniquely matched commands are
+exhibited within the square braces.)"
+
+  ;; 'all-completions' doesn't like empty
+  ;; minibuffer-completion-table's (ie: (nil))
+  (if (and (listp candidates) (null (car candidates)))
+      (setq candidates nil))
 
   (let ((comps (all-completions name candidates predicate))
                                         ; "-determined" - only one candidate
         (open-bracket-determined (if require-match "(" "["))
-        (close-bracket-determined (if require-match ")" "]"))
-                                        ;"-prospects" - more than one candidate
-        (open-bracket-prospects "{")
-        (close-bracket-prospects "}")
-        )
-    (cond ((null comps) (format " %sNo matches%s"
-				open-bracket-determined
-				close-bracket-determined))
-	  ((null (cdr comps))           ;one match
-	   (concat (if (and (> (length (car comps))
-			       (length name)))
-		       (concat open-bracket-determined
-			       (substring (car comps) (length name))
-			       close-bracket-determined)
-		     "")
-		   " [Matched]"
-	   ;; XEmacs
-		   (if (and icomplete-show-key-bindings
-			    (commandp (intern-soft (car comps))))
-		       (icomplete-get-keys (car comps))
-		     "")
-		   ))
-	  (t                            ;multiple matches
-	   (let* ((most (try-completion name candidates predicate))
-		  (most-len (length most))
-		  most-is-exact
-		  (alternatives
-		   (apply
-		    (function concat)
-		    (cdr (apply
-			  (function nconc)
-			  (mapcar '(lambda (com)
-				     (if (= (length com) most-len)
-					 ;; Most is one exact match,
-					 ;; note that and leave out
-					 ;; for later indication:
-					 (progn
-					   (setq most-is-exact t)
-					   ())
-				       (list ","
-					     (substring com
-							most-len))))
-				  comps))))))
-	     (concat (and (> most-len (length name))
+        (close-bracket-determined (if require-match ")" "]")))
+    ;; `concat'/`mapconcat' is the slow part.  With the introduction of
+    ;; `icomplete-prospects-length', there is no need for `catch'/`throw'.
+    (if (null comps) (format " %sNo matches%s"
+			     open-bracket-determined
+			     close-bracket-determined)
+      (let* ((most-try (try-completion name (mapcar (function list) comps)))
+	     (most (if (stringp most-try) most-try (car comps)))
+	     (most-len (length most))
+	     (determ (and (> most-len (length name))
 			  (concat open-bracket-determined
 				  (substring most (length name))
-				  close-bracket-determined))
-		     open-bracket-prospects
-		     (if most-is-exact
-			 (concat "," alternatives)
-		       alternatives)
-		     close-bracket-prospects))))))
-
-;;;_ + Initialization
-;;; If user hasn't setq-default icomplete-mode to nil, then setup for
-;;; activation:
-(if icomplete-mode
-    (icomplete-mode))
+				  close-bracket-determined)))
+	     (open-bracket-prospects "{")
+	     (close-bracket-prospects "}")
+                                        ;"-prospects" - more than one candidate
+	     (prospects-len 0)
+	     prospects most-is-exact comp)
+	(if (eq most-try t)
+	    (setq prospects nil)
+	  (while (and comps (< prospects-len icomplete-prospects-length))
+	    (setq comp (substring (car comps) most-len)
+		  comps (cdr comps))
+	    (cond ((string-equal comp "") (setq most-is-exact t))
+		  ((member comp prospects))
+		  (t (setq prospects (cons comp prospects)
+			   prospects-len (+ (length comp) 1 prospects-len))))))
+	(if prospects
+	    (concat determ
+		    open-bracket-prospects
+		    (and most-is-exact ",")
+		    (mapconcat 'identity
+			       (sort prospects (function string-lessp))
+			       ",")
+		    (and comps ",...")
+		    close-bracket-prospects)
+	  (concat determ
+		  " [Matched"
+		  (let ((keys (and icomplete-show-key-bindings
+				   (commandp (intern-soft most))
+				   (icomplete-get-keys most))))
+		    (if keys
+			(concat "; " keys)
+		      ""))
+		  "]"))))))
 
+(if (string-match "XEmacs\\|Lucid" emacs-version)
+    (add-hook 'icomplete-minibuffer-setup-hook 'icomplete-exhibit))
 
 ;;;_* Local emacs vars.
 ;;;Local variables:
