@@ -1,14 +1,11 @@
 ;;; ispell.el --- spell checking using Ispell
 
-;; Copyright (C) 1994, 1995 Free Software Foundation, Inc.
+;; Copyright (C) 1994, 1995, 1997 Free Software Foundation, Inc.
 
 ;; Authors         : Ken Stevens <k.stevens@ieee.org>
-;; Last Modified On: Tue Jun 13 12:05:28 EDT 1995
-;; Update Revision : 2.37
 ;; Syntax          : emacs-lisp
-;; Status	    : Release with 3.1.12+ ispell.
-;; Version	    : International Ispell Version 3.1 by Geoff Kuenning.
-;; Bug Reports	    : ispell-el-bugs@itcorp.com
+;; Version	   : International Ispell Version 3.1 by Geoff Kuenning.
+;; Bug Reports	   : ispell-el-bugs@itcorp.com
 
 ;; Note: version numbers and time stamp are not updated
 ;;   when this file is edited for release with GNU emacs.
@@ -130,7 +127,7 @@
 ;;
 ;; HISTORY
 ;;
-;; Revision 2.38  1996/5/30    ethanb@phys.washington.edu
+;; Revision 2.38  1996/5/30	ethanb@phys.washington.edu
 ;; Update ispell-message for gnus 5 (news-inews-hook => message-send-hook;
 ;; different header for quoted message).
 ;;
@@ -212,7 +209,7 @@
 ;; lookup-words rehacked to use call-process (Jamie).
 ;; ispell-complete-word rehacked to be compatible with the rest of the
 ;; system for word searching and to include multiple wildcards,
-;; and it's own dictionary.
+;; and its own dictionary.
 ;; query-replace capability added.  New options 'X', 'R', and 'A'.
 ;; buffer-local modes for dictionary, word-spelling, and formatter-parsing.
 ;; Many random bugs, like commented comments being skipped, fix to
@@ -409,7 +406,7 @@ Some machines (like the NeXT) don't support \"-i\"")
   "Name of the look command for search processes.
 This must be an absolute file name.")
 
-(defcustom ispell-look-p `(file-exists-p ispell-look-command)
+(defcustom ispell-look-p (file-exists-p ispell-look-command)
   "*Non-nil means use `look' rather than `grep'.
 Default is based on whether `look' seems to be available."
   :type 'boolean
@@ -444,7 +441,9 @@ Otherwise use the minibuffer."
   :type 'boolean
   :group 'ispell)
 
-(defcustom ispell-format-word (function upcase)
+;; upcase is not good for iso-latin!
+(defcustom ispell-format-word (function (lambda (str) (format "\"%s\"" str))) 
+  ;; was (function upcase) 
   "*Formatting function for displaying word being spell checked.
 The function must take one string argument and return a string."
   :type 'function
@@ -491,24 +490,26 @@ for language-specific arguments."
 (defvar ispell-dictionary-alist-1	; sk  9-Aug-1991 18:28
   '((nil				; default (english.aff)
      "[A-Za-z]" "[^A-Za-z]" "[']" nil ("-B") nil)
-    ("english"				; make english explicitly selectable
+    ("english"				; make English explicitly selectable
      "[A-Za-z]" "[^A-Za-z]" "[']" nil ("-B") nil)
-    ("british"				; british version
+    ("british"				; British version
      "[A-Za-z]" "[^A-Za-z]" "[']" nil ("-B" "-d" "british") nil)
+    ("american"				; American version
+     "[A-Za-z]" "[^A-Za-z]" "[']" nil ("-B" "-d" "american") nil)
     ("deutsch"				; deutsch.aff
      "[a-zA-Z\"]" "[^a-zA-Z\"]" "[']" t ("-C") "~tex")
     ("deutsch8"
      "[a-zA-Z\304\326\334\344\366\337\374]"
      "[^a-zA-Z\304\326\334\344\366\337\374]"
-     "[']" t ("-C" "-d" "deutsch") "~latin1")
+     "[']" t ("-C" "-d" "deutsch") "~latin1" iso-8859-1)
     ("nederlands"				; nederlands.aff
      "[A-Za-z\300-\305\307\310-\317\322-\326\331-\334\340-\345\347\350-\357\361\362-\366\371-\374]"
      "[^A-Za-z\300-\305\307\310-\317\322-\326\331-\334\340-\345\347\350-\357\361\362-\366\371-\374]"
-     "[']" t ("-C") nil)
+     "[']" t ("-C") nil iso-8859-1)
     ("nederlands8"				; dutch8.aff
      "[A-Za-z\300-\305\307\310-\317\322-\326\331-\334\340-\345\347\350-\357\361\362-\366\371-\374]"
      "[^A-Za-z\300-\305\307\310-\317\322-\326\331-\334\340-\345\347\350-\357\361\362-\366\371-\374]"
-     "[']" t ("-C") nil)))
+     "[']" t ("-C") nil iso-8859-1)))
 
 ;;;###autoload
 (defvar ispell-dictionary-alist-2
@@ -517,27 +518,29 @@ for language-specific arguments."
      "[']" nil ("-C") nil)
     ("svenska8"				;8 bit swedish mode
      "[A-Za-z\345\344\366\305\304\366]"  "[^A-Za-z\345\344\366\305\304\366]"
-     "[']" nil ("-C" "-d" "svenska") "~list") ; Add `"-T" "list"' instead?
+     "[']" nil ("-C" "-d" "svenska") "~list" ; Add `"-T" "list"' instead?
+     iso-8859-1)
     ("norsk"                           ;8 bit norwegian mode
      "[A-Za-z\351\346\370\345\311\306\330\305]"  "[^A-Za-z\351\346\370\345\311\306\330\305]"
-     "[']" nil ("-C" "-d" "norsk") "~list")
+     "[']" nil ("-C" "-d" "norsk") "~list"
+     iso-8859-1)
     ("francais7"
      "[A-Za-z]" "[^A-Za-z]" "[`'^---]" t nil nil)
     ("francais"				; francais.aff
      "[A-Za-z\300\302\306\307\310\311\312\313\316\317\324\331\333\334\340\342\347\350\351\352\353\356\357\364\371\373\374]"
      "[^A-Za-z\300\302\306\307\310\311\312\313\316\317\324\331\333\334\340\342\347\350\351\352\353\356\357\364\371\373\374]"
-     "[---']" t nil "~list")
+     "[---']" t nil "~list" iso-8859-1)
     ("francais-tex"			; francais.aff
      "[A-Za-z\300\302\306\307\310\311\312\313\316\317\324\331\333\334\340\342\347\350\351\352\353\356\357\364\371\373\374\\]"
      "[^A-Za-z\300\302\306\307\310\311\312\313\316\317\324\331\333\334\340\342\347\350\351\352\353\356\357\364\371\373\374\\]"
-     "[---'^`\"]" t nil "~tex")
+     "[---'^`\"]" t nil "~tex" iso-8859-1)
     ("italiano"				; italiano.aff
      "[A-Za-z\300\310\311\314\315\316\322\331\332\340\350\351\354\355\356\362\371\372]"
      "[^A-Za-z\300\310\311\314\315\316\322\331\332\340\350\351\354\355\356\362\371\372]"
      "[']" t ("-d" "italiano") "~list")
     ("dansk"				; dansk.aff
      "[A-Z\306\330\305a-z\346\370\345]" "[^A-Z\306\330\305a-z\346\370\345]"
-     "" nil ("-C") nil)
+      "[']" nil ("-C") nil iso-8859-1)
     ))
 
 
@@ -551,7 +554,7 @@ for language-specific arguments."
 Each element of this list is also a list:
 
 \(DICTIONARY-NAME CASECHARS NOT-CASECHARS OTHERCHARS MANY-OTHERCHARS-P
-        ISPELL-ARGS EXTENDED-CHARACTER-MODE\)
+        ISPELL-ARGS EXTENDED-CHARACTER-MODE CODING-SYSTEM\)
 
 DICTIONARY-NAME is a possible value of variable `ispell-dictionary', nil
 means the default dictionary.
@@ -561,15 +564,19 @@ word.
 
 NOT-CASECHARS is the opposite regexp of CASECHARS.
 
-OTHERCHARS is a regular expression of other characters that are valid
-in word constructs.  Otherchars cannot be adjacent to each other in a
-word, nor can they begin or end a word.  This implies we can't check
-\"Stevens'\" as a correct possessive and other correct formations.
-
+OTHERCHARS are characters in the NOT-CASECHARS set but which can be used to
+construct words in some special way.  If OTHERCHARS characters follow and
+precede characters from CASECHARS, they are parsed as part of a word,
+otherwise they become word-breaks.  As an example in English, assume the
+set ['] (as a regular expression) for OTHERCHARS.  Then \"they're\" and
+\"Steven's\" are parsed as single words including the \"'\" character, but
+\"Stevens'\" does not include the quote character as part of the word.
+If you want OTHERCHARS to be empty, use nil.
 Hint: regexp syntax requires the hyphen to be declared first here.
 
-MANY-OTHERCHARS-P is non-nil if many otherchars are to be allowed in a
-word instead of only one.
+MANY-OTHERCHARS-P is non-nil when multiple OTHERCHARS are allowed in a word.
+Otherwise only a single OTHERCHARS character is allowed to be part of any
+single word.
 
 ISPELL-ARGS is a list of additional arguments passed to the ispell
 subprocess.
@@ -583,6 +590,9 @@ but the dictionary can control the extended character mode.
 Both defaults can be overruled in a buffer-local fashion. See
 `ispell-parsing-keyword' for details on this.
 
+CODING-SYSTEM is the coding system to be used for the ispell process
+and for decoding CASECHARS, NOT-CASECHARS and OTHERCHARS.
+
 Note that the CASECHARS and OTHERCHARS slots of the alist should
 contain the same character set as casechars and otherchars in the
 language.aff file \(e.g., english.aff\).")
@@ -709,18 +719,29 @@ language.aff file \(e.g., english.aff\).")
 (defvar ispell-offset 1
   "Offset that maps protocol differences between ispell 3.1 versions.")
 
+(defun ispell-decode-string (str)
+  (let (coding-system)
+    (if (setq coding-system (ispell-get-coding-system))
+	(decode-coding-string str coding-system)
+      str)))
+
 (defun ispell-get-casechars ()
-  (nth 1 (assoc ispell-dictionary ispell-dictionary-alist)))
+  (ispell-decode-string
+   (nth 1 (assoc ispell-dictionary ispell-dictionary-alist))))
 (defun ispell-get-not-casechars ()
-  (nth 2 (assoc ispell-dictionary ispell-dictionary-alist)))
+  (ispell-decode-string
+   (nth 2 (assoc ispell-dictionary ispell-dictionary-alist))))
 (defun ispell-get-otherchars ()
-  (nth 3 (assoc ispell-dictionary ispell-dictionary-alist)))
+  (ispell-decode-string
+   (nth 3 (assoc ispell-dictionary ispell-dictionary-alist))))
 (defun ispell-get-many-otherchars-p ()
   (nth 4 (assoc ispell-dictionary ispell-dictionary-alist)))
 (defun ispell-get-ispell-args ()
   (nth 5 (assoc ispell-dictionary ispell-dictionary-alist)))
 (defun ispell-get-extended-character-mode ()
   (nth 6 (assoc ispell-dictionary ispell-dictionary-alist)))
+(defun ispell-get-coding-system ()
+  (nth 7 (assoc ispell-dictionary ispell-dictionary-alist)))
 
 (defvar ispell-process nil
   "The process object for Ispell.")
@@ -948,10 +969,11 @@ Word syntax described by `ispell-dictionary-alist' (which see)."
 	 (ispell-many-otherchars-p (ispell-get-many-otherchars-p))
 	 (word-regexp (concat ispell-casechars
 			      "+\\("
-			      ispell-otherchars
-			      "?"
-			      (if extra-otherchars
-				  (concat extra-otherchars "?"))
+			      (unless (string-equal ispell-otherchars "")
+				(concat ispell-otherchars "?"))
+			      (unless (or (null extra-otherchars)
+					  (string-equal extra-otherchars ""))
+				(concat extra-otherchars "?"))
 			      ispell-casechars
 			      "+\\)"
 			      (if (or ispell-many-otherchars-p
@@ -1066,13 +1088,15 @@ used."
 		     max-lines))
 	;; not so good if there are over 20 or 30 options, but then, if
 	;; there are that many you don't want to scan them all anyway...
+	(when (integerp count)
+	  (setq count (int-char count)))
 	(while (memq count command-characters) ; skip command characters.
-	  (setq count (1+ count)
+	  (setq count (int-char (1+ count))
 		skipped (1+ skipped)))
 	(insert "(" count ") " (car choices) "  ")
 	(setq choices (cdr choices)
 	      count (1+ count)))
-      (setq count (- count ?0 skipped)))
+      (setq count (int-char (- count ?0 skipped))))
 
     ;; Assure word is visible
     (if (not (pos-visible-in-window-p end))
@@ -1117,14 +1141,15 @@ used."
 		result
 		(progn
 		  (undo-boundary)
-		  (message (concat "C-h or ? for more options; SPC to leave "
-				   "unchanged, Character to replace word"))
+		  (display-message 'prompt
+		      (concat "C-h or ? for more options; SPC to leave "
+			      "unchanged, Character to replace word"))
 		  (let ((inhibit-quit t))
 		    (setq char (if (fboundp 'read-char-exclusive)
 				   (read-char-exclusive)
 				 (read-char))
 			  skipped 0)
-		    (if (or quit-flag (= char ?\C-g)) ; C-g is like typing X
+		    (if (or quit-flag (eq char ?\C-g)) ; C-g is like typing X
 			(setq char ?X
 			      quit-flag nil)))
 		  ;; Adjust num to array offset skipping command characters.
@@ -1136,36 +1161,36 @@ used."
 		    (setq num (- char ?0 skipped)))
 
 		  (cond
-		   ((= char ? ) nil)	; accept word this time only
-		   ((= char ?i)		; accept and insert word into pers dict
+		   ((eq char ? ) nil)	; accept word this time only
+		   ((eq char ?i)	; accept and insert word into pers dict
 		    (process-send-string ispell-process (concat "*" word "\n"))
 		    (setq ispell-pdict-modified-p '(t)) ; dictionary modified!
 		    nil)
-		   ((or (= char ?a) (= char ?A)) ; accept word without insert
+		   ((or (eq char ?a) (eq char ?A)) ; accept word without insert
 		    (process-send-string ispell-process (concat "@" word "\n"))
 		    (if (null ispell-pdict-modified-p)
 			(setq ispell-pdict-modified-p
 			      (list ispell-pdict-modified-p)))
-		    (if (= char ?A) 0))	; return 0 for ispell-add buffer-local
-		   ((or (= char ?r) (= char ?R)) ; type in replacement
-		    (if (or (= char ?R) ispell-query-replace-choices)
+		    (if (eq char ?A) 0)) ; return 0 for ispell-add buffer-local
+		   ((or (eq char ?r) (eq char ?R)) ; type in replacement
+		    (if (or (eq char ?R) ispell-query-replace-choices)
 			(list (read-string "Query-replacement for: " word) t)
 		      (cons (read-string "Replacement for: " word) nil)))
-		   ((or (= char ??)
+		   ((or (eq char ??)
 			;; XEmacs change: help-char may not be an int.
 			(eq char (event-to-character
 				  (character-to-event help-char)))
-			(= char ?\C-h))
+			(eq char ?\C-h))
 		    (ispell-help)
 		    t)
 		   ;; Quit and move point back.
-		   ((= char ?x)
+		   ((eq char ?x)
 		    (ispell-pdict-save ispell-silently-savep)
 		    (message "Exited spell-checking")
 		    (setq ispell-quit t)
 		    nil)
 		   ;; Quit and preserve point.
-		   ((= char ?X)
+		   ((eq char ?X)
 		    (ispell-pdict-save ispell-silently-savep)
 		    (message "%s"
 		     (substitute-command-keys
@@ -1174,7 +1199,7 @@ used."
 		    (setq ispell-quit (max (point-min)
 					   (- (point) (length word))))
 		    nil)
-		   ((= char ?q)
+		   ((eq char ?q)
 		    (if (y-or-n-p "Really kill Ispell process? ")
 			(progn
 			  (ispell-kill-ispell t) ; terminate process.
@@ -1182,7 +1207,7 @@ used."
 						(point))
 				ispell-pdict-modified-p nil))
 		      t))		; continue if they don't quit.
-		   ((= char ?l)
+		   ((eq char ?l)
 		    (let ((new-word (read-string
 				     "Lookup string (`*' is wildcard): "
 				     word))
@@ -1212,7 +1237,7 @@ used."
 					       new-line)
 					     max-lines))
 				(while (memq count command-characters)
-				  (setq count (1+ count)
+				  (setq count (int-char (1+ count))
 					skipped (1+ skipped)))
 				(insert "(" count ") " (car choices) "  ")
 				(setq choices (cdr choices)
@@ -1235,12 +1260,12 @@ used."
 				  (setq line new-line)))
 			    (select-window (next-window)))))
 		    t)			; reselect from new choices
-		   ((= char ?u)
+		   ((eq char ?u)
 		    (process-send-string ispell-process
 					 (concat "*" (downcase word) "\n"))
 		    (setq ispell-pdict-modified-p '(t)) ; dictionary modified!
 		    nil)
-		   ((= char ?m)		; type in what to insert
+		   ((eq char ?m)	; type in what to insert
 		    (process-send-string
 		     ispell-process (concat "*" (read-string "Insert: " word)
 					    "\n"))
@@ -1250,11 +1275,11 @@ used."
 		    (if ispell-query-replace-choices ; Query replace flag
 			(list (nth num miss) 'query-replace)
 		      (nth num miss)))
-		   ((= char ?\C-l)
+		   ((eq char ?\C-l)
 		    (redraw-display) t)
-		   ((= char ?\C-r)
+		   ((eq char ?\C-r)
 		    (save-window-excursion (recursive-edit)) t)
-		   ((= char ?\C-z)
+		   ((eq char ?\C-z)
 		    (funcall (key-binding "\C-z"))
 		    t)
 		   (t (ding) t))))))
@@ -1437,7 +1462,7 @@ otherwise it is displayed normally."
 	(inhibit-quit t)		; inhibit interrupt processing here.
 	(buffer-undo-list t))		; don't clutter the undo list.
     (delete-region start end)
-    (insert-char ?  (- end start))	; mimimize amount of redisplay
+    (insert-char ?  (- end start))	; minimize amount of redisplay
     (sit-for 0)				; update display
     (if highlight (setq inverse-video (not inverse-video))) ; toggle video
     (delete-region start end)		; delete whitespace
@@ -1620,6 +1645,8 @@ scrolling the current window.  Leave the new window selected."
 	  ispell-filter-continue nil
 	  ispell-process-directory default-directory)
     (set-process-filter ispell-process 'ispell-filter)
+    (if ispell-dictionary
+	(set-process-coding-system ispell-process (ispell-get-coding-system)))
     (accept-process-output ispell-process) ; Get version ID line
     (cond ((null ispell-filter)
 	   (error "%s did not output version line" ispell-program-name))
@@ -1927,8 +1954,9 @@ With prefix argument, set the default directory."
 					offset-change (+ offset-change change)
 					end (+ end change)))))
 			      (if (not ispell-quit)
-				  (message "Continuing spelling check using %s dictionary..."
-					   (or ispell-dictionary "default")))
+				  (display-message 'command
+				    "Continuing spelling check using %s dictionary..."
+				    (or ispell-dictionary "default")))
 			      (sit-for 0)))
 			;; finished with line!
 			(setq ispell-filter (cdr ispell-filter)))))
@@ -2454,7 +2482,7 @@ Both should not be used to define a buffer-local dictionary."
 ;;; Local Variables:
 ;;; mode: emacs-lisp
 ;;; comment-column: 40
-;;; ispell-local-dictionary: "english"
+;;; ispell-local-dictionary: "american"
 ;;; End:
 
 
