@@ -1,7 +1,7 @@
 ;;; man.el --- browse UNIX manual pages
 ;; Keywords: help
 
-;; Copyright (C) 1985, 1993, 1994, 1996 Free Software Foundation, Inc.
+;; Copyright (C) 1985, 1993, 1994, 1996, 1997 Free Software Foundation, Inc.
 ;;
 ;; This file is part of XEmacs.
 
@@ -31,7 +31,7 @@
 ;;   I've deleted. ]
 
 (defgroup man nil
-  "Browse manual pages"
+  "Browse Unix manual pages"
   :group 'help)
 
 (defcustom Manual-program "man" "\
@@ -44,9 +44,9 @@
 nil means leave the buffer in fundamental-mode in another window.
 t means use `view-buffer' to display the man page in the current window.
 Any other value means use `view-buffer-other-window'."
-  :type '(choice (const :tag "fundamental-mode other window" nil)
-		 (const :tag "view-mode current window" t)
-		 (const :tag "view-mode other window" other))
+  :type '(radio (const :tag "Fundamental-mode other window" nil)
+		(const :tag "View-mode current window" t)
+		(sexp :format "%t\n" :tag "View-mode other window" other))
   :group 'man)
 
 (defcustom Manual-mode-hook nil
@@ -57,6 +57,8 @@ Any other value means use `view-buffer-other-window'."
 (defvar Manual-page-history nil "\
 A list of names of previously visited man page buffers.")
 
+(defvar Manual-page-minibuffer-history nil "\
+Minibuffer completion history for `manual-entry'.")
 
 ;; New variables.
 
@@ -68,71 +70,70 @@ This is relavent for Solaris and, perhaps, other systems which have
   :type 'boolean
   :group 'man)
 
+;;Here is information on RosettaMan, from Neal.Becker@comsat.com (Neal Becker):
+
+;;RosettaMan is a filter for UNIX manual pages.  It takes as input man
+;;pages formatted for a variety of UNIX flavors (not [tn]roff source)
+;;and produces as output a variety of file formats.  Currently
+;;RosettaMan accepts man pages as formatted by the following flavors of
+;;UNIX: Hewlett-Packard HP-UX, AT&T System V, SunOS, Sun Solaris, OSF/1,
+;;DEC Ultrix, SGI IRIX, Linux, SCO; and produces output for the following
+;;formats: printable ASCII only (stripping page headers and footers),
+;;section and subsection headers only, TkMan, [tn]roff, Ensemble, RTF,
+;;SGML (soon--I finally found a DTD), HTML, MIME, LaTeX, LaTeX 2e, Perl 5's pod.
+
+;;RosettaMan improves on other man page filters in several ways: (1) its
+;;analysis recognizes the structural pieces of man pages, enabling high
+;;quality output, (2) its modular structure permits easy augmentation of
+;;output formats, (3) it accepts man pages formatted with the varient
+;;macros of many different flavors of UNIX, and (4) it doesn't require
+;;modification or cooperation with any other program.
+
+;;RosettaMan is a rewrite of TkMan's man page filter, called bs2tk.  (If
+;;you haven't heard about TkMan, a hypertext man page browser, you
+;;should grab it via anonymous ftp from ftp.cs.berkeley.edu:
+;;/ucb/people/phelps/tkman.tar.Z.)  Whereas bs2tk generated output only for
+;;TkMan, RosettaMan generalizes the process so that the analysis can be
+;;leveraged to new output formats.  A single analysis engine recognizes
+;;section heads, subsection heads, body text, lists, references to other
+;;man pages, boldface, italics, bold italics, special characters (like
+;;bullets), tables (to a degree) and strips out page headers and
+;;footers.  The engine sends signals to the selected output functions so
+;;that an enhancement in the engine improves the quality of output of
+;;all of them.  Output format functions are easy to add, and thus far
+;;average about about 75 lines of C code each.
+
+
+
+;;*** NOTES ON CURRENT VERSION ***
+
+;;Help!  I'm looking for people to help with the following projects.
+;;\(1) Better RTF output format.  The current one works, but could be
+;;made better.  (2) Roff macros that produce text that is easily
+;;parsable.  RosettaMan handles a great variety, but some things, like
+;;H-P's tables, are intractable.  If you write an output format or
+;;otherwise improve RosettaMan, please send in your code so that I may
+;;share the wealth in future releases.
+
+;;This version can try to identify tables (turn this on with the -T
+;;switch) by looking for lines with a large amount of interword spacing,
+;;reasoning that this is space between columns of a table.  This
+;;heuristic doesn't always work and sometimes misidentifies ordinary
+;;text as tables.  In general I think it is impossible to perfectly
+;;identify tables from nroff formatted text.  However, I do think the
+;;heuristics can be tuned, so if you have a collection of manual pages
+;;with unrecognized tables, send me the lot, in formatted form (i.e.,
+;;after formatting with nroff -man), and uuencode them to preserve the
+;;control characters.  Better, if you can think of heuristics that
+;;distinguish tables from ordinary text, I'd like to hear them.
+
+;;Notes for HTML consumers: This filter does real (heuristic)
+;;parsing--no <PRE>!  Man page references are turned into hypertext links.
+
 (defcustom Manual-use-rosetta-man (not (null (locate-file "rman" exec-path))) "\
 If non-nil, use RosettaMan (rman) to filter man pages.
 This makes man-page cleanup virtually instantaneous, instead of
-potentially taking a long time.
-
-Here is information on RosettaMan, from Neal.Becker@comsat.com (Neal Becker):
-
-RosettaMan is a filter for UNIX manual pages.  It takes as input man
-pages formatted for a variety of UNIX flavors (not [tn]roff source)
-and produces as output a variety of file formats.  Currently
-RosettaMan accepts man pages as formatted by the following flavors of
-UNIX: Hewlett-Packard HP-UX, AT&T System V, SunOS, Sun Solaris, OSF/1,
-DEC Ultrix, SGI IRIX, Linux, SCO; and produces output for the following
-formats: printable ASCII only (stripping page headers and footers),
-section and subsection headers only, TkMan, [tn]roff, Ensemble, RTF,
-SGML (soon--I finally found a DTD), HTML, MIME, LaTeX, LaTeX 2e, Perl 5's pod.
-
-RosettaMan improves on other man page filters in several ways: (1) its
-analysis recognizes the structural pieces of man pages, enabling high
-quality output, (2) its modular structure permits easy augmentation of
-output formats, (3) it accepts man pages formatted with the varient
-macros of many different flavors of UNIX, and (4) it doesn't require
-modification or cooperation with any other program.
-
-RosettaMan is a rewrite of TkMan's man page filter, called bs2tk.  (If
-you haven't heard about TkMan, a hypertext man page browser, you
-should grab it via anonymous ftp from ftp.cs.berkeley.edu:
-/ucb/people/phelps/tkman.tar.Z.)  Whereas bs2tk generated output only for
-TkMan, RosettaMan generalizes the process so that the analysis can be
-leveraged to new output formats.  A single analysis engine recognizes
-section heads, subsection heads, body text, lists, references to other
-man pages, boldface, italics, bold italics, special characters (like
-bullets), tables (to a degree) and strips out page headers and
-footers.  The engine sends signals to the selected output functions so
-that an enhancement in the engine improves the quality of output of
-all of them.  Output format functions are easy to add, and thus far
-average about about 75 lines of C code each.
-
-
-
-*** NOTES ON CURRENT VERSION ***
-
-Help!  I'm looking for people to help with the following projects.
-\(1) Better RTF output format.  The current one works, but could be
-made better.  (2) Roff macros that produce text that is easily
-parsable.  RosettaMan handles a great variety, but some things, like
-H-P's tables, are intractable.  If you write an output format or
-otherwise improve RosettaMan, please send in your code so that I may
-share the wealth in future releases.
-
-This version can try to identify tables (turn this on with the -T
-switch) by looking for lines with a large amount of interword spacing,
-reasoning that this is space between columns of a table.  This
-heuristic doesn't always work and sometimes misidentifies ordinary
-text as tables.  In general I think it is impossible to perfectly
-identify tables from nroff formatted text.  However, I do think the
-heuristics can be tuned, so if you have a collection of manual pages
-with unrecognized tables, send me the lot, in formatted form (i.e.,
-after formatting with nroff -man), and uuencode them to preserve the
-control characters.  Better, if you can think of heuristics that
-distinguish tables from ordinary text, I'd like to hear them.
-
-
-Notes for HTML consumers: This filter does real (heuristic)
-parsing--no <PRE>!  Man page references are turned into hypertext links."
+potentially taking a long time."
   :type 'boolean
   :group 'man)
 
@@ -165,7 +166,7 @@ parsing--no <PRE>!  Man page references are turned into hypertext links."
 (defun manual-entry (topic &optional arg silent)
   "Display the Unix manual entry (or entries) for TOPIC."
   (interactive
-   (list (let* ((fmh "-A-Za-z0-9_.")
+   (list (let* ((fmh "-A-Za-z0-9_.:")
 		(default (save-excursion
 			   (buffer-substring
 			    (progn
@@ -174,20 +175,18 @@ parsing--no <PRE>!  Man page references are turned into hypertext links."
 			    (progn (skip-chars-forward fmh) (point)))))
 		(thing (read-string
 			(if (equal default "") "Manual entry: "
-			  (concat "Manual entry: (default " default ") ")))))
+			  (concat "Manual entry: (default " default ") "))
+			nil 'Manual-page-minibuffer-history)))
 	   (if (equal thing "") default thing))
 	 (prefix-numeric-value current-prefix-arg)))
-  ;;(interactive "sManual entry (topic): \np")
   (or arg (setq arg 1))
   (let (section apropos-mode)
     (let ((case-fold-search nil))
       (if (and (null section)
 	       (string-match "\\`[ \t]*\\([^( \t]+\\)[ \t]*(\\(.+\\))[ \t]*\\'"
 			     topic))
-	  (setq section (substring topic (match-beginning 2)
-				   (match-end 2))
-		topic (substring topic (match-beginning 1)
-				 (match-end 1)))
+	  (setq section (match-string 2 topic)
+		topic (match-string 1 topic))
 	(if (string-match "\\`[ \t]*-k[ \t]+\\([^ \t]+\\)\\'" topic)
 	    (setq section "-k"
 		  topic (substring topic (match-beginning 1))))))
@@ -195,16 +194,15 @@ parsing--no <PRE>!  Man page references are turned into hypertext links."
     (when Manual-snip-subchapter
       ;; jwz: turn section "3x11" and "3n" into "3".
       (if (and section (string-match "\\`\\([0-9]+\\)[^0-9]" section))
-	  (setq section (substring section 0 (match-end 1)))))
+	  (setq section (match-string 1 section))))
+
     (if (equal section "-k")
 	(setq apropos-mode t))
 
-    (let ((bufname (cond (apropos-mode
-			  (concat "*man apropos " topic "*"))
-			 (t
-			  (concat "*man " topic
-				  (if section (concat "." section) "")
-				  "*"))))
+    (let ((bufname (concat "Man"
+			   (when apropos-mode " apropos")
+			   ": " topic
+			   (when section (concat "(" section ")"))))
 	  (temp-buffer-show-function 
 	   (cond ((eq 't Manual-buffer-view-mode)
 		  'view-buffer)
@@ -233,7 +231,8 @@ parsing--no <PRE>!  Man page references are turned into hypertext links."
 		       args-string)
 		   (if section
 		       (setq args
-			     (if (eq system-type 'usg-unix-v)
+			     (if (and (eq system-type 'usg-unix-v)
+				      (null apropos-mode))
 				 (cons "-s" (cons section args))
 			       (cons section args))))
 		   (setq args-string
@@ -243,26 +242,27 @@ parsing--no <PRE>!  Man page references are turned into hypertext links."
 			     (substring args-string (match-end 0))))
 
 		   (message "%s (running...)" args-string)
-		   (apply 'call-process Manual-program nil t nil args)
+		   (apply 'call-process Manual-program nil '(t nil) nil args)
 
 		   (if (< (buffer-size) 200)
 		       (progn
-			 (goto-char (point-min))
-			 (error (buffer-substring (point)
-						  (progn (end-of-line)
-							 (point))))))
+			 (kill-buffer (current-buffer))
+			 (error "%s not found" args-string)))
 
 		   (message "%s (cleaning...)" args-string)
 		   (Manual-nuke-nroff-bs apropos-mode)
-		   (message "%s (done.)" args-string)
-		   )
-
+		   (message "%s (done.)" args-string))
 		 (set-buffer-modified-p nil)
-		 (Manual-mode)
-		 ))))
-      (setq Manual-page-history
-	    (cons (buffer-name)
-		  (delete (buffer-name) Manual-page-history)))))
+		 (Manual-mode)))))
+      (let ((page (if section
+		      (concat topic "(" section ")")
+		    topic)))
+	(setq Manual-page-history
+	      (cons (buffer-name)
+		    (delete (buffer-name) Manual-page-history))
+	      Manual-page-minibuffer-history
+	      (cons page (delete page Manual-page-minibuffer-history))))))
+
   (message nil)
   t)
 
@@ -279,15 +279,24 @@ parsing--no <PRE>!  Man page references are turned into hypertext links."
   ;; turn off horizontal scrollbars in this buffer
   (when (featurep 'scrollbar)
     (set-specifier scrollbar-height (cons (current-buffer) 0)))
+  (make-local-hook 'kill-buffer-hook)
+  (add-hook 'kill-buffer-hook #'(lambda ()
+				  (setq Manual-page-history
+					(delete (buffer-name)
+						Manual-page-history)))
+	    nil t)
   (run-hooks 'Manual-mode-hook))
 
 (defun Manual-last-page ()
   (interactive)
-  (while (or (not (get-buffer (car (or Manual-page-history
-				       (error "No more history.")))))
-	     (eq (get-buffer (car Manual-page-history)) (current-buffer)))
-    (setq Manual-page-history (cdr Manual-page-history)))
-  (switch-to-buffer (car Manual-page-history)))
+  (if Manual-page-history
+      (let ((page (pop Manual-page-history)))
+	(if page
+	    (progn
+	      (get-buffer page)
+	      (cons Manual-page-history page)
+	      (switch-to-buffer page))))
+    (error "No manual page buffers found. Use `M-x manual-entry'")))
 
 
 (defmacro Manual-delete-char (n)
@@ -296,7 +305,7 @@ parsing--no <PRE>!  Man page references are turned into hypertext links."
   ;; in delete-char alone.)
   (list 'delete-region '(point) (list '+ '(point) n)))
 
-;; Hint: BS stands form more things than "back space"
+;; Hint: BS stands for more things than "back space"
 (defun Manual-nuke-nroff-bs (&optional apropos-mode)
   (interactive "*")
   (if Manual-use-rosetta-man
@@ -393,6 +402,7 @@ parsing--no <PRE>!  Man page references are turned into hypertext links."
 
 
 (defun Manual-nuke-nroff-bs-footers ()
+  "For info see comments in packages/man.el"
   ;; Nuke headers and footers.
   ;;
   ;; nroff assumes pages are 66 lines high.  We assume that, and that the
@@ -490,36 +500,60 @@ parsing--no <PRE>!  Man page references are turned into hypertext links."
 
 (defun Manual-mouseify-xrefs ()
   (goto-char (point-min))
-  (forward-line 1)
+  ;; skip the top line of manual pages, but not apropos listings.
+  (unless apropos-mode (forward-line 1))
   (let ((case-fold-search nil)
-	s e name extent)
+	s e name splitp extent)
     ;; possibly it would be faster to rewrite this expression to search for
     ;; a less common sequence first (like "([0-9]") and then back up to see
     ;; if it's really a match.  This function is 15% of the total time, 13%
     ;; of which is this call to re-search-forward.
-    (while (re-search-forward "[a-zA-Z_][-a-zA-Z0-9_.]*([0-9][a-zA-Z0-9]*)"
+    (while (re-search-forward "[a-zA-Z_][-a-zA-Z0-9_.:]*([0-9][a-zA-Z0-9]*)"
 			      nil t)
       (setq s (match-beginning 0)
 	    e (match-end 0)
-	    name (buffer-substring s e))
+	    name (buffer-substring s e)
+	    splitp nil)
+
       (goto-char s)
-      (skip-chars-backward " \t")
-      (if (and (bolp)
-	       (progn (backward-char 1) (= (preceding-char) ?-)))
-	  (progn
-	    (setq s (point))
-	    (skip-chars-backward "-a-zA-Z0-9_.")
-	    (setq name (concat (buffer-substring (point) (1- s)) name))
-	    (setq s (point))))
+      ;; if this is a hyphenated xref, we're on the second line, 1st char now.
+      (when (progn
+	      (beginning-of-line)
+	      (and (looking-at (concat "^[ \t]+" (regexp-quote name)))
+		   (progn
+		     (backward-char 1)
+		     (or (equal (char-before) ?-)
+			 (equal (char-before) ?\255)))
+		   (setq s (progn
+			     (skip-chars-backward "-\255_a-zA-Z0-9")
+			     (point))
+			 name (buffer-substring s e))))
+	(setq splitp t)
+	;; delete the spaces and dash from `name'
+	(let (i)
+	  (while (setq i (string-match "[-\255 \n\t]+" name i))
+	    (setq name (concat (substring name 0 i)
+			       (substring name (match-end 0)))
+		  i (1+ i)))))
+
       ;; if there are upper case letters in the section, downcase them.
       (if (string-match "(.*[A-Z]+.*)$" name)
 	  (setq name (concat (substring name 0 (match-beginning 0))
 			     (downcase (substring name (match-beginning 0))))))
-      ;; (setq already-fontified (extent-at s))
-      (setq extent (make-extent s e))
+
+      ;; if the xref was hyphenated, don't highlight the indention spaces.
+      (if splitp
+	  (progn
+	    (setq extent (make-extent s (progn (goto-char s) (end-of-line) (point))))
+	    (set-extent-property extent 'man (list 'Manual-follow-xref name))
+	    (set-extent-property extent 'highlight t)
+	    (set-extent-face extent 'man-xref)
+	    (goto-char e)
+	    (skip-chars-backward "-_a-zA-Z0-9()")
+	    (setq extent (make-extent (point) e)))
+	(setq extent (make-extent s e)))
       (set-extent-property extent 'man (list 'Manual-follow-xref name))
       (set-extent-property extent 'highlight t)
-      ;; (if (not already-fontified)...
       (set-extent-face extent 'man-xref)
       (goto-char e))))
 
@@ -557,7 +591,6 @@ on the menu in the order in which they appear in the buffer."
   (interactive "e")
   (let ((buffer (current-buffer))
 	(sep "---")
-	(prefix "Show Manual Page for ")
 	xref items)
     (cond (event
 	   (setq buffer (event-buffer event))
@@ -575,13 +608,14 @@ on the menu in the order in which they appear in the buffer."
 		    nil))
 		 buffer)
     (if (eq sep (car items)) (setq items (cdr items)))
-    (let ((popup-menu-titles nil))
+    (let ((popup-menu-titles t))
+      (and (null items) (setq popup-menu-titles nil))
       (popup-menu
        (cons "Manual Entry"
 	     (mapcar #'(lambda (item)
 			 (if (eq item sep)
 			     item
-                           (vector (concat prefix item)
+                           (vector item
                                    (list 'Manual-follow-xref item) t)))
 		     (nreverse items)))))))
 
@@ -603,9 +637,9 @@ on the menu in the order in which they appear in the buffer."
 						  (match-end 1)))
 		(setq manpage "???"))
 	      (setq buffer
-		    (rename-buffer
-		     (generate-new-buffer-name (concat "*man " manpage "*"))))
-	      (setq buffer-file-name nil)
+		    (rename-buffer (generate-new-buffer-name
+				    (concat "Man: " manpage)))
+		    buffer-file-name nil)
 	      (goto-char (point-min))
 	      (insert (format "%s\n" buf-name))
 	      (goto-char (point-min))
