@@ -208,7 +208,8 @@
 
 (defgroup recent-files nil
   "Maintain a menu of recently opened files."
-  :group 'data)
+  :group 'files
+  :group 'menu)
 
 (defgroup recent-files-menu nil
   "Menu options of recent-files."
