@@ -975,7 +975,7 @@ Match addresses of the style ``name%[stuff].'' when called with DELIM
 of \"%\" and addresses of the style ``[stuff]name@[stuff]'' when
 called with DELIM \"@\".  If DELIM is nil or not provided, matches
 addresses of the style ``name''."
-  (and (string-match (concat "[-a-zA-Z0-9_.]+" delim) from 0)
+  (and (string-match (concat "[-+a-zA-Z0-9_.]+" delim) from 0)
        (substring from
 		  (match-beginning 0)
 		  (- (match-end 0) (if (null delim) 0 1)))))
@@ -1008,7 +1008,7 @@ AUTHOR is the author's name (which is removed from the address)."
 		   (= (aref address (1- (length address))) ?>))
 	      (substring address 1 (1- (length address)))
 	    address))
-      (if (string-match "[-a-zA-Z0-9!@%._]+" from 0)
+      (if (string-match "[-+a-zA-Z0-9!@%._]+" from 0)
 	  (sc-submatch 0 from)
 	"")
       )))
@@ -1052,10 +1052,10 @@ This should be the author's full name minus an optional title."
 	(let ((bos 0)
 	      (eos (1- (length namestring))))
 	  (while (and (<= bos eos)
-		      (memq (aref namestring bos) '(32 ?\t)))
+		      (memq (aref namestring bos) '(?\  ?\t)))
 	    (setq bos (1+ bos)))
 	  (while (and (> eos bos)
-		      (memq (aref namestring eos) '(32 ?\t)))
+		      (memq (aref namestring eos) '(?\  ?\t)))
 	    (setq eos (1- eos)))
 	  (substring namestring bos (1+ eos))))))
 
@@ -1964,7 +1964,7 @@ Note on function names in this list: all functions of the form
 (defun sc-cite-original ()
   "Workhorse citing function which performs the initial citation.
 This is callable from the various mail and news readers' reply
-function according to the agreed upon standard.  See `\\[sc-describe]'
+function according to the agreed upon standard.  See `sc-describe'
 for more details.  `sc-cite-original' does not do any yanking of the
 original message but it does require a few things:
 
