@@ -90,3 +90,7 @@ EXTENT, until no more children can be found."
 (defun set-extent-keymap (extent keymap)
   "Set EXTENT's `keymap' property to KEYMAP."
   (set-extent-property extent 'keymap keymap))
+
+(defun extent-keymap (extent)
+  "Return EXTENT's `keymap' property."
+  (extent-property extent 'keymap))
