@@ -27,12 +27,16 @@
 
 ;;; Code:
 
+(defgroup frames nil
+  "Support for Emacs frames and window systems."
+  :group 'environment)
+
 ; No need for `frame-creation-function'.
 
 ;;; The initial value given here for this must ask for a minibuffer.
 ;;; There must always exist a frame with a minibuffer, and after we
 ;;; delete the terminal frame, this will be the only frame.
-(defvar initial-frame-plist '(minibuffer t)
+(defcustom initial-frame-plist '(minibuffer t)
   "Plist of frame properties for creating the initial X window frame.
 You can set this in your `.emacs' file; for example,
   (setq initial-frame-plist '(top 1 left 1 width 80 height 55))
@@ -52,30 +56,42 @@ as it appears, you need to use this three-step process:
 * Set `default-frame-plist' to override these options so that they
   don't affect subsequent frames.
 * Set `initial-frame-plist' in a way that matches the X resources,
-  to override what you put in `default-frame-plist'.")
+  to override what you put in `default-frame-plist'."
+  :type '(repeat (group :inline t
+			(symbol :tag "Property")
+			(sexp :tag "Value")))
+  :group 'frames)
 
-(defvar minibuffer-frame-plist '(width 80 height 2 menubar-visible-p nil
+(defcustom minibuffer-frame-plist '(width 80 height 2 menubar-visible-p nil
 				       default-toolbar-visible-p nil)
   "Plist of frame properties for initially creating a minibuffer frame.
 You can set this in your `.emacs' file; for example,
   (setq minibuffer-frame-plist '(top 1 left 1 width 80 height 2))
 Properties specified here supersede the values given in
 `default-frame-plist'.
-The format of this can also be an alist for backward compatibility.")
+The format of this can also be an alist for backward compatibility."
+  :type '(repeat (group :inline t
+			(symbol :tag "Property")
+			(sexp :tag "Value")))
+  :group 'frames)
 
-(defvar pop-up-frame-plist nil
+(defcustom pop-up-frame-plist nil
   "Plist of frame properties used when creating pop-up frames.
 Pop-up frames are used for completions, help, and the like.
 This variable can be set in your init file, like this:
   (setq pop-up-frame-plist '(width 80 height 20))
 These supersede the values given in `default-frame-plist'.
-The format of this can also be an alist for backward compatibility.")
+The format of this can also be an alist for backward compatibility."
+  :type '(repeat (group :inline t
+			(symbol :tag "Property")
+			(sexp :tag "Value")))
+  :group 'frames)
 
 (setq pop-up-frame-function
       (function (lambda ()
 		  (make-frame pop-up-frame-plist))))
 
-(defvar special-display-frame-plist '(height 14 width 80 unsplittable t)
+(defcustom special-display-frame-plist '(height 14 width 80 unsplittable t)
   "*Plist of frame properties used when creating special frames.
 Special frames are used for buffers whose names are in
 `special-display-buffer-names' and for buffers whose names match
@@ -83,7 +99,11 @@ one of the regular expressions in `special-display-regexps'.
 This variable can be set in your init file, like this:
   (setq special-display-frame-plist '(width 80 height 20))
 These supersede the values given in `default-frame-plist'.
-The format of this can also be an alist for backward compatibility.")
+The format of this can also be an alist for backward compatibility."
+  :type '(repeat (group :inline t
+			(symbol :tag "Property")
+			(sexp :tag "Value")))
+  :group 'frames)
 
 (defun safe-alist-to-plist (cruftiness)
   (if (consp (car cruftiness))
@@ -525,7 +545,8 @@ A negative ARG moves in the opposite order."
 ; this is in C in FSFmacs
 (defun frame-list ()
   "Return a list of all frames on all devices/consoles."
-  (apply 'append (mapcar 'device-frame-list (device-list))))
+  ;; Lists are copies, so nconc is safe here.
+  (apply 'nconc (mapcar 'device-frame-list (device-list))))
 
 (defun frame-type (&optional frame)
   "Return the type of the specified frame (e.g. `x' or `tty').
@@ -593,26 +614,25 @@ is given and non-nil, the unwanted frames are iconified instead."
 	      (list 'frame-configuration-p configuration)))
   (let ((config-plist (cdr configuration))
 	frames-to-delete)
-    (mapcar (function
-	     (lambda (frame)
-	       (let ((properties (assq frame config-plist)))
-		 (if properties
-		     (progn
-		       (set-frame-properties
-			frame
-			;; Since we can't set a frame's minibuffer status,
-			;; we might as well omit the parameter altogether.
-			(lax-plist-remprop (nth 1 properties) 'minibuffer))
-		       (set-window-configuration (nth 2 properties)))
-		   (setq frames-to-delete (cons frame frames-to-delete))))))
-	    (frame-list))
+    (mapc (lambda (frame)
+	    (let ((properties (assq frame config-plist)))
+	      (if properties
+		  (progn
+		    (set-frame-properties
+		     frame
+		     ;; Since we can't set a frame's minibuffer status,
+		     ;; we might as well omit the parameter altogether.
+		     (lax-plist-remprop (nth 1 properties) 'minibuffer))
+		    (set-window-configuration (nth 2 properties)))
+		(setq frames-to-delete (cons frame frames-to-delete)))))
+	  (frame-list))
     (if nodelete
 	;; Note: making frames invisible here was tried
 	;; but led to some strange behavior--each time the frame
 	;; was made visible again, the window manager asked afresh
 	;; for where to put it.
-	(mapcar 'iconify-frame frames-to-delete)
-      (mapcar 'delete-frame frames-to-delete))))
+	(mapc 'iconify-frame frames-to-delete)
+      (mapc 'delete-frame frames-to-delete))))
 
 ; this function is in subr.el in FSFmacs.
 ; that's because they don't always include frame.el, while we do.
@@ -767,15 +787,19 @@ all frames that were visible, and iconify all frames that were not."
 
 ;;; auto-raise and auto-lower
 
-(defvar auto-raise-frame nil
+(defcustom auto-raise-frame nil
   "*If true, frames will be raised to the top when selected.
 Under X, most ICCCM-compliant window managers will have an option to do this
-for you, but this variable is provided in case you're using a broken WM.")
+for you, but this variable is provided in case you're using a broken WM."
+  :type 'boolean
+  :group 'frames)
 
-(defvar auto-lower-frame nil
+(defcustom auto-lower-frame nil
   "*If true, frames will be lowered to the bottom when no longer selected.
 Under X, most ICCCM-compliant window managers will have an option to do this
-for you, but this variable is provided in case you're using a broken WM.")
+for you, but this variable is provided in case you're using a broken WM."
+  :type 'boolean
+  :group 'frames)
 
 (defun default-select-frame-hook ()
   "Implements the `auto-raise-frame' variable.
@@ -785,7 +809,8 @@ For use as the value of `select-frame-hook'."
 (defun default-deselect-frame-hook ()
   "Implements the `auto-lower-frame' variable.
 For use as the value of `deselect-frame-hook'."
-  (if auto-lower-frame (lower-frame (selected-frame))))
+  (if auto-lower-frame (lower-frame (selected-frame)))
+  (highlight-extent nil nil))
 
 (or select-frame-hook
     (add-hook 'select-frame-hook 'default-select-frame-hook))
@@ -796,8 +821,10 @@ For use as the value of `deselect-frame-hook'."
 (defun default-drag-and-drop-functions (frame filepath &optional data)
   "Implements the `drag-and-drop-functions' variable.
 For use as the value of `drag-and-drop-functions'.
-This default simply pops up the file in the selected frame or,
-if the dragged object is a buffer, inserts it at point."
+A file is popped up in a new buffer, some data without
+is inserted at point."
+  ;; changed this back -- hope it works for CDE ;-) Oliver Graf <ograf@fga.de>
+  ;; the OffiX drop stuff has moved to mouse.el (mouse-offix-drop)
   (if data
       (insert data)
     (let ((x pop-up-windows))
@@ -821,6 +848,29 @@ Calls the internal function cde-start-drag-internal to do the actual work."
 	       (buffer-substring-no-properties begin end))
     (error "CDE functionality not compiled in.")))
 
+;; the OffiX drag stuff will soon move also (perhaps mouse.el)
+;; if the drag event is done
+(defun offix-start-drag (event data &optional type)
+  "Implements the OffiX drag operation.
+Calls the internal function offix-start-drag-internal to do the actual work.
+If type is not given, DndText is assumed."
+  ;; Oliver Graf <ograf@fga.de>
+  (interactive "esi")
+  (if (featurep 'offix)
+      (funcall (intern "offix-start-drag-internal") event data type)
+    (error "OffiX functionality not compiled in.")))
+
+(defun offix-start-drag-region (event begin end)
+  "Implements the OffiX drag operation for a region.
+Calls the internal function offix-start-drag-internal to do the actual work.
+This always assumes DndText as type."
+  ;; Oliver Graf <ograf@fga.de>
+  (interactive "_er")
+  (if (featurep 'offix)
+      (funcall (intern "offix-start-drag-internal")
+	       event (buffer-substring-no-properties begin end))
+    (error "OffiX functionality not compiled in.")))
+
 
 
 ;;; Application-specific frame-management
@@ -837,9 +887,8 @@ Calls the internal function cde-start-drag-internal to do the actual work."
     (or (get mode 'frame-name)
 	get-frame-for-buffer-default-frame-name)))
 
-
-(defun get-frame-for-buffer-make-new-frame (buffer &optional frame-name)
-  (let* ((fr (make-frame (and frame-name (list (cons 'name frame-name)))))
+(defun get-frame-for-buffer-make-new-frame (buffer &optional frame-name plist)
+  (let* ((fr (make-frame plist))
 	 (w (frame-root-window fr)))
     ;;
     ;; Make the one buffer being displayed in this newly created
@@ -857,6 +906,11 @@ Calls the internal function cde-start-drag-internal to do the actual work."
 	(set-window-buffer w buffer))
     fr))
 
+(defcustom get-frame-for-buffer-default-to-current nil
+  "*When non-nil, `get-frame-for-buffer' will default to the current frame."
+  :type 'boolean
+  :group 'frames)
+
 (defun get-frame-for-buffer-noselect (buffer
 				      &optional not-this-window-p on-frame)
   "Return a frame in which to display BUFFER.
@@ -874,6 +928,7 @@ This is a subroutine of `get-frame-for-buffer' (which see)."
       ;; name.  That always takes priority.
       ;;
       (let ((limit (get name 'instance-limit))
+	    (defaults (get name 'frame-defaults))
 	    (matching-frames '())
 	    frames frame already-visible)
 	;; Sort the list so that iconic frames will be found last.  They
@@ -911,7 +966,11 @@ This is a subroutine of `get-frame-for-buffer' (which see)."
 	      ((or (null matching-frames)
 		   (eq limit 0) ; means create with reckless abandon
 		   (and limit (< (length matching-frames) limit)))
-	       (get-frame-for-buffer-make-new-frame buffer name))
+	       (get-frame-for-buffer-make-new-frame
+		buffer
+		name
+		(alist-to-plist (acons 'name name
+				       (plist-to-alist defaults)))))
 	      (t
 	       ;; do not switch any of the window/buffer associations in an
 	       ;; existing frame; this function only picks a frame; the
@@ -957,6 +1016,13 @@ This is a subroutine of `get-frame-for-buffer' (which see)."
 	    (get-frame-for-buffer-make-new-frame buffer)
 	  (car frames))))
 
+     ;;
+     ;; This buffer's mode did not express a preference for a frame of a
+     ;; particular name.  However, the user wants the frame to be the
+     ;; current one in that case.
+     ;;
+     (get-frame-for-buffer-default-to-current nil)
+
      (t
       ;;
       ;; This buffer's mode did not express a preference for a frame of a
@@ -1046,11 +1112,15 @@ is first in the list.  VISIBLE-ONLY will only list non-iconified frames."
 	(append (list save-frame) frames)
       frames)))
 
-(defvar temp-buffer-shrink-to-fit nil
-  "*When non-nil resize temporary output buffers to minimize blank lines.")
+(defcustom temp-buffer-shrink-to-fit nil
+  "*When non-nil resize temporary output buffers to minimize blank lines."
+  :type 'boolean
+  :group 'frames)
 
-(defvar temp-buffer-max-height .5
-  "*Proportion of frame to use for temp windows.")
+(defcustom temp-buffer-max-height .5
+  "*Proportion of frame to use for temp windows."
+  :type 'number
+  :group 'frames)
 
 (defun show-temp-buffer-in-current-frame (buffer)
   "For use as the value of temp-buffer-show-function:
@@ -1082,7 +1152,7 @@ is normally set to `get-frame-for-buffer' (which see)."
 (defun delete-other-frames (&optional frame)
   "Delete all but FRAME (or the selected frame)."
   (interactive)
-  (mapcar 'delete-frame (delq (or frame (selected-frame)) (frame-list))))
+  (mapc 'delete-frame (delq (or frame (selected-frame)) (frame-list))))
 
 ;; By adding primitives to directly access the window hierarchy,
 ;; we can move many functions into Lisp.  We do it this way
