@@ -95,7 +95,9 @@ Keymap for characters following C-c.")
 (define-key global-map "\C-e" 'end-of-line)
 (define-key global-map "\C-f" 'forward-char)
 (define-key global-map "\C-d" 'delete-char)
-(define-key global-map "\177" 'delete-backward-char)
+(define-key global-map 'delete 'backward-or-forward-delete-char)
+(define-key global-map '(meta delete) 'backward-or-forward-kill-word)
+(define-key global-map [(control x) (delete)] 'backward-or-forward-kill-sentence)
 
 ;; FSFmacs files.el
 
@@ -139,8 +141,6 @@ Keymap for characters following C-c.")
 (define-key global-map "\C-x52" 'make-frame)
 (define-key global-map "\C-x50" 'delete-frame)
 (define-key global-map "\C-x5o" 'other-frame)
-;; XEmacs addition:
-(define-key global-map "\C-x5m" 'mail-other-frame)
 
 ;; FSFmacs help.el
 
@@ -179,6 +179,7 @@ Keymap for characters following C-c.")
 (put 'narrow-to-region 'disabled t)
 (define-key global-map "\C-xnn" 'narrow-to-region)
 (define-key global-map "\C-xnw" 'widen)
+(define-key global-map "\C-xnd" 'narrow-to-defun)
 ;; Old v18 bindings
 ;(define-key global-map "\C-xn" 'narrow-to-region)
 ;(define-key global-map "\C-xw" 'widen)
@@ -270,7 +271,6 @@ Keymap for characters following C-c.")
 (define-key global-map "\M-f" 'forward-word)
 (define-key global-map "\M-b" 'backward-word)
 (define-key global-map "\M-d" 'kill-word)
-(define-key global-map "\M-\177" 'backward-kill-word)
 
 (define-key global-map "\M-<" 'beginning-of-buffer)
 (define-key global-map "\M->" 'end-of-buffer)
@@ -292,7 +292,8 @@ Keymap for characters following C-c.")
 (define-key global-map "\M-\)" 'move-past-close-and-reindent)
 (define-key global-map "\M-\t" 'lisp-complete-symbol)
 
-(define-key global-map '(control meta delete) 'backward-kill-sexp)
+(define-key global-map '(control meta backspace) 'backward-kill-sexp)
+(define-key global-map '(control meta delete) 'backward-or-forward-kill-sexp)
 
 
 (define-key global-map "\C-x/" 'point-to-register)
@@ -336,7 +337,7 @@ Keymap for characters following C-c.")
 (define-key global-map "\M-a" 'backward-sentence)
 (define-key global-map "\M-e" 'forward-sentence)
 (define-key global-map "\M-k" 'kill-sentence)
-(define-key global-map "\C-x\177" 'backward-kill-sentence)
+;;(define-key global-map "\C-x\177" 'backward-kill-sentence)
 
 (define-key global-map "\C-x[" 'backward-page)
 (define-key global-map "\C-x]" 'forward-page)
@@ -371,9 +372,8 @@ Keymap for characters following C-c.")
 
 ;; Default binding of "Backspace" is no longer the same as delete.
 ;; Default binding of "Control-h" is help.
-(define-key global-map 'backspace "\C-?")
-(define-key global-map '(meta backspace) "\e\C-?")
-
+(define-key global-map 'backspace 'delete-backward-char)
+(define-key global-map '(meta backspace) 'backward-kill-word)
 
 (define-key global-map "\M-\C-z" 'activate-region)
 
@@ -426,7 +426,7 @@ Keymap for characters following C-c.")
 ;; New FSF19 bindings
 (define-key global-map "\C-x-" 'shrink-window-if-larger-than-buffer)
 (define-key global-map "\C-x+" 'balance-windows)
-
+(define-key ctl-x-4-map "0" 'kill-buffer-and-window)
 
 ;;(define-key global-map "\C-g" 'keyboard-quit)
 (let ((ch (quit-char)))
@@ -460,9 +460,9 @@ Keymap for characters following C-c.")
 
 (define-key global-map "\M-$" 'ispell-word)
 
-(define-key global-map "\C-x4m" 'mail-other-window)
-
-(define-key global-map "\C-xm" 'mail)
+(define-key global-map "\C-xm" 'compose-mail)
+(define-key global-map "\C-x4m" 'compose-mail-other-window)
+(define-key global-map "\C-x5m" 'compose-mail-other-frame)
 
 (define-key global-map "\M-." 'find-tag)
 
