@@ -66,9 +66,9 @@ See `current-menubar' for a description of the syntax of a menubar."
     (setq menu (cdr menu)))
   (let (menuitem item)
     (while (keywordp (setq item (car menu)))
-      (or (memq item '(:config :included :filter))
+      (or (memq item '(:config :included :filter :accelerator))
 	  (signal 'error
-		  (list "menu keyword must be :config, :included, or :filter"
+		  (list "menu keyword must be :config, :included, :accelerator or :filter"
 			item)))
       (if (or (not (cdr menu))
 	      (vectorp (nth 1 menu))
@@ -133,7 +133,8 @@ See `current-menubar' for a description of the syntax of a menubar."
 		(while (< i L)
 		  (setq item (aref menuitem i))
 		  (cond ((not (memq item '(:active :suffix :keys :style
-						   :full :included :selected)))
+						   :full :included :selected
+						   :accelerator)))
 			 (signal 'error
 				 (list (if (keywordp item)
 					   "unknown menu item keyword"
@@ -168,7 +169,7 @@ If the item does not exist, the car of the returned value is nil.
 If some menu in the ITEM-PATH-LIST does not exist, an error is signalled."
   (or (listp item-path-list)
       (signal 'wrong-type-argument (list 'listp item-path-list)))
-  (or parent (setq item-path-list (mapcar 'downcase item-path-list)))
+  (or parent (setq item-path-list (mapcar 'normalize-menu-item-name item-path-list)))
   (if (not (consp menubar))
       nil
     (let ((rest menubar)
@@ -180,7 +181,7 @@ If some menu in the ITEM-PATH-LIST does not exist, an error is signalled."
       (while rest
 	(if (and (car rest)
 		 (equal (car item-path-list)
-			(downcase (if (vectorp (car rest))
+			(normalize-menu-item-name (if (vectorp (car rest))
 				      (aref (car rest) 0)
 				    (if (stringp (car rest))
 					(car rest)
@@ -198,7 +199,7 @@ If some menu in the ITEM-PATH-LIST does not exist, an error is signalled."
 (defun add-menu-item-1 (leaf-p menu-path new-item before)
   ;; This code looks like it could be cleaned up some more
   ;; Do we really need 6 calls to find-menu-item?
-  (when before (setq before (downcase before)))
+  (when before (setq before (normalize-menu-item-name before)))
   (let* ((item-name
 	  (cond ((vectorp new-item) (aref new-item 0))
 		((consp   new-item) (car  new-item))
@@ -491,5 +492,22 @@ MENU-DESC."
 	   (funcall (event-function response)
 		    (event-object response))))))
 
+;; provide default bindings for menu accelerator map
+(and (boundp 'menu-accelerator-map)
+     (keymapp menu-accelerator-map)
+     (progn
+       (define-key menu-accelerator-map "\e" 'menu-escape)
+       (define-key menu-accelerator-map [left] 'menu-left)
+       (define-key menu-accelerator-map [right] 'menu-right)
+       (define-key menu-accelerator-map [up] 'menu-up)
+       (define-key menu-accelerator-map [down] 'menu-down)
+       (define-key menu-accelerator-map [return] 'menu-select)
+       (define-key menu-accelerator-map [kp_down] 'menu-down)
+       (define-key menu-accelerator-map [kp_up] 'menu-down)
+       (define-key menu-accelerator-map [kp_left] 'menu-left)
+       (define-key menu-accelerator-map [kp_right] 'menu-right)
+       (define-key menu-accelerator-map [kp_enter] 'menu-select)
+       (define-key menu-accelerator-map "\C-g" 'menu-quit)))
+
 
 (provide 'menubar)
