@@ -32,7 +32,19 @@
 (global-set-key '(control shift button1) 'mouse-track-delete-and-insert)
 (global-set-key '(meta button1) 'mouse-track-do-rectangle)
 
-(global-set-key 'button2 'mouse-yank)
+;; enable drag regions (ograf@fga.de)
+;; if button2 is dragged from within a region, this becomes a drop
+(if (or (featurep 'offix) ;; do we have DnD support?
+	(featurep 'cde))
+    (global-set-key 'button2 'mouse-drag-or-yank)
+  (global-set-key 'button2 'mouse-yank))
+
+;; enable drops from OffiX (ograf@fga.de)
+;; accept any button1,2,3 drop with `mouse-offix-drop'
+(cond ((featurep 'offix)
+       (global-set-key 'drop1 'mouse-offix-drop)
+       (global-set-key 'drop2 'mouse-offix-drop)
+       (global-set-key 'drop3 'mouse-offix-drop)))
 
 (defcustom mouse-track-rectangle-p nil
   "*If true, then dragging out a region with the mouse selects rectangles
@@ -47,9 +59,16 @@ text is inserted."
   :type 'boolean
   :group 'mouse)
 
-(defvar mouse-yank-function 'yank	; x11/x-mouse changes this...
+(defvar mouse-yank-function 'mouse-consolidated-yank
   "Function that is called upon by `mouse-yank' to actually insert text.")
 
+(defun mouse-consolidated-yank ()
+  (interactive)
+  (case (device-type)
+    (x (x-yank-function))
+    (tty (yank))
+    (otherwise (yank))))
+
 
 (defun mouse-select ()
   "Select Emacs window the mouse is on."
@@ -134,16 +153,74 @@ primary selection-extent, nil otherwise."
   (or (point-inside-extent-p primary-selection-extent)
       (point-inside-extent-p zmacs-region-extent)))
 
-;;; #### - finish this...
-;;; (defun mouse-drag-or-yank (event)
-;;;   "Either drag or paste the current selection.  If the variable
-;;; `mouse-yank-at-point' is non-nil, then moves the cursor to the location of
-;;; the click before pasting."
-;;;   (interactive "e")
-;;;   (if (click-inside-selection-p event)
-;;;       ;; okay, this is a drag
-;;;       )
-;;;   )
+(defun mouse-drag-or-yank (event)
+  "Either drag or paste the current selection.  If the variable
+ `mouse-yank-at-point' is non-nil, then moves the cursor to the location of
+ the click before pasting.
+ This functions has to be improved. Until now it is just a (working) test."
+  ;; by Oliver Graf <ograf@fga.de>
+  (interactive "e")
+  (if (click-inside-extent-p event zmacs-region-extent)
+      ;; okay, this is a drag
+      (cond ((featurep 'offix)
+	     (offix-start-drag-region event
+				      (extent-start-position zmacs-region-extent)
+				      (extent-end-position zmacs-region-extent)))
+	    ((featurep 'cde)
+	     ;; should also work with CDE
+	     (cde-start-drag
+	      (extent-start-position zmacs-region-extent)
+	      (extent-end-position zmacs-region-extent)))
+	    (t ding))
+    ;; no drag, call region-funct
+    (and (not mouse-yank-at-point)
+	 (mouse-set-point event))
+    (funcall mouse-yank-function))
+  )
+
+(defun mouse-offix-drop (event)
+  "Do something with an OffiX drop event. Inserts Text drops and
+ executes appropriate commands for specific drops.
+ Text drops follow the `mouse-yank-at-point' variable."
+  ;; by Oliver Graf <ograf@fga.de>
+  (interactive "e")
+  (let ((type (car (event-drag-and-drop-data event)))
+	(data (cadr (event-drag-and-drop-data event)))
+	(frame (event-channel event)))
+    (cond ((= type 2)
+	   (let ((x pop-up-windows))
+	     (setq pop-up-windows nil)
+	     (pop-to-buffer (find-file-noselect data) nil frame)
+	     (make-frame-visible frame)
+	     (setq pop-up-windows x)))
+	  ((= type 3)
+	   (let ((x pop-up-windows))
+	     (setq pop-up-windows nil)
+	     (while (not (eq data ()))
+	       (pop-to-buffer (find-file-noselect (car data)) nil frame)
+	       (setq data (cdr data)))
+	     (make-frame-visible frame)
+	     (setq pop-up-windows x)))
+	  ((= type 4)
+	   (and (not mouse-yank-at-point)
+		(mouse-set-point event))
+	   (insert data))
+	  ((= type 5) (dired data))
+	  ((or (= type 6) (= type 7)) (dired data)) ;; this is junk
+	  ((= type 8) (funcall browse-url-browser-function data))
+	  ((= type 9)
+	   (let ((buf (generate-new-buffer "DndMIME")))
+	     (set-buffer buf)
+	     (pop-to-buffer buf nil frame)
+	     (insert data)
+	     (make-frame-visible frame)))
+	  (t ;; this is raw data or unknown stuff
+	   (let ((buf (generate-new-buffer "DndRawData")))
+	     (set-buffer buf)
+	     (pop-to-buffer buf nil frame)
+	     (insert data)
+	     (hexlify-buffer)
+	     (make-frame-visible frame))))))
 
 (defun mouse-eval-sexp (click force-window)
   "Evaluate the sexp under the mouse.  Usually, this is the last sexp before
@@ -699,14 +776,51 @@ at the initial click position."
 (defvar default-mouse-track-result nil)
 (defvar default-mouse-track-down-event nil)
 
+;; D. Verna Feb. 17 1998
+;; This function used to assume that when (event-window event) differs from 
+;; window, we have to scroll. This is WRONG, for instance when there are
+;; toolbars on the side, in which case window-event returns nil.
 (defun default-mouse-track-set-point-in-window (event window)
-  (if (not (and (not (event-over-modeline-p event))
-		(eq (event-window event) window)
-		(let ((p (event-closest-point event)))
-		  (and p (pos-visible-in-window-p p window)))))
-      nil
-    (mouse-set-point event)
-    t))
+  (if (event-over-modeline-p event)
+      nil ;; Scroll
+    ;; Not over a modeline
+    (if (eq (event-window event) window)
+	(let ((p (event-closest-point event)))
+	  (if  (or (not p) (not (pos-visible-in-window-p p window)))
+	      nil ;; Scroll
+	    (mouse-set-point event)
+	    t))
+      ;; Not over a modeline, not the same window. Check if the Y position
+      ;; is still overlapping the original window. 
+      (let* ((edges (window-pixel-edges window))
+	     (row (event-y-pixel event))
+	     (text-start (nth 1 edges))
+	     (text-end (+ (nth 3 edges))))
+	(if (or (< row text-start) 
+		(> row text-end))
+	    nil ;; Scroll
+	  ;; The Y pos in overlapping the original window. Check however if
+	  ;; the position is really visible, because there could be a 
+	  ;; scrollbar or a modeline at this place.
+	  ;; Find the mean line height (height / lines nb), and approximate
+	  ;; the line number for Y pos.
+	  (select-window window)
+	  (let ((line (/ (* (- row text-start) (window-height)) 
+			 (- text-end text-start))))
+	    (if (not (save-excursion
+		       (goto-char (window-start))
+		       (pos-visible-in-window-p
+			(point-at-bol (+ 1 line)))))
+		nil ;; Scroll
+	      ;; OK, we can go to that position
+	      (goto-char (window-start))
+	      (forward-line line)
+	      ;; On the right side: go to end-of-line.
+	      (when (>= (event-x-pixel event) (nth 2 edges))
+		(goto-char (point-at-eol)))
+	      t))))
+      )))
+
 
 (defun default-mouse-track-scroll-and-set-point (event window)
   (select-window window)
@@ -1081,7 +1195,7 @@ at the initial click position."
 			 (extent-detached-p x)))))
 	(extent default-mouse-track-extent))
     (if (consp extent)
-	(if (some dead-func extent)
+	(if (funcall dead-func extent)
 	    (let (newval)
 	      (mapcar (function (lambda (x)
 				  (if (not (funcall dead-func x))
