@@ -48,9 +48,15 @@ as provided for compatibility only."
 (defsubst define-obsolete-variable-alias (oldvar newvar)
   "Define OLDVAR as an obsolete alias for variable NEWVAR.
 This makes referencing or setting OLDVAR equivalent to referencing or
-setting NEWVAR and marks OLDVAR as obsolete."
-  (defvaralias oldvar newvar)
-  (make-obsolete-variable oldvar newvar))
+setting NEWVAR and marks OLDVAR as obsolete. 
+If OLDVAR was bound and NEWVAR was not, Set NEWVAR to OLDVAR.
+
+Note: Use this before any other references (defvar/defcustom) to NEWVAR"
+  (let ((needs-setting (and (boundp oldvar) (not (boundp newvar))))
+        (value (and (boundp oldvar) (symbol-value oldvar))))
+     (defvaralias oldvar newvar)
+     (make-obsolete-variable oldvar newvar)
+     (and needs-setting (set newvar value))))
 
 (defsubst define-compatible-variable-alias (oldvar newvar)
   "Define OLDVAR as a compatible alias for variable NEWVAR.
@@ -244,7 +250,7 @@ setting NEWVAR and marks OLDVAR as provided for compatibility only."
   "Returns the topmost, leftmost window of FRAME.
 If omitted, FRAME defaults to the currently selected frame."
   (frame-highest-window frame 0))
-(make-obsolete 'frame-first-window 'frame-highest-window)
+(make-compatible 'frame-first-window 'frame-highest-window)
 
 (define-obsolete-variable-alias 'initial-frame-alist 'initial-frame-plist)
 (define-obsolete-variable-alias 'minibuffer-frame-alist
@@ -321,8 +327,8 @@ See `set-frame-properties' for built-in property names."
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;; insertion and deletion
 
-(define-obsolete-function-alias 'insert-and-inherit 'insert)
-(define-obsolete-function-alias 'insert-before-markers-and-inherit
+(define-compatible-function-alias 'insert-and-inherit 'insert)
+(define-compatible-function-alias 'insert-before-markers-and-inherit
   'insert-before-markers)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;; keymaps
@@ -347,19 +353,20 @@ See `set-frame-properties' for built-in property names."
 (make-obsolete 'add-menu-item 'add-menu-button)
 
 (defun add-menu (menu-path menu-name menu-items &optional before)
-  "Obsolete.  See the function `add-submenu'."
+  "See the function `add-submenu'."
   (or menu-name (error (gettext "must specify a menu name")))
   (or menu-items (error (gettext "must specify some menu items")))
   (add-submenu menu-path (cons menu-name menu-items) before))
-(make-obsolete 'add-menu 'add-submenu)
+;; Can't make this obsolete.  easymenu depends on it.
+(make-compatible 'add-menu 'add-submenu)
 
 (define-obsolete-function-alias 'popup-menu-up-p 'popup-up-p)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;; minibuffer
 
-(define-obsolete-function-alias 'read-minibuffer
+(define-compatible-function-alias 'read-minibuffer
   'read-expression) ; misleading name
-(define-obsolete-function-alias 'read-input 'read-string)
+(define-compatible-function-alias 'read-input 'read-string)
 (make-obsolete 'read-no-blanks-input 'read-string) ; mocklisp crud
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;; misc
@@ -615,6 +622,78 @@ If FRAME is omitted or nil, use the selected frame."
 
 ;; Two loser functions which shouldn't be used.
 (make-obsolete 'following-char 'char-after)
-(make-obsolete 'preceding-char 'char-after)
+(make-obsolete 'preceding-char 'char-before)
+
+
+;; The following several functions are useful in GNU Emacs 20 because
+;; of the multibyte "characters" the internal representation of which
+;; leaks into Lisp.  In XEmacs/Mule they are trivial and unnecessary.
+;; We provide them for compatibility reasons solely.
+
+(defun string-to-sequence (string type)
+  "Convert STRING to a sequence of TYPE which contains characters in STRING.
+TYPE should be `list' or `vector'.
+Multibyte characters are concerned."
+  (ecase type
+    (list
+     (mapcar #'identity string))
+    (vector
+     (mapvector #'identity string))))
+
+(defun string-to-list (string)
+  "Return a list of characters in STRING."
+  (mapcar #'identity string))
+
+(defun string-to-vector (string)
+  "Return a vector of characters in STRING."
+  (mapvector #'identity string))
+
+(defun store-substring (string idx obj)
+  "Embed OBJ (string or character) at index IDX of STRING."
+  (let* ((str (cond ((stringp obj) obj)
+		    ((characterp obj) (char-to-string obj))
+		    (t (error
+			"Invalid argument (should be string or character): %s"
+			obj))))
+	 (string-len (length string))
+	 (len (length str))
+	 (i 0))
+    (while (and (< i len) (< idx string-len))
+      (aset string idx (aref str i))
+      (setq idx (1+ idx) i (1+ i)))
+    string))
+
+;; ### This function is not compatible with FSF in some cases.  Hard
+;; to fix, because it is hard to trace the logic of the FSF function.
+;; In case we need the exact behaviour, we can always copy the FSF
+;; version, which is very long and does lots of unnecessary stuff.
+(defun truncate-string-to-width (str end-column &optional start-column padding)
+  "Truncate string STR to end at column END-COLUMN.
+The optional 2nd arg START-COLUMN, if non-nil, specifies
+the starting column; that means to return the characters occupying
+columns START-COLUMN ... END-COLUMN of STR.
+
+The optional 3rd arg PADDING, if non-nil, specifies a padding character
+to add at the end of the result if STR doesn't reach column END-COLUMN,
+or if END-COLUMN comes in the middle of a character in STR.
+PADDING is also added at the beginning of the result
+if column START-COLUMN appears in the middle of a character in STR.
+
+If PADDING is nil, no padding is added in these cases, so
+the resulting string may be narrower than END-COLUMN."
+  (or start-column
+      (setq start-column 0))
+  (let ((len (length str)))
+    (concat (substring str (min start-column len) (min end-column len))
+	    (and padding (> end-column len)
+		 (make-string (- end-column len) padding)))))
+
+(defalias 'truncate-string 'truncate-string-to-width)
+(make-obsolete 'truncate-string 'truncate-string-to-width)
+
+;; Keywords already do The Right Thing in XEmacs
+(make-compatible 'define-widget-keywords "Just use them")
+
+(make-obsolete 'function-called-at-point 'function-at-point)
 
 ;;; obsolete.el ends here
