@@ -35,27 +35,28 @@
 
 ;;;; Lisp language features.
 
-(defmacro lambda (&rest cdr)
-  "Return a lambda expression.
-A call of the form (lambda ARGS DOCSTRING INTERACTIVE BODY) is
-self-quoting; the result of evaluating the lambda expression is the
-expression itself.  The lambda expression may then be treated as a
-function, i.e., stored as the function value of a symbol, passed to
-funcall or mapcar, etc.
+;; Moved to packages.el
+;(defmacro lambda (&rest cdr)
+;  "Return a lambda expression.
+;A call of the form (lambda ARGS DOCSTRING INTERACTIVE BODY) is
+;self-quoting; the result of evaluating the lambda expression is the
+;expression itself.  The lambda expression may then be treated as a
+;function, i.e., stored as the function value of a symbol, passed to
+;funcall or mapcar, etc.
 
-ARGS should take the same form as an argument list for a `defun'.
-DOCSTRING is an optional documentation string.
- If present, it should describe how to call the function.
- But documentation strings are usually not useful in nameless functions.
-INTERACTIVE should be a call to the function `interactive', which see.
-It may also be omitted.
-BODY should be a list of lisp expressions."
-  ;; Note that this definition should not use backquotes; subr.el should not
-  ;; depend on backquote.el.
-  ;; #### - I don't see why.  So long as backquote.el doesn't use anything
-  ;; from subr.el, there's no problem with using backquotes here.  --Stig 
-  ;;(list 'function (cons 'lambda cdr)))
-  `(function (lambda ,@cdr)))
+;ARGS should take the same form as an argument list for a `defun'.
+;DOCSTRING is an optional documentation string.
+; If present, it should describe how to call the function.
+; But documentation strings are usually not useful in nameless functions.
+;INTERACTIVE should be a call to the function `interactive', which see.
+;It may also be omitted.
+;BODY should be a list of lisp expressions."
+;  ;; Note that this definition should not use backquotes; subr.el should not
+;  ;; depend on backquote.el.
+;  ;; #### - I don't see why.  So long as backquote.el doesn't use anything
+;  ;; from subr.el, there's no problem with using backquotes here.  --Stig 
+;  ;;(list 'function (cons 'lambda cdr)))
+;  `(function (lambda ,@cdr)))
 
 (defmacro defun-when-void (&rest args)
   "Define a function, just like `defun', unless it's already defined.
@@ -85,6 +86,17 @@ Used for compatibility among different emacs variants."
 ;; The call to `read' is to ensure that the value is computed at load time
 ;; and not compiled into the .elc file.  The value is negative on most
 ;; machines, but not on all!
+(defconst listify-key-sequence-1 (logior 128 (read "?\\M-\\^@")))
+
+(defun listify-key-sequence (key)
+  "Convert a key sequence to a list of events."
+  (if (vectorp key)
+      (append key nil)
+    (mapcar (function (lambda (c)
+			(if (> c 127)
+			    (logxor c listify-key-sequence-1)
+			  c)))
+	    (append key nil))))
 ;; XEmacs: This stuff is done in C Code.
 
 ;;;; Obsolescent names for functions.
@@ -235,40 +247,29 @@ The buffer about to be killed is current when this hook is run.")
 just before emacs is actually killed.")
 
 ;; not obsolete.
+;; #### These are a bad idea, because the CL RPLACA and RPLACD
+;; return the cons cell, not the new CAR/CDR.         -hniksic
+;; The proper definition would be:
+;; (defun rplaca (conscell newcar)
+;;   (setcar conscell newcar)
+;;   conscell)
+;; ...and analogously for RPLACD.
 (define-function 'rplaca 'setcar)
 (define-function 'rplacd 'setcdr)
 
-;; XEmacs
-(defun mapvector (__function __seq)
-  "Apply FUNCTION to each element of SEQ, making a vector of the results.
-The result is a vector of the same length as SEQ.
-SEQ may be a list, a vector or a string."
-  (let* ((len (length __seq))
-	 (vec (make-vector len 'nil))
-	 (i 0))
-    (while (< i len)
-      (aset vec i (funcall __function (cond ((listp __seq)
-					     (nth i __seq))
-					    (t (aref __seq i)))))
-      (setq i (+ i 1)))
-    vec))
-
 ;;;; String functions.
 
 ;; XEmacs
 (defun replace-in-string (str regexp newtext &optional literal)
-  "Replaces all matches in STR for REGEXP with NEWTEXT string.
+  "Replaces all matches in STR for REGEXP with NEWTEXT string,
+ and returns the new string.
 Optional LITERAL non-nil means do a literal replacement.
 Otherwise treat \\ in NEWTEXT string as special:
   \\& means substitute original matched text,
   \\N means substitute match for \(...\) number N,
   \\\\ means insert one \\."
-  (if (not (stringp str))
-      (error "(replace-in-string): First argument must be a string: %s" str))
-  (if (stringp newtext)
-      nil
-    (error "(replace-in-string): 3rd arg must be a string: %s"
-	   newtext))
+  (check-argument-type 'stringp str)
+  (check-argument-type 'stringp newtext)
   (let ((rtn-str "")
 	(start 0)
 	(special)
@@ -282,63 +283,92 @@ Otherwise treat \\ in NEWTEXT string as special:
 	      (substring str prev-start match)
 	      (cond (literal newtext)
 		    (t (mapconcat
-			 (function
-			   (lambda (c)
-			     (if special
-				 (progn
-				   (setq special nil)
-				   (cond ((eq c ?\\) "\\")
-					 ((eq c ?&)
-					  (substring str
-						     (match-beginning 0)
-						     (match-end 0)))
-					 ((and (>= c ?0) (<= c ?9))
-					  (if (> c (+ ?0 (length
-							   (match-data))))
-					      ;; Invalid match num
-					      (error "(replace-in-string) Invalid match num: %c" c)
-					    (setq c (- c ?0))
-					    (substring str
-						       (match-beginning c)
-						       (match-end c))))
-					 (t (char-to-string c))))
-			       (if (eq c ?\\) (progn (setq special t) nil)
-				 (char-to-string c)))))
+			(lambda (c)
+			  (if special
+			      (progn
+				(setq special nil)
+				(cond ((eq c ?\\) "\\")
+				      ((eq c ?&)
+				       (substring str
+						  (match-beginning 0)
+						  (match-end 0)))
+				      ((and (>= c ?0) (<= c ?9))
+				       (if (> c (+ ?0 (length
+						       (match-data))))
+					   ;; Invalid match num
+					   (error "Invalid match num: %c" c)
+					 (setq c (- c ?0))
+					 (substring str
+						    (match-beginning c)
+						    (match-end c))))
+				      (t (char-to-string c))))
+			    (if (eq c ?\\) (progn (setq special t) nil)
+			      (char-to-string c))))
 			 newtext ""))))))
     (concat rtn-str (substring str start))))
 
-(defun split-string (string pattern)
-  "Return a list of substrings of STRING which are separated by PATTERN."
+(defun split-string (string &optional pattern)
+  "Return a list of substrings of STRING which are separated by PATTERN.
+If PATTERN is omitted, it defaults to \"[ \\f\\t\\n\\r\\v]+\"."
+  (or pattern
+      (setq pattern "[ \f\t\n\r\v]+"))
+  ;; The FSF version of this function takes care not to cons in case
+  ;; of infloop.  Maybe we should synch?
   (let (parts (start 0))
     (while (string-match pattern string start)
       (setq parts (cons (substring string start (match-beginning 0)) parts)
 	    start (match-end 0)))
-    (nreverse (cons (substring string start) parts))
-    ))
+    (nreverse (cons (substring string start) parts))))
 
 (defmacro with-output-to-string (&rest forms)
   "Collect output to `standard-output' while evaluating FORMS and return
 it as a string."
   ;; by "William G. Dubuque" <wgd@zurich.ai.mit.edu> w/ mods from Stig
-  (` (save-excursion
-       (set-buffer (get-buffer-create " *string-output*"))
-       (setq buffer-read-only nil)
-       (buffer-disable-undo (current-buffer))
-       (erase-buffer)
-       (let ((standard-output (current-buffer)))
-	 (,@ forms))
-       (prog1
-	   (buffer-string)
-	 (erase-buffer)))))
+  `(with-current-buffer (get-buffer-create " *string-output*")
+     (setq buffer-read-only nil)
+     (buffer-disable-undo (current-buffer))
+     (erase-buffer)
+     (let ((standard-output (current-buffer)))
+       ,@forms)
+     (prog1
+	 (buffer-string)
+       (erase-buffer))))
+
+(defmacro with-current-buffer (buffer &rest body)
+  "Execute the forms in BODY with BUFFER as the current buffer.
+The value returned is the value of the last form in BODY.
+See also `with-temp-buffer'."
+  `(save-current-buffer
+    (set-buffer ,buffer)
+    ,@body))
+
+(defmacro with-temp-file (file &rest forms)
+  "Create a new buffer, evaluate FORMS there, and write the buffer to FILE.
+The value of the last form in FORMS is returned, like `progn'.
+See also `with-temp-buffer'."
+  (let ((temp-file (make-symbol "temp-file"))
+	(temp-buffer (make-symbol "temp-buffer")))
+    `(let ((,temp-file ,file)
+	   (,temp-buffer
+	    (get-buffer-create (generate-new-buffer-name " *temp file*"))))
+       (unwind-protect
+	   (prog1
+	       (with-current-buffer ,temp-buffer
+		 ,@forms)
+	     (with-current-buffer ,temp-buffer
+               (widen)
+	       (write-region (point-min) (point-max) ,temp-file nil 0)))
+	 (and (buffer-name ,temp-buffer)
+	      (kill-buffer ,temp-buffer))))))
 
 (defmacro with-temp-buffer (&rest forms)
-  "Create a temporary buffer, and evaluate FORMS there like `progn'."
+  "Create a temporary buffer, and evaluate FORMS there like `progn'.
+See also `with-temp-file' and `with-output-to-string'."
   (let ((temp-buffer (make-symbol "temp-buffer")))
     `(let ((,temp-buffer
 	    (get-buffer-create (generate-new-buffer-name " *temp*"))))
        (unwind-protect
-	   (save-excursion
-	     (set-buffer ,temp-buffer)
+	   (with-current-buffer ,temp-buffer
 	     ,@forms)
 	 (and (buffer-name ,temp-buffer)
 	      (kill-buffer ,temp-buffer))))))
@@ -348,18 +378,16 @@ it as a string."
   "With the contents of the current buffer being STR, run BODY.
 Returns the new contents of the buffer, as modified by BODY.
 The original current buffer is restored afterwards."
-  `(let ((curbuf (current-buffer))
-         (tempbuf (get-buffer-create " *string-as-buffer-contents*")))
-     (unwind-protect
-         (progn
-           (set-buffer tempbuf)
-           (buffer-disable-undo (current-buffer))
-           (erase-buffer)
-           (insert ,str)
-           ,@body
-           (buffer-string))
-       (erase-buffer tempbuf)
-       (set-buffer curbuf))))
+  `(let ((tempbuf (get-buffer-create " *string-as-buffer-contents*")))
+     (with-current-buffer tempbuf
+       (unwind-protect
+	   (progn
+	     (buffer-disable-undo (current-buffer))
+	     (erase-buffer)
+	     (insert ,str)
+	     ,@body
+	     (buffer-string))
+	 (erase-buffer tempbuf)))))
 
 (defun insert-face (string face)
   "Insert STRING and highlight with FACE.  Returns the extent created."
@@ -514,29 +542,28 @@ yourself.]"
     (set-text-properties 0 (length string) nil string)
     string))
 
+;; This should probably be written in C (i.e., without using `walk-windows').
+(defun get-buffer-window-list (buffer &optional minibuf frame)
+  "Return windows currently displaying BUFFER, or nil if none.
+See `walk-windows' for the meaning of MINIBUF and FRAME."
+  (let ((buffer (if (bufferp buffer) buffer (get-buffer buffer))) windows)
+    (walk-windows (function (lambda (window)
+			      (if (eq (window-buffer window) buffer)
+				  (setq windows (cons window windows)))))
+		  minibuf frame)
+    windows))
+
 (defun ignore (&rest ignore)
   "Do nothing and return nil.
 This function accepts any number of arguments, but ignores them."
   (interactive)
   nil)
 
-(defmacro save-current-buffer (&rest forms)
-  "Restore the current buffer setting after executing FORMS.
-Does not restore the values of point and mark.
-See also: `save-excursion'."
-  ;; by Stig@hackvan.com
-  (` (let ((_cur_buf_ (current-buffer)))
-       (unwind-protect
-	   (progn (,@ forms))
-	 (set-buffer _cur_buf_)))))
+(define-function 'mapc-internal 'mapc)
+(make-obsolete 'mapc-internal 'mapc)
 
-(defmacro eval-in-buffer (buffer &rest forms)
-  "Evaluate FORMS in BUFFER.
-See also: `save-current-buffer' and `save-excursion'."
-  ;; by Stig@hackvan.com
-  (` (save-current-buffer
-      (set-buffer (, buffer))
-      (,@ forms))))
+(define-function 'eval-in-buffer 'with-current-buffer)
+(make-obsolete 'eval-in-buffer 'with-current-buffer)
 
 ;;; The real defn is in abbrev.el but some early callers
 ;;;  (eg lisp-mode-abbrev-table) want this before abbrev.el is loaded...
@@ -550,15 +577,12 @@ See also: `save-current-buffer' and `save-excursion'."
                                          (cons (cons name defs)
                                                abbrev-table-name-list)))))))
 
-(defun functionp (obj)
-  "Returns t if OBJ is a function, nil otherwise."
-  (cond
-   ((symbolp obj) (fboundp obj))
-   ((subrp obj))
-   ((compiled-function-p obj))
-   ((consp obj)
-    (if (eq (car obj) 'lambda) (listp (car (cdr obj)))))
-   (t nil)))
+(defun functionp (object)
+  "Non-nil if OBJECT can be called as a function."
+  (or (and (symbolp object) (fboundp object))
+      (subrp object)
+      (compiled-function-p object)
+      (eq (car-safe object) 'lambda)))
 
 ;; This was not present before.  I think Jamie had some objections
 ;; to this, so I'm leaving this undefined for now. --ben
@@ -570,6 +594,9 @@ See also: `save-current-buffer' and `save-excursion'."
 ;;; it should work off of a hook on `provide'.  Features are unique and
 ;;; the arguments to (load) are not.  --Stig
 
+;; We provide this for FSFmacs compatibility, at least until we devise
+;; something better.
+
 ;;;; Specifying things to do after certain files are loaded.
 
 (defun eval-after-load (file form)
