@@ -37,7 +37,7 @@
 ;;;; Variable declarations
 
 (defvar sgml-display-char-list-filename
-  (expand-file-name "sgml/iso88591.map" data-directory)
+  (concat (locate-data-directory "sgml") "iso88591.map")
   "*Name of file holding relations between character codes and character
 names of displayable characters")
 
