@@ -1,9 +1,9 @@
 ;;; tm-def.el --- definition module for tm
 
-;; Copyright (C) 1995,1996 Free Software Foundation, Inc.
+;; Copyright (C) 1995,1996,1997 Free Software Foundation, Inc.
 
 ;; Author: MORIOKA Tomohiko <morioka@jaist.ac.jp>
-;; Version: $Id: tm-def.el,v 7.69 1997/01/21 06:13:15 shuhei-k Exp $
+;; Version: $Id: tm-def.el,v 7.74 1997/04/05 06:42:32 morioka Exp $
 ;; Keywords: mail, news, MIME, multimedia, definition
 
 ;; This file is part of tm (Tools for MIME).
@@ -100,8 +100,8 @@
 ;;;
 
 (defun tm:set-face-region (b e face)
-  (let ((overlay (tl:make-overlay b e)))
-    (tl:overlay-put overlay 'face face)
+  (let ((overlay (make-overlay b e)))
+    (overlay-put overlay 'face face)
     ))
 
 (defvar tm:button-face 'bold
@@ -117,13 +117,13 @@
 (defun tm:add-button (from to func &optional data)
   "Create a button between FROM and TO with callback FUNC and data DATA."
   (and tm:button-face
-       (tl:overlay-put (tl:make-overlay from to) 'face tm:button-face))
-  (tl:add-text-properties from to
-			  (append (and tm:mouse-face
-				       (list 'mouse-face tm:mouse-face))
-				  (list 'tm-callback func)
-				  (and data (list 'tm-data data))
-				  ))
+       (overlay-put (make-overlay from to) 'face tm:button-face))
+  (add-text-properties from to
+		       (append (and tm:mouse-face
+				    (list 'mouse-face tm:mouse-face))
+			       (list 'tm-callback func)
+			       (and data (list 'tm-data data))
+			       ))
   )
 
 (defvar tm:mother-button-dispatcher nil)
@@ -170,6 +170,46 @@
     ))
 
 
+;;; @ PGP
+;;;
+
+(defvar pgp-function-alist
+  '(
+    ;; for tm-pgp
+    (verify		mc-verify			"mc-toplev")
+    (decrypt		mc-decrypt			"mc-toplev")
+    (fetch-key		mc-pgp-fetch-key		"mc-pgp")
+    (snarf-keys		mc-snarf-keys			"mc-toplev")
+    ;; for tm-edit
+    (mime-sign		tm:mc-pgp-sign-region		"tm-edit-mc")
+    (traditional-sign	mc-pgp-sign-region		"mc-pgp")
+    (encrypt		tm:mc-pgp-encrypt-region	"tm-edit-mc")
+    (insert-key		mc-insert-public-key		"mc-toplev")
+    )
+  "Alist of service names vs. corresponding functions and its filenames.
+Each element looks like (SERVICE FUNCTION FILE).
+
+SERVICE is a symbol of PGP processing.  It allows `verify', `decrypt',
+`fetch-key', `snarf-keys', `mime-sign', `traditional-sign', `encrypt'
+or `insert-key'.
+
+Function is a symbol of function to do specified SERVICE.
+
+FILE is string of filename which has definition of corresponding
+FUNCTION.")
+
+(defmacro pgp-function (method)
+  "Return function to do service METHOD."
+  (` (car (cdr (assq (, method) (symbol-value 'pgp-function-alist)))))
+  )
+
+(mapcar (function
+	 (lambda (method)
+	   (autoload (second method)(third method))
+	   ))
+	pgp-function-alist)
+
+
 ;;; @ definitions about MIME
 ;;;
 
