@@ -1,7 +1,8 @@
 ;;; autoload.el --- maintain autoloads in loaddefs.el.
-;;; Copyright (C) 1991, 1992, 1993, 1994 Free Software Foundation, Inc.
-;;; Copyright (C) 1995 Tinker Systems and INS Engineering Corp.
-;;; Copyright (C) 1996 Ben Wing.
+
+;; Copyright (C) 1991, 1992, 1993, 1994, 1997 Free Software Foundation, Inc.
+;; Copyright (C) 1995 Tinker Systems and INS Engineering Corp.
+;; Copyright (C) 1996 Ben Wing.
 
 ;; Author: Roland McGrath <roland@gnu.ai.mit.edu>
 ;; Keywords: maint
@@ -32,6 +33,10 @@
 ;; lisp source files in various useful ways.  To learn more, read the
 ;; source; if you're going to use this, you'd better be able to.
 
+;; ChangeLog:
+
+;; Sep-26-1997:  slb removed code dealing with customization.
+
 ;;; Code:
 
 (defun make-autoload (form file)
@@ -272,7 +277,7 @@ are used."
 			    ))
 		      ;; Copy the rest of the line to the output.
 		      (let ((begin (point)))
-			(terpri outbuf)
+			;; (terpri outbuf)
 			(cond ((looking-at "immediate\\s *$") ; XEmacs
 			       ;; This is here so that you can automatically
 			       ;; have small hook functions copied to
@@ -294,11 +299,9 @@ are used."
 		    (forward-line 1)))
 		  (if dofiles
 		      (setq funlist (cdr funlist)))))))
-	;;(unless visited
+	(unless visited
 	    ;; We created this buffer, so we should kill it.
-	    ;; Customize needs it later, we don't want to read the file
-	    ;; in twice.
-	    ;;(kill-buffer (current-buffer)))
+	    (kill-buffer (current-buffer)))
 	(set-buffer outbuf)
 	(setq output-end (point-marker))))
     (if t ;; done-any
@@ -316,13 +319,15 @@ are used."
 	  (when (< output-end (point))
 	    (setq output-end (point-marker)))
 	  (while (< (point) output-end)
-	    (let ((beg (point)))
+	    ;; (let ((beg (point)))
 	      (end-of-line)
-	      (if (> (- (point) beg) 900)
-		  (progn
-		    (message "A line is too long--over 900 characters")
-		    (sleep-for 2)
-		    (goto-char output-end))))
+	      ;; Emacs -- I still haven't figured this one out.
+	      ;; (if (> (- (point) beg) 900)
+		  ;; (progn
+		    ;; (message "A line is too long--over 900 characters")
+		    ;; (sleep-for 2)
+		    ;; (goto-char output-end)))
+	      ;; )
 	    (forward-line 1))
 	  (goto-char output-end)
 	  (insert generate-autoload-section-trailer)))
@@ -330,80 +335,72 @@ are used."
 	(message "Generating autoloads for %s...done" file))))
 
 
+(defconst autoload-file-name "auto-autoloads.el"
+  "Generic filename to put autoloads into.
+Unless you are an XEmacs maintainer, it is probably unwise to change this.")
+
+(defvar autoload-target-directory "../lisp/prim/"
+  "Directory to put autoload declaration file into.
+Unless you know what you're doing, don't mess with this.")
+
 (defvar generated-autoload-file
-  (expand-file-name "../lisp/prim/auto-autoloads.el" data-directory)
+  (expand-file-name (concat autoload-target-directory
+			    autoload-file-name)
+		    data-directory)
   "*File `update-file-autoloads' puts autoloads into.
 A .el file can set this in its local variables section to make its
 autoloads go somewhere else.")
 
-(defvar generated-custom-file
-  (expand-file-name "../lisp/prim/custom-load.el" data-directory)
-  "*File `update-file-autoloads' puts customization into.")
-
-;; Written by Per Abrahamsen
-(defun autoload-snarf-defcustom (file)
-  "Snarf all customizations in the current buffer."
-  (let ((visited (get-file-buffer file)))
-    (save-excursion
-      (set-buffer (or visited (find-file-noselect file)))
-      (when (and file
-		 (string-match "\\`\\(.*\\)\\.el\\'" file)
-		 (not (buffer-modified-p)))
-	(goto-char (point-min))
-	(condition-case nil
-	    (let ((name (file-name-nondirectory (match-string 1 file))))
-	      (while t
-		(let ((expr (read (current-buffer))))
-		  (when (and (listp expr)
-			     (memq (car expr) '(defcustom defface defgroup)))
-		    (eval expr)
-		    (put (nth 1 expr) 'custom-where name)))))
-	  (error nil)))
-      (unless (buffer-modified-p)
-	(kill-buffer (current-buffer))))))
+(defconst cusload-file-name "custom-load.el"
+  "Generic filename ot put custom loads into.
+Unless you are an XEmacs maintainr, it is probably unwise to change this.")
 
 ;;;###autoload
 (defun update-file-autoloads (file)
   "Update the autoloads for FILE in `generated-autoload-file'
-\(which FILE might bind in its local variables)."
+\(which FILE might bind in its local variables).
+This functions refuses to update autoloads files."
   (interactive "fUpdate autoloads for file: ")
   (setq file (expand-file-name file))
-  (let ((load-name (replace-in-string (file-name-nondirectory file)
-				      "\\.elc?$"
-				      ""))
-	(trim-name (autoload-trim-file-name file))
-	section-begin form)
-    (save-excursion
-      (let ((find-file-hooks nil))
-	(set-buffer (or (get-file-buffer generated-autoload-file)
-			(find-file-noselect generated-autoload-file))))
-      ;; First delete all sections for this file.
-      (goto-char (point-min))
-      (while (search-forward generate-autoload-section-header nil t)
-	(setq section-begin (match-beginning 0))
-	(setq form (read (current-buffer)))
-	(when (string= (nth 2 form) load-name)
-	  (search-forward generate-autoload-section-trailer)
-	  (delete-region section-begin (point))))
+  (when (and (file-newer-than-file-p file generated-autoload-file)
+	     (not (member (file-name-nondirectory file)
+			  (list autoload-file-name))))
 
-      ;; Now find insertion point for new section
-      (block find-insertion-point
+    (let ((load-name (replace-in-string (file-name-nondirectory file)
+					"\\.elc?$"
+					""))
+	  (trim-name (autoload-trim-file-name file))
+	  section-begin form)
+      (save-excursion
+	(let ((find-file-hooks nil))
+	  (set-buffer (or (get-file-buffer generated-autoload-file)
+			  (find-file-noselect generated-autoload-file))))
+	;; First delete all sections for this file.
 	(goto-char (point-min))
 	(while (search-forward generate-autoload-section-header nil t)
+	  (setq section-begin (match-beginning 0))
 	  (setq form (read (current-buffer)))
-	  (when (string< trim-name (nth 3 form))
-	    ;; Found alphabetically correct insertion point
-	    (goto-char (match-beginning 0))
-	    (return-from find-insertion-point))
-	  (search-forward generate-autoload-section-trailer))
-	(when (eq (point) (point-min))	; No existing entries?
-	  (goto-char (point-max))))	; Append.
+	  (when (string= (nth 2 form) load-name)
+	    (search-forward generate-autoload-section-trailer)
+	    (delete-region section-begin (point))))
 
-      ;; Add in new sections for file
-      (generate-file-autoloads file)
-      (autoload-snarf-defcustom file))
+	;; Now find insertion point for new section
+	(block find-insertion-point
+	  (goto-char (point-min))
+	  (while (search-forward generate-autoload-section-header nil t)
+	    (setq form (read (current-buffer)))
+	    (when (string< trim-name (nth 3 form))
+	      ;; Found alphabetically correct insertion point
+	      (goto-char (match-beginning 0))
+	      (return-from find-insertion-point))
+	    (search-forward generate-autoload-section-trailer))
+	  (when (eq (point) (point-min))	; No existing entries?
+	    (goto-char (point-max))))	; Append.
 
-    (when (interactive-p) (save-buffer))))
+	;; Add in new sections for file
+	(generate-file-autoloads file))
+
+      (when (interactive-p) (save-buffer)))))
 
 ;;;###autoload
 (defun update-autoloads-here ()
@@ -489,44 +486,20 @@ Obsolete autoload entries for files that no longer exist are deleted."
       (unless noninteractive
 	(save-buffer)))))
 
-;; Based on code from Per Abrahamsen
-(defun autoload-save-customization ()
-  (save-excursion
-    (set-buffer (find-file-noselect generated-custom-file))
-    (erase-buffer)
-    (insert
-     (with-output-to-string
-      (mapatoms (lambda (symbol)
-		  (let ((members (get symbol 'custom-group))
-			item where found)
-		    (when members
-		      (princ "(put '")
-		      (princ symbol)
-		      (princ " 'custom-loads '(")
-		      (while members
-			(setq item (car (car members))
-			      members (cdr members)
-			      where (get item 'custom-where))
-			(unless (or (null where)
-				    (member where found))
-			  (when found
-			    (princ " "))
-			  (prin1 where)
-			  (push where found)))
-		      (princ "))\n")))))))))
-
 ;;;###autoload
 (defun batch-update-autoloads ()
   "Update the autoloads for the files or directories on the command line.
 Runs `update-file-autoloads' on files and `update-directory-autoloads'
 on directories.  Must be used only with -batch, and kills Emacs on completion.
 Each file will be processed even if an error occurred previously.
-For example, invoke `xemacs -batch -f batch-update-autoloads *.el'."
+For example, invoke `xemacs -batch -f batch-update-autoloads *.el'.
+The directory to which the auto-autoloads.el file must be the first parameter
+on the command line."
   (unless noninteractive
     (error "batch-update-autoloads is to be used only with -batch"))
   (let ((defdir default-directory)
 	(enable-local-eval nil))	; Don't query in batch mode.
-    (message "Updating autoloads in %s..." generated-autoload-file)
+    ;; (message "Updating autoloads in %s..." generated-autoload-file)
     (dolist (arg command-line-args-left)
       (setq arg (expand-file-name arg defdir))
       (cond
@@ -536,11 +509,56 @@ For example, invoke `xemacs -batch -f batch-update-autoloads *.el'."
        ((file-exists-p arg)
 	(update-file-autoloads arg))
        (t (error "No such file or directory: %s" arg))))
-    (autoload-save-customization)
+    (fixup-autoload-buffer (concat (if autoload-package-name
+				       autoload-package-name
+				     (file-name-nondirectory defdir))
+				   "-autoloads"))
     (save-some-buffers t)
-    (message "Done")
+    ;; (message "Done")
     (kill-emacs 0)))
 
+(defun fixup-autoload-buffer (sym)
+  (save-excursion
+    (set-buffer (find-file-noselect generated-autoload-file))
+    (goto-char (point-min))
+    (if (and (not (= (point-min) (point-max)))
+	     (not (looking-at ";;; DO NOT MODIFY THIS FILE")))
+	(progn
+	  (insert ";;; DO NOT MODIFY THIS FILE\n")
+	  (insert "(if (featurep '" sym ")")
+	  (insert " (error \"Already loaded\"))\n")
+	  (goto-char (point-max))
+	  (insert "\n(provide '" sym ")\n")))))
+
+(defvar autoload-package-name nil)
+
+;;;###autoload
+(defun batch-update-directory ()
+  "Update the autoloads for the directory on the command line.
+Runs `update-file-autoloads' on each file in the given directory, must
+be used only with -batch and kills XEmacs on completion."
+  (unless noninteractive
+    (error "batch-update-directory is to be used only with -batch"))
+  (let ((defdir default-directory)
+	(enable-local-eval nil))	; Don't query in batch mode.
+    (dolist (arg command-line-args-left)
+      (setq arg (expand-file-name arg defdir))
+      (let ((generated-autoload-file (concat arg "/" autoload-file-name)))
+	(cond
+	 ((file-directory-p arg)
+	  (message "Updating autoloads in directory %s..." arg)
+	  (update-autoloads-from-directory arg))
+	 (t (error "No such file or directory: %s" arg)))
+	(fixup-autoload-buffer (concat (if autoload-package-name
+					   autoload-package-name
+					 (file-name-nondirectory arg))
+				"-autoloads"))
+	(save-some-buffers t))
+      ;; (message "Done")
+      ;; (kill-emacs 0)
+      )
+    (setq command-line-args-left nil)))
+
 (provide 'autoload)
 
 ;;; autoload.el ends here
