@@ -178,7 +178,9 @@ The new version of the list, minus the deleted strings, is returned."
 	(set-buffer where)
 	(goto-char (point-max))
 	(insert string))
-    (let ((temp-buffer nil))
+    (let ((temp-buffer nil)
+	  (coding-system-for-read 'no-conversion)
+	  (coding-system-for-write 'no-conversion))
       (unwind-protect
 	  (save-excursion
 	    (setq temp-buffer (generate-new-buffer "*vm-work*"))
@@ -339,6 +341,16 @@ The new version of the list, minus the deleted strings, is returned."
 (defun vm-delete-auto-save-file-names (list)
   (vm-delete 'auto-save-file-name-p list))
 
+(defun vm-delete-index-file-names (list)
+  (vm-delete 'vm-index-file-name-p list))
+
+(defun vm-index-file-name-p (file)
+  (and (file-regular-p file)
+       (stringp vm-index-file-suffix)
+       (let ((str (concat (regexp-quote vm-index-file-suffix) "$")))
+	 (string-match str file))
+       t ))
+
 (defun vm-delete-duplicates (list &optional all hack-addresses)
   "Delete duplicate equivalent strings from the list.
 If ALL is t, then if there is more than one occurrence of a string in the list,
@@ -423,31 +435,30 @@ If HACK-ADDRESSES is t, then the strings are considered to be mail addresses,
 	((markerp object) (copy-marker object))
 	(t object)))
 
-;; make-frame might be defined and still not work.  This would
-;; be true since the user could be running on a tty and using
-;; XEmacs 19.12, or using FSF Emacs 19.28 (or prior FSF Emacs versions).
-;;
-;; make-frame works on ttys in FSF Emacs 19.29, but other than
-;; looking at the version number I don't know a sane way to
-;; test for it without just running make-frame.  I'll just
-;; let it not work for now... someone will complain eventually
-;; and I'll think of something.
-
 (defun vm-multiple-frames-possible-p () 
   (cond (vm-xemacs-p 
-         (eq (device-type) 'x)) 
-        (vm-fsfemacs-19-p 
-         (not (eq window-system nil))))) 
+	 (or (memq 'win (device-matching-specifier-tag-list))
+	     (featurep 'tty-frames)))
+        (vm-fsfemacs-p 
+         (fboundp 'make-frame))))
  
 (defun vm-mouse-support-possible-p () 
-  (vm-multiple-frames-possible-p)) 
+  (cond (vm-xemacs-p 
+         (featurep 'window-system)) 
+        (vm-fsfemacs-p 
+         (fboundp 'track-mouse))))
  
+(defun vm-mouse-support-possible-here-p ()
+  (cond (vm-xemacs-p
+	 (memq 'win (device-matching-specifier-tag-list)))
+	(vm-fsfemacs-p
+	 (eq window-system 'x))))
+
 (defun vm-menu-support-possible-p ()
   (cond (vm-xemacs-p
 	 (featurep 'menubar))
-	(vm-fsfemacs-19-p
-	 (fboundp 'menu-bar-mode))
-	(t nil)))
+	(vm-fsfemacs-p
+	 (fboundp 'menu-bar-mode))))
  
 (defun vm-toolbar-support-possible-p ()
   (and vm-xemacs-p (featurep 'toolbar)))
@@ -455,8 +466,9 @@ If HACK-ADDRESSES is t, then the strings are considered to be mail addresses,
 (defun vm-multiple-fonts-possible-p ()
   (cond (vm-xemacs-p
 	 (eq (device-type) 'x))
-	(vm-fsfemacs-19-p
-	 (eq window-system 'x))))
+	(vm-fsfemacs-p
+	 (or (eq window-system 'x)
+	     (eq window-system 'win32)))))
 
 (defun vm-run-message-hook (message &optional hook-variable)
   (save-excursion
@@ -652,12 +664,19 @@ If HACK-ADDRESSES is t, then the strings are considered to be mail addresses,
 (defun vm-make-tempfile-name ()
   (let ((done nil) (pid (emacs-pid)) filename)
     (while (not done)
-      (setq filename (format "%s/vm%d.%d" vm-temp-file-directory pid
-			     vm-tempfile-counter)
+      (setq filename (expand-file-name (format "vm%d.%d" pid
+					       vm-tempfile-counter)
+				       vm-temp-file-directory)
 	    vm-tempfile-counter (1+ vm-tempfile-counter)
 	    done (not (file-exists-p filename))))
     filename ))
 
+(defun vm-make-work-buffer (&optional name)
+  (let (work-buffer)
+    (setq work-buffer (generate-new-buffer (or name "*vm-workbuf*")))
+    (buffer-disable-undo work-buffer)
+    work-buffer ))
+
 (defun vm-insert-char (char &optional count ignored buffer)
   (condition-case nil
       (progn
@@ -703,35 +722,6 @@ If HACK-ADDRESSES is t, then the strings are considered to be mail addresses,
 	  (buffer-string))
       (and work-buffer (kill-buffer work-buffer)))))
 
-(defmacro vm-with-virtual-selector-variables (&rest forms)
-  (append '(let ((any 'vm-vs-any)
-		 (and 'vm-vs-and)
-		 (or 'vm-vs-or)
-		 (not 'vm-vs-not)
-		 (header 'vm-vs-header)
-		 (label 'vm-vs-label)
-		 (text 'vm-vs-text)
-		 (recipient 'vm-vs-recipient)
-		 (author 'vm-vs-author)
-		 (subject 'vm-vs-subject)
-		 (sent-before 'vm-vs-sent-before)
-		 (sent-after 'vm-vs-sent-after)
-		 (more-chars-than 'vm-vs-more-chars-than)
-		 (less-chars-than 'vm-vs-less-chars-than)
-		 (more-lines-than 'vm-vs-more-lines-than)
-		 (less-lines-than 'vm-vs-less-lines-than)
-		 (new 'vm-vs-new)
-		 (unread 'vm-vs-unread)
-		 (read 'vm-vs-read)
-		 (deleted 'vm-vs-deleted)
-		 (replied 'vm-vs-replied)
-		 (forwarded 'vm-vs-forwarded)
-		 (filed 'vm-vs-filed)
-		 (written 'vm-vs-written)
-		 (edited 'vm-vs-edited)
-		 (marked 'vm-vs-marked)))
-	  forms))
-
 (defun vm-string-assoc (elt list)
   (let ((case-fold-search t)
 	(found nil)
@@ -760,3 +750,24 @@ If HACK-ADDRESSES is t, then the strings are considered to be mail addresses,
 	      (list 'setq 'debug-on-error t)
 	      (list 'error "assertion failed: %S"
 		    (list 'quote expression)))))
+
+(defun vm-time-difference (t1 t2)
+  (let (usecs secs 65536-secs carry)
+    (setq usecs (- (nth 2 t1) (nth 2 t2)))
+    (if (< usecs 0)
+	(setq carry 1
+	      usecs (+ usecs 1000000))
+      (setq carry 0))
+    (setq secs (- (nth 1 t1) (nth 1 t2) carry))
+    (if (< secs 0)
+	 (setq carry 1
+	       secs (+ secs 65536))
+      (setq carry 0))
+    (setq 65536-secs (- (nth 0 t1) (nth 0 t2) carry))
+    (+ (* 65536-secs 65536)
+       secs
+       (/ usecs (if (featurep 'lisp-float-type) 1e6 1000000)))))
+
+(if (fboundp 'char-to-int)
+    (fset 'vm-char-to-int 'char-to-int)
+  (fset 'vm-char-to-int 'identity))
