@@ -18,20 +18,29 @@
 (provide 'vm-mouse)
 
 (defun vm-mouse-fsfemacs-mouse-p ()
-  (and vm-fsfemacs-19-p
+  (and vm-fsfemacs-p
        (fboundp 'set-mouse-position)))
 
 (defun vm-mouse-xemacs-mouse-p ()
   (and vm-xemacs-p
        (fboundp 'set-mouse-position)))
 
-(defun vm-mouse-set-mouse-track-highlight (start end)
-  (cond (vm-fsfemacs-19-p
-	 (let ((o (make-overlay start end)))
-	   (overlay-put o 'mouse-face 'highlight)))
-	(vm-xemacs-p
-	 (let ((o (make-extent start end)))
-	   (set-extent-property o 'highlight t)))))
+(defun vm-mouse-set-mouse-track-highlight (start end &optional overlay)
+  (if (null overlay)
+	(cond (vm-fsfemacs-p
+	       (let ((o (make-overlay start end)))
+		 (overlay-put o 'mouse-face 'highlight)
+		 o ))
+	      (vm-xemacs-p
+	       (let ((o (make-extent start end)))
+		 (set-extent-property o 'start-open t)
+		 (set-extent-property o 'priority 10)
+		 (set-extent-property o 'highlight t)
+		 o )))
+    (cond (vm-fsfemacs-p
+	   (move-overlay overlay start end))
+	  (vm-xemacs-p
+	   (set-extent-endpoints overlay start end)))))
 
 (defun vm-mouse-button-2 (event)
   (interactive "e")
@@ -90,7 +99,7 @@
 	  ((vm-mouse-fsfemacs-mouse-p)
 	   (set-buffer (window-buffer (posn-window (event-start event))))
 	   (goto-char (posn-point (event-start event)))))
-    (cond (vm-fsfemacs-19-p
+    (cond (vm-fsfemacs-p
 	   (let ((o-list (overlays-at (point)))
 		 (string nil))
 	     (while o-list
@@ -209,6 +218,8 @@
   (if (null new-mosaic)
       (let ((pid-file "~/.mosaicpid")
 	    (work-buffer " *mosaic work*")
+	    (coding-system-for-read 'no-conversion)
+	    (coding-system-for-write 'no-conversion)
 	    pid)
 	(cond ((file-exists-p pid-file)
 	       (set-buffer (get-buffer-create work-buffer))
@@ -298,6 +309,7 @@
 (defvar vm-mouse-read-file-name-initial)
 (defvar vm-mouse-read-file-name-history)
 (defvar vm-mouse-read-file-name-return-value)
+(defvar vm-mouse-read-file-name-should-delete-frame)
 
 (defun vm-mouse-read-file-name (prompt &optional dir default
 				       must-match initial history)
@@ -316,6 +328,7 @@ HISTORY argument is ignored."
     (make-local-variable 'vm-mouse-read-file-name-initial)
     (make-local-variable 'vm-mouse-read-file-name-history)
     (make-local-variable 'vm-mouse-read-file-name-return-value)
+    (make-local-variable 'vm-mouse-read-file-name-should-delete-frame)
     (setq vm-mouse-read-file-name-prompt prompt)
     (setq vm-mouse-read-file-name-dir dir)
     (setq vm-mouse-read-file-name-default default)
@@ -324,8 +337,11 @@ HISTORY argument is ignored."
     (setq vm-mouse-read-file-name-history history)
     (setq vm-mouse-read-file-name-prompt prompt)
     (setq vm-mouse-read-file-name-return-value nil)
-    (if (and vm-frame-per-completion (vm-multiple-frames-possible-p))
+    (setq vm-mouse-read-file-name-should-delete-frame nil)
+    (if (and vm-mutable-frames vm-frame-per-completion
+	     (vm-multiple-frames-possible-p))
 	(save-excursion
+	  (setq vm-mouse-read-file-name-should-delete-frame t)
 	  (vm-goto-new-frame 'completion)))
     (switch-to-buffer (current-buffer))
     (vm-mouse-read-file-name-event-handler)
@@ -345,9 +361,6 @@ HISTORY argument is ignored."
 	(cond ((equal string key-doc)
 	       (condition-case nil
 		   (save-excursion
-		     (save-excursion
-		       (let ((vm-mutable-frames t))
-			 (vm-delete-windows-or-frames-on (current-buffer))))
 		     (setq vm-mouse-read-file-name-return-value
 			   (save-excursion
 			     (vm-keyboard-read-file-name
@@ -383,22 +396,24 @@ HISTORY argument is ignored."
     (insert ?\n ?\n)
     (setq list (vm-delete-backup-file-names
 		(vm-delete-auto-save-file-names
-		 (directory-files default-directory))))
+		 (vm-delete-index-file-names
+		  (directory-files default-directory)))))
     (vm-show-list list 'vm-mouse-read-file-name-event-handler)
     (setq buffer-read-only t)))
 
 (defun vm-mouse-read-file-name-quit-handler (&optional normal-exit)
   (interactive)
-  (let ((vm-mutable-frames t))
-    (vm-delete-windows-or-frames-on (current-buffer))
-    (if normal-exit
-	(throw 'exit nil)
-      (throw 'exit t))))
+  (if vm-mouse-read-file-name-should-delete-frame
+      (vm-maybe-delete-windows-or-frames-on (current-buffer)))
+  (if normal-exit
+      (throw 'exit nil)
+    (throw 'exit t)))
 
 (defvar vm-mouse-read-string-prompt)
 (defvar vm-mouse-read-string-completion-list)
 (defvar vm-mouse-read-string-multi-word)
 (defvar vm-mouse-read-string-return-value)
+(defvar vm-mouse-read-string-should-delete-frame)
 
 (defun vm-mouse-read-string (prompt completion-list &optional multi-word)
   (save-excursion
@@ -409,12 +424,16 @@ HISTORY argument is ignored."
     (make-local-variable 'vm-mouse-read-string-completion-list)
     (make-local-variable 'vm-mouse-read-string-multi-word)
     (make-local-variable 'vm-mouse-read-string-return-value)
+    (make-local-variable 'vm-mouse-read-string-should-delete-frame)
     (setq vm-mouse-read-string-prompt prompt)
     (setq vm-mouse-read-string-completion-list completion-list)
     (setq vm-mouse-read-string-multi-word multi-word)
     (setq vm-mouse-read-string-return-value nil)
-    (if (and vm-frame-per-completion (vm-multiple-frames-possible-p))
+    (setq vm-mouse-read-string-should-delete-frame nil)
+    (if (and vm-mutable-frames vm-frame-per-completion
+	     (vm-multiple-frames-possible-p))
 	(save-excursion
+	  (setq vm-mouse-read-string-should-delete-frame t)
 	  (vm-goto-new-frame 'completion)))
     (switch-to-buffer (current-buffer))
     (vm-mouse-read-string-event-handler)
@@ -438,9 +457,6 @@ HISTORY argument is ignored."
 	(cond ((equal string key-doc)
 	       (condition-case nil
 		   (save-excursion
-		     (save-excursion
-		       (let ((vm-mutable-frames t))
-			 (vm-delete-windows-or-frames-on (current-buffer))))
 		     (setq vm-mouse-read-string-return-value
 			   (vm-keyboard-read-string
 			    vm-mouse-read-string-prompt
@@ -491,8 +507,8 @@ HISTORY argument is ignored."
 
 (defun vm-mouse-read-string-quit-handler (&optional normal-exit)
   (interactive)
-  (let ((vm-mutable-frames t))
-    (vm-delete-windows-or-frames-on (current-buffer))
-    (if normal-exit
-	(throw 'exit nil)
-      (throw 'exit t))))
+  (if vm-mouse-read-string-should-delete-frame
+      (vm-maybe-delete-windows-or-frames-on (current-buffer)))
+  (if normal-exit
+      (throw 'exit nil)
+    (throw 'exit t)))
