@@ -25,11 +25,18 @@ value of vm-auto-next-message is non-nil.
 Prefix argument N means scroll forward N lines."
   (interactive "P")
   (let ((mp-changed (vm-follow-summary-cursor))
+	needs-decoding 
 	(was-invisible nil))
     (vm-select-folder-buffer)
     (vm-check-for-killed-summary)
     (vm-check-for-killed-presentation)
     (vm-error-if-folder-empty)
+    (setq needs-decoding (and vm-display-using-mime
+			      (not vm-mime-decoded)
+			      (not (vm-mime-plain-message-p
+				    (car vm-message-pointer)))
+			      vm-auto-decode-mime-messages
+			      (eq vm-system-state 'previewing)))
     (and vm-presentation-buffer
 	 (set-buffer vm-presentation-buffer))
     (let ((point (point))
@@ -46,7 +53,7 @@ Prefix argument N means scroll forward N lines."
 	    (if (= (window-start w) (point-max))
 		(set-window-start w (point-min)))
 	    (setq was-invisible t))))
-    (if (or mp-changed was-invisible
+    (if (or mp-changed was-invisible needs-decoding
 	    (and (eq vm-system-state 'previewing)
 		 (pos-visible-in-window-p
 		  (point-max)
@@ -193,7 +200,7 @@ Prefix argument N means scroll forward N lines."
     (message "End of message %s"
 			(vm-number-of (car vm-message-pointer)))))
 
-(defun vm-scroll-backward (arg)
+(defun vm-scroll-backward (&optional arg)
   "Scroll backward a screenful of text.
 Prefix N scrolls backward N lines."
   (interactive "P")
@@ -299,7 +306,7 @@ Prefix N scrolls backward N lines."
 		  (set-extent-property e 'balloon-help 'vm-url-help)
 		  (set-extent-property e 'highlight t))))
 	  (setq search-pairs (cdr search-pairs)))))
-     ((and vm-fsfemacs-19-p
+     ((and vm-fsfemacs-p
 	   (fboundp 'overlay-put))
       (let (o-lists o p)
 	(setq o-lists (overlay-lists)
@@ -377,7 +384,7 @@ Prefix N scrolls backward N lines."
 	  (set-extent-property e 'balloon-help 'vm-mouse-3-help)
 	  (set-extent-property e 'highlight t))
 	(setq search-tuples (cdr search-tuples)))))
-   ((and vm-fsfemacs-19-p
+   ((and vm-fsfemacs-p
 	 (fboundp 'overlay-put))
     (let ((search-tuples '(("^From:" vm-menu-fsfemacs-author-menu)
 			   ("^Subject:" vm-menu-fsfemacs-subject-menu)))
@@ -430,7 +437,10 @@ Prefix N scrolls backward N lines."
 	    (setq g (intern h vm-xface-cache))
 	    (if (boundp g)
 		(setq g (symbol-value g))
-	      (set g (make-glyph h))
+	      (set g (make-glyph
+		      (list
+		       (list 'global (cons '(tty) [nothing]))
+		       (list 'global (cons '(win) (vector 'xface ':data h))))))
 	      (setq g (symbol-value g))
 	      ;; XXX broken.  Gives extra pixel lines at the
 	      ;; bottom of the glyph in 19.12
@@ -488,7 +498,6 @@ Use mouse button 3 to choose a Web browser for the URL."
   ;; display xfaces, if we can
   (if (and vm-display-xfaces
 	   vm-xemacs-p
-	   (vm-multiple-frames-possible-p)
 	   (featurep 'xface))
       (save-restriction
 	(widen)
