@@ -296,17 +296,18 @@ vm-included-text-prefix is prepended to every yanked line."
 		     (setq parts (copy-sequence (vm-mm-layout-parts o))))
 		    (t (setq parts (list o))))
 	      (while parts
-		(cond ((vm-mime-text-type-p (car parts))
+		(cond ((vm-mime-text-type-layout-p (car parts))
 		       (if (cond ((vm-mime-types-match
-				   "text/html"
-				   (car (vm-mm-layout-type (car parts))))
-				  (vm-mime-display-internal-text/html
-				   (car parts)))
-				 ((vm-mime-types-match
 				   "text/enriched"
 				   (car (vm-mm-layout-type (car parts))))
 				  (vm-mime-display-internal-text/enriched
 				   (car parts)))
+;; no text/html for now
+;;				 ((vm-mime-types-match
+;;				   "text/html"
+;;				   (car (vm-mm-layout-type (car parts))))
+;;				  (vm-mime-display-internal-text/html
+;;				   (car parts)))
 				 ((vm-mime-display-internal-text/plain
 				   (car parts) t)))
 			   nil
@@ -331,6 +332,11 @@ vm-included-text-prefix is prepended to every yanked line."
 			      (vm-text-end-of message))
 	    (setq end (vm-marker (+ start (- (vm-text-end-of message)
 					     (vm-headers-of message))) b)))))
+      ;; get rid of read-only text properties on the text, as
+      ;; they will only cause trouble.
+      (let ((inhibit-read-only t))
+	(remove-text-properties (point-min) (point-max) '(read-only nil)
+				(current-buffer)))
       (push-mark end)
       (cond (mail-citation-hook (run-hooks 'mail-citation-hook))
 	    (mail-yank-hooks (run-hooks 'mail-yank-hooks))
@@ -401,6 +407,22 @@ as replied to, forwarded, etc, if appropriate."
     (if (and vm-confirm-mail-send
 	     (not (y-or-n-p "Send the message? ")))
 	(error "Message not sent.")))
+  ;; insert a Date header if it's not already present.
+  (if (null (vm-mail-mode-get-header-contents "Date:"))
+      (save-restriction
+	(save-excursion
+	  (let* ((timezone (car (current-time-zone)))
+		 (hour (/ timezone 3600))
+		 (min (/ (- timezone (* hour 3600)) 60)))
+	    (widen)
+	    (goto-char (point-min))
+	    (insert (format-time-string "Date: %a, %e %b %Y %H:%M:%S")
+		    (format " %s%02d%02d"
+			    (if (< timezone 0) "-" "+")
+			    (abs hour)
+			    (abs min))
+		    (format-time-string " (%Z)")
+		    "\n")))))
   ;; send mail using MIME if user requests it and if the buffer
   ;; has not already been MIME encoded.
   (if (and vm-send-using-mime
@@ -442,14 +464,14 @@ as replied to, forwarded, etc, if appropriate."
     ;; buffer inside mail-send.
     (if (eq (current-buffer) composition-buffer)
 	(progn
+	  (cond ((eq vm-system-state 'replying)
+		 (vm-mail-mark-replied))
+		((eq vm-system-state 'forwarding)
+		 (vm-mail-mark-forwarded))
+		((eq vm-system-state 'redistributing)
+		 (vm-mail-mark-redistributed)))
 	  (vm-rename-current-mail-buffer)
 	  (vm-keep-mail-buffer (current-buffer))))
-    (cond ((eq vm-system-state 'replying)
-	   (vm-mail-mark-replied))
-	  ((eq vm-system-state 'forwarding)
-	   (vm-mail-mark-forwarded))
-	  ((eq vm-system-state 'redistributing)
-	   (vm-mail-mark-redistributed)))
     (vm-display nil nil '(vm-mail-send) '(vm-mail-send))))
 
 (defun vm-mail-mode-get-header-contents (header-name-regexp)
@@ -946,7 +968,8 @@ found, the current buffer remains selected."
 	  ;; different composition buffer.
 	  (vm-unbury-buffer b)
 	  (set-buffer b)
-	  (if (and vm-frame-per-composition (vm-multiple-frames-possible-p)
+	  (if (and vm-mutable-frames vm-frame-per-composition
+		   (vm-multiple-frames-possible-p)
 		   ;; only pop up a frame if there's an undisplay
 		   ;; hook in place to make the frame go away.
 		   vm-undisplay-buffer-hook)
@@ -1010,7 +1033,7 @@ found, the current buffer remains selected."
       (if (eq mail-aliases t)
 	  (progn
 	    (setq mail-aliases nil)
-	    (if (file-exists-p "~/.mailrc")
+	    (if (file-exists-p (or mail-personal-alias-file "~/.mailrc"))
 		(build-mail-aliases)))))
     (if (stringp vm-mail-header-from)
 	(insert "From: " vm-mail-header-from "\n"))
@@ -1021,11 +1044,15 @@ found, the current buffer remains selected."
     (and in-reply-to (insert "In-Reply-To: " in-reply-to "\n"))
     (and references (insert "References: " references "\n"))
     (insert "X-Mailer: VM " vm-version " under "
-	    (if vm-fsfemacs-19-p "Emacs " "")
+	    (if vm-fsfemacs-p "Emacs " "")
 	    emacs-version "\n")
+    (let ((time (current-time)))
+      (insert (format "Message-ID: <%d.%d.%d.%d@%s>\n"
+		      (car time) (nth 1 time) (nth 2 time) (random 1000000)
+		      (system-name))))
     ;; REPLYTO environmental variable support
     ;; note that in FSF Emacs v19.29 we would initialize if the
-    ;; value was t.  nil is the treigger value used now.
+    ;; value was t.  nil is the trigger value used now.
     (and (eq mail-default-reply-to nil)
 	 (setq mail-default-reply-to (getenv "REPLYTO")))
     (if mail-default-reply-to
@@ -1044,7 +1071,7 @@ found, the current buffer remains selected."
 	     (insert mail-signature)))
 	  ((eq mail-signature t)
 	   (save-excursion
-	     (insert "-- \n")
+	     (insert "\n-- \n")
 	     (insert-file-contents (or (and (boundp 'mail-signature-file)
 					    (stringp mail-signature-file)
 					    mail-signature-file)
@@ -1053,7 +1080,8 @@ found, the current buffer remains selected."
     ;; config stuff will select it as the composition buffer.
     (vm-unbury-buffer (current-buffer))
     ;; make a new frame if the user wants it.
-    (if (and vm-frame-per-composition (vm-multiple-frames-possible-p))
+    (if (and vm-mutable-frames vm-frame-per-composition
+	     (vm-multiple-frames-possible-p))
 	(progn
 	  (vm-goto-new-frame 'composition)
 	  (vm-set-hooks-for-frame-deletion)))
