@@ -183,11 +183,13 @@ The saved messages are flagged as `filed'."
 	      (t
 	       (vm-read-file-name "Save in folder: " dir nil)))))
     (prefix-numeric-value current-prefix-arg)))
-  (let (unexpanded-folder)
-    (setq unexpanded-folder folder)
+  (let (auto-folder unexpanded-folder)
     (vm-select-folder-buffer)
     (vm-check-for-killed-summary)
     (vm-error-if-folder-empty)
+    (setq unexpanded-folder folder
+	  auto-folder (vm-auto-select-folder vm-message-pointer
+					     vm-auto-folder-alist))
     (vm-display nil nil '(vm-save-message) '(vm-save-message))
     (or count (setq count 1))
     ;; Expand the filename, forcing relative paths to resolve
@@ -347,7 +349,9 @@ The saved messages are flagged as `filed'."
 	    (if (interactive-p)
 		(message "%d message%s saved to %s"
 			 count (if (/= 1 count) "s" "") folder)))))
-    (setq vm-last-save-folder unexpanded-folder)
+    (if (or (null vm-last-save-folder)
+	    (not (equal unexpanded-folder auto-folder)))
+	(setq vm-last-save-folder unexpanded-folder))
     (if vm-delete-after-saving
 	(vm-delete-message count))))
 
@@ -514,14 +518,19 @@ Output, if any, is displayed.  The message is not altered."
   (vm-check-for-killed-summary)
   (vm-error-if-folder-empty)
   (or count (setq count 1))
-  (let ((buffer (get-buffer-create "*Shell Command Output*"))
-	(command (mapconcat (function identity)
-			    (nconc (list vm-print-command)
-				   vm-print-command-switches)
-			    " "))
-	(m nil)
-	(pop-up-windows (and pop-up-windows (eq vm-mutable-windows t)))
-	(mlist (vm-select-marked-or-prefixed-messages count)))
+  (let* ((buffer (get-buffer-create "*Shell Command Output*"))
+	 (need-tempfile (string-match ".*-.*-\\(win95\\|nt\\)"
+				      system-configuration))
+	 (tempfile (if need-tempfile (vm-make-tempfile-name)))
+	 (command (mapconcat (function identity)
+			     (nconc (list vm-print-command)
+				    (copy-sequence vm-print-command-switches)
+				    (if need-tempfile
+					(list tempfile)))
+			     " "))
+	 (m nil)
+	 (pop-up-windows (and pop-up-windows (eq vm-mutable-windows t)))
+	 (mlist (vm-select-marked-or-prefixed-messages count)))
     (set-buffer buffer)
     (erase-buffer)
     (while mlist
@@ -545,20 +554,30 @@ Output, if any, is displayed.  The message is not altered."
 		    (vm-decode-mime-layout (vm-mm-layout m)))
 		  (let ((pop-up-windows (and pop-up-windows
 					     (eq vm-mutable-windows t))))
+		    (if need-tempfile
+			(write-region (point-min) (point-max)
+				      tempfile nil 0))
 		    (call-process-region (point-min) (point-max)
 					 (or shell-file-name "sh")
 					 nil buffer nil
-					 shell-command-switch command)))
+					 shell-command-switch command)
+		    (if need-tempfile
+			(vm-error-free-call 'delete-file tempfile))))
 	      (and work-buffer (kill-buffer work-buffer))))
 	(save-restriction
 	  (widen)
 	  (narrow-to-region (vm-vheaders-of m) (vm-text-end-of m))
 	  (let ((pop-up-windows (and pop-up-windows
 				     (eq vm-mutable-windows t))))
+	    (if need-tempfile
+		(write-region (point-min) (point-max)
+			      tempfile nil 0))
 	    (call-process-region (point-min) (point-max)
 				 (or shell-file-name "sh")
 				 nil buffer nil
-				 shell-command-switch command))))
+				 shell-command-switch command)
+	    (if need-tempfile
+		(vm-error-free-call 'delete-file tempfile)))))
       (setq mlist (cdr mlist)))
     (set-buffer buffer)
     (if (not (zerop (buffer-size)))
