@@ -1,5 +1,5 @@
 ;;; Entry points for VM
-;;; Copyright (C) 1994-1997 Kyle E. Jones
+;;; Copyright (C) 1994-1998 Kyle E. Jones
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
 
 (provide 'vm-startup)
 
+;;;###autoload
 (defun vm (&optional folder read-only)
   "Read mail under Emacs.
 Optional first arg FOLDER specifies the folder to visit.  It defaults
@@ -45,6 +46,7 @@ See the documentation for vm-mode for more information."
   ;; set enable-local-variables to nil for newer Emacses
   (catch 'done
     (let ((full-startup (not (bufferp folder)))
+	  (did-read-index-file nil)
 	  folder-buffer first-time totals-blurb
 	  preserve-auto-save-file)
       (setq folder-buffer
@@ -129,23 +131,29 @@ See the documentation for vm-mode for more information."
       ;; needs to be done 
       (if first-time
 	  (progn
-	    (if (fboundp 'buffer-disable-undo)
-		(buffer-disable-undo (current-buffer))
-	      ;; obfuscation to make the v19 compiler not whine
-	      ;; about obsolete functions.
-	      (let ((x 'buffer-flush-undo))
-		(funcall x (current-buffer))))
+	    (buffer-disable-undo (current-buffer))
 	    (abbrev-mode 0)
 	    (auto-fill-mode 0)
-	    (vm-mode-internal)))
-      (vm-assimilate-new-messages nil t)
-      (if first-time
+	    (vm-mode-internal)
+	    ;; If the buffer is modified we don't know if the
+	    ;; folder format has been changed to be different
+	    ;; from index file, so don't read the index file in
+	    ;; that case.
+	    (if (not (buffer-modified-p))
+		(setq did-read-index-file (vm-read-index-file-maybe)))))
+
+      (vm-assimilate-new-messages nil (not did-read-index-file) nil)
+
+      (if (and first-time (not did-read-index-file))
 	  (progn
 	    (vm-gobble-visible-header-variables)
 	    (vm-gobble-bookmark)
+	    (vm-gobble-pop-retrieved)
 	    (vm-gobble-summary)
-	    (vm-gobble-labels)
-	    (vm-start-itimers-if-needed)))
+	    (vm-gobble-labels)))
+
+      (if first-time
+	  (vm-start-itimers-if-needed))
 
       ;; make a new frame if the user wants one.  reuse an
       ;; existing frame that is showing this folder.
@@ -179,7 +187,9 @@ See the documentation for vm-mode for more information."
       ;; toolbar sets frame-specific height and width specifiers.
       (and (vm-toolbar-support-possible-p) vm-use-toolbar
 	   (progn
+	     (message "Initializing toolbar...")
 	     (vm-toolbar-install-toolbar)
+	     (message "Initializing toolbar... done")
 	     (vm-toolbar-update-toolbar)))
 
       (and vm-use-menus (vm-menu-support-possible-p)
@@ -250,6 +260,7 @@ See the documentation for vm-mode for more information."
 	    (if (not (input-pending-p))
 		(message totals-blurb)))))))
 
+;;;###autoload
 (defun vm-other-frame (&optional folder read-only)
   "Like vm, but run in a newly created frame."
   (interactive (list nil current-prefix-arg))
@@ -264,6 +275,7 @@ See the documentation for vm-mode for more information."
   (if (vm-multiple-frames-possible-p)
       (vm-set-hooks-for-frame-deletion)))
 
+;;;###autoload
 (defun vm-other-window (&optional folder read-only)
   "Like vm, but run in a different window."
   (interactive (list nil current-prefix-arg))
@@ -277,10 +289,11 @@ See the documentation for vm-mode for more information."
 
 (put 'vm-mode 'mode-class 'special)
 
+;;;###autoload
 (defun vm-mode (&optional read-only)
   "Major mode for reading mail.
 
-This is VM 6.34.
+This is VM 6.43.
 
 Commands:
    h - summarize folder contents
@@ -344,7 +357,7 @@ Commands:
 
  M N - use marks; the next vm command will affect only marked messages
        if it makes sense for the command to do so.  These commands
-       apply and remove marks to messages.
+       apply and remove marks to messages:
 
        M M - mark the current message
        M U - unmark the current message
@@ -360,6 +373,7 @@ Commands:
        M a - unmark messages with the same author as the current message
        M R - mark messages within the point/mark region in the summary
        M r - unmark messages within the point/mark region in the summary
+       M V - toggle the marked-ness of all messages
 
        M ? - partial help for mark commands
 
@@ -371,7 +385,11 @@ Commands:
  V V - visit a virtual folder (must be defined in vm-virtual-folder-alist)
  V C - create a virtual folder composed of a subset of the
        current folder's messages.
- V A - apply the selectors of a named virtual folder to the
+ V A - create a virtual folder containing all the messages in the current
+       folder with the same author as the current message.
+ V S - create a virtual folder containing all the messages in the current
+       folder with the same subject as the current message.
+ V X - apply the selectors of a named virtual folder to the
        messages in the current folder and create a virtual folder
        containing the selected messages.
  V M - toggle whether this virtual folder's messages mirror the
@@ -451,6 +469,7 @@ Variables:
    vm-highlighted-header-regexp
    vm-honor-page-delimiters
    vm-image-directory
+   vm-index-file-suffix
    vm-in-reply-to-format
    vm-included-text-attribution-format
    vm-included-text-discard-header-regexp
@@ -466,6 +485,7 @@ Variables:
    vm-mail-mode-hook
    vm-make-crash-box-name
    vm-make-spool-file-name
+   vm-mime-7bit-composition-charset
    vm-mime-8bit-composition-charset
    vm-mime-8bit-text-transfer-encoding
    vm-mime-alternative-select-method
@@ -476,6 +496,7 @@ Variables:
    vm-mime-base64-decoder-switches
    vm-mime-base64-encoder-program
    vm-mime-base64-encoder-switches
+   vm-mime-button-format-alist
    vm-mime-button-face
    vm-mime-charset-font-alist
    vm-mime-default-face-charsets
@@ -497,7 +518,9 @@ Variables:
    vm-mutable-windows
    vm-netscape-program
    vm-netscape-program-switches
+   vm-pop-auto-expunge-alist
    vm-pop-bytes-per-session
+   vm-pop-expunge-after-retrieving
    vm-pop-max-message-size
    vm-pop-md5-program
    vm-pop-messages-per-session
@@ -564,6 +587,7 @@ Variables:
   (vm (current-buffer) read-only)
   (vm-display nil nil '(vm-mode) '(vm-mode)))
 
+;;;###autoload
 (defun vm-visit-folder (folder &optional read-only)
   "Visit a mail file.
 VM will parse and present its messages to you in the usual way.
@@ -602,6 +626,7 @@ visited folder."
     (setq folder (expand-file-name folder)))
   (vm folder read-only))
 
+;;;###autoload
 (defun vm-visit-folder-other-frame (folder &optional read-only)
   "Like vm-visit-folder, but run in a newly created frame."
   (interactive
@@ -630,6 +655,7 @@ visited folder."
   (if (vm-multiple-frames-possible-p)
       (vm-set-hooks-for-frame-deletion)))
 
+;;;###autoload
 (defun vm-visit-folder-other-window (folder &optional read-only)
   "Like vm-visit-folder, but run in a different window."
   (interactive
@@ -673,6 +699,7 @@ vm-visit-virtual-folder.")
 
 (defvar scroll-in-place)
 
+;;;###autoload
 (defun vm-visit-virtual-folder (folder-name &optional read-only)
   (interactive
    (let ((last-command last-command)
@@ -768,6 +795,7 @@ vm-visit-virtual-folder.")
 	 (vm-display-startup-message)
 	 (message blurb))))
 
+;;;###autoload
 (defun vm-visit-virtual-folder-other-frame (folder-name &optional read-only)
   "Like vm-visit-virtual-folder, but run in a newly created frame."
   (interactive
@@ -787,6 +815,7 @@ vm-visit-virtual-folder.")
   (if (vm-multiple-frames-possible-p)
       (vm-set-hooks-for-frame-deletion)))
 
+;;;###autoload
 (defun vm-visit-virtual-folder-other-window (folder-name &optional read-only)
   "Like vm-visit-virtual-folder, but run in a different window."
   (interactive
@@ -805,6 +834,7 @@ vm-visit-virtual-folder.")
 	(vm-search-other-frames nil))
     (vm-visit-virtual-folder folder-name read-only)))
 
+;;;###autoload
 (defun vm-mail (&optional to)
   "Send a mail message from within VM, or from without.
 Optional argument TO is a string that should contain a comma separated
@@ -817,6 +847,7 @@ recipient list."
   (run-hooks 'vm-mail-hook)
   (run-hooks 'vm-mail-mode-hook))
 
+;;;###autoload
 (defun vm-mail-other-frame (&optional to)
   "Like vm-mail, but run in a newly created frame.
 Optional argument TO is a string that should contain a comma separated
@@ -831,6 +862,7 @@ recipient list."
   (if (vm-multiple-frames-possible-p)
       (vm-set-hooks-for-frame-deletion)))
 
+;;;###autoload
 (defun vm-mail-other-window (&optional to)
   "Like vm-mail, but run in a different window.
 Optional argument TO is a string that should contain a comma separated
@@ -844,6 +876,7 @@ recipient list."
 	(vm-search-other-frames nil))
     (vm-mail to)))
 
+;;;###autoload
 (defun vm-submit-bug-report ()
   "Submit a bug report, with pertinent information to the VM bug list."
   (interactive)
@@ -911,6 +944,7 @@ recipient list."
       'vm-included-text-discard-header-regexp
       'vm-included-text-headers
       'vm-included-text-prefix
+      'vm-index-file-suffix
       'vm-init-file
       'vm-invisible-header-regexp
       'vm-jump-to-new-messages
@@ -923,6 +957,7 @@ recipient list."
       'vm-make-spool-file-name
       'vm-mail-check-interval
       'vm-mail-mode-hook
+      'vm-mime-7bit-composition-charset
       'vm-mime-8bit-composition-charset
       'vm-mime-8bit-text-transfer-encoding
       'vm-mime-alternative-select-method
@@ -933,6 +968,7 @@ recipient list."
       'vm-mime-base64-decoder-switches
       'vm-mime-base64-encoder-program
       'vm-mime-base64-encoder-switches
+      'vm-mime-button-format-alist
       'vm-mime-button-face
       'vm-mime-charset-font-alist
       'vm-mime-default-face-charsets
@@ -949,13 +985,17 @@ recipient list."
       'vm-mosaic-program-switches
       'vm-move-after-deleting
       'vm-move-after-undeleting
+      'vm-move-after-killing
       'vm-move-messages-physically
       'vm-movemail-program
       'vm-mutable-frames
       'vm-mutable-windows
       'vm-netscape-program
       'vm-netscape-program-switches
+;; POP passwords might be listed here
+;;      'vm-pop-auto-expunge-alist
       'vm-pop-bytes-per-session
+      'vm-pop-expunge-after-retrieving
       'vm-pop-max-message-size
       'vm-pop-messages-per-session
       'vm-pop-md5-program
@@ -1048,12 +1088,14 @@ recipient list."
 		       (< emacs-minor-version 14))))
 	 (error "VM %s must be run on XEmacs 19.14 or a later version."
 		vm-version))
-	((and vm-fsfemacs-19-p
+	((and vm-fsfemacs-p
 	      (or (< emacs-major-version 19)
 		  (and (= emacs-major-version 19)
 		       (< emacs-minor-version 34))))
-	 (error "VM %s must be run on Emacs 19.34 or a later version."
-		vm-version))))
+	 (error "VM %s must be run on Emacs 19.34 or a later v19 version."
+		vm-version))
+	((and vm-fsfemacs-p (= emacs-major-version 20))
+	 (error "VM has not been ported to v20 Emacs.  Running VM in this environment is not advised."))))
 
 (defun vm-set-debug-flags ()
   (or stack-trace-on-error
@@ -1065,9 +1107,11 @@ recipient list."
 	      args-out-of-range
 	      void-function
 	      void-variable
+	      invalid-function
 	     ))))
 
 (defun vm-session-initialization ()
+  (require 'vm)
   (vm-note-emacs-version)
   (vm-check-emacs-version)
 ;;  (vm-set-debug-flags)
