@@ -61,13 +61,13 @@
      (code-convert-region (point-min) (point-max) code *internal*)
      (set-file-coding-system code))
     (2.4
-     (setq enable-multibyte-characters t)
+     (set (make-local-variable 'enable-multibyte-characters) t)
      (if (memq code '(autodetect coding-system-automatic))
 	 nil
        (decode-coding-region st nd code)
        (set-buffer-file-coding-system code)))
     (3.0
-     (setq enable-multibyte-characters t)
+     (set (make-local-variable 'enable-multibyte-characters) t)
      (if (memq code '(autodetect automatic-conversion))
 	 nil
        (or code (setq code 'automatic-conversion))
@@ -156,6 +156,25 @@
 		 ""
 	       (make-string (- len (length str)) pad))))))
 
+(defun mule-find-charset-region (beg end &optional table)
+  (case mule-sysdep-version
+    (2.3 (code-detect-region beg end))
+    ((2.4 3.0) (find-charset-region beg end table))
+    (xemacs (charsets-in-region beg end))
+    (otherwise '(no-conversion))))
+
+(defun mule-coding-system-name (codesys)
+  (case mule-sysdep-version
+    (3.0 nil)
+    (xemacs (coding-system-name codesys))))
+
+(defun mule-find-coding-system (sys)
+  (case mule-sysdep-version
+    ((2.3 2.4) nil)
+    (3.0 (if (get sys 'coding-system) sys nil))
+    (xemacs (find-coding-system sys))
+    (otherwise nil)))
+     
 (defun mule-make-iso-character (char)
   (if (<= char 127)
       char
