@@ -1,11 +1,11 @@
 ;;; url-auth.el --- Uniform Resource Locator authorization modules
 ;; Author: wmperry
-;; Created: 1997/01/19 01:17:29
-;; Version: 1.5
+;; Created: 1997/10/22 16:44:35
+;; Version: 1.9
 ;; Keywords: comm, data, processes, hypermedia
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;; Copyright (c) 1993-1996 by William M. Perry (wmperry@cs.indiana.edu)
+;;; Copyright (c) 1993-1996 by William M. Perry <wmperry@cs.indiana.edu>
 ;;; Copyright (c) 1996, 1997 Free Software Foundation, Inc.
 ;;;
 ;;; This file is not part of GNU Emacs, but the same permissions apply.
@@ -28,6 +28,15 @@
 
 (require 'url-vars)
 (require 'url-parse)
+
+(defsubst url-auth-user-prompt (url realm)
+  "String to usefully prompt for a username."
+  (concat "Username [for "
+	  (or realm (url-truncate-url-for-viewing
+		     (url-recreate-url url)
+		     (- (window-width) 10 20)))
+	  "]: "))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Basic authorization code
 ;;; ------------------------
@@ -54,7 +63,7 @@ instead of the pathname inheritance method."
   (let* ((href (if (stringp url)
 		   (url-generic-parse-url url)
 		 url))
-	 (server (or (url-host href) url-current-server))
+	 (server (url-host href))
 	 (port (or (url-port href) "80"))
 	 (path (url-filename href))
 	 user pass byserv retval data)
@@ -66,7 +75,8 @@ instead of the pathname inheritance method."
 	  byserv (cdr-safe (assoc server url-basic-auth-storage)))
     (cond
      ((and prompt (not byserv))
-      (setq user (read-string "Username: " (user-real-login-name))
+      (setq user (read-string (url-auth-user-prompt url realm)
+			      (user-real-login-name))
 	    pass (funcall url-passwd-entry-func "Password: ")
 	    url-basic-auth-storage
 	    (cons (list server
@@ -89,7 +99,8 @@ instead of the pathname inheritance method."
 	    (setq byserv (cdr byserv))))
       (if (or (and (not retval) prompt) overwrite)
 	  (progn
-	    (setq user (read-string "Username: " (user-real-login-name))
+	    (setq user (read-string (url-auth-user-prompt url realm)
+				    (user-real-login-name))
 		  pass (funcall url-passwd-entry-func "Password: ")
 		  retval (base64-encode (format "%s:%s" user pass))
 		  byserv (assoc server url-basic-auth-storage))
@@ -134,7 +145,7 @@ instead of hostname:portnum."
       (let* ((href (if (stringp url)
 		       (url-generic-parse-url url)
 		     url))
-	     (server (or (url-host href) url-current-server))
+	     (server (url-host href))
 	     (port (or (url-port href) "80"))
 	     (path (url-filename href))
 	     user pass byserv retval data)
@@ -146,7 +157,8 @@ instead of hostname:portnum."
 	      byserv (cdr-safe (assoc server url-digest-auth-storage)))
 	(cond
 	 ((and prompt (not byserv))
-	  (setq user (read-string "Username: " (user-real-login-name))
+	  (setq user (read-string (url-auth-user-prompt url realm)
+				  (user-real-login-name))
 		pass (funcall url-passwd-entry-func "Password: ")
 		url-digest-auth-storage
 		(cons (list server
@@ -172,7 +184,8 @@ instead of hostname:portnum."
 		(setq byserv (cdr byserv))))
 	  (if (or (and (not retval) prompt) overwrite)
 	      (progn
-		(setq user (read-string "Username: " (user-real-login-name))
+		(setq user (read-string (url-auth-user-prompt url realm)
+					(user-real-login-name))
 		      pass (funcall url-passwd-entry-func "Password: ")
 		      retval (setq retval
 				   (cons user
@@ -199,7 +212,7 @@ instead of hostname:portnum."
   "A list of the registered authorization schemes and various and sundry
 information associated with them.")
 
-;;###autoload
+;;;###autoload
 (defun url-get-authentication (url realm type prompt &optional args)
   "Return an authorization string suitable for use in the WWW-Authenticate
 header in an HTTP/1.0 request.
@@ -259,7 +272,7 @@ PROMPT is boolean - specifies whether to ask the user for a username/password
 			(funcall scheme url nil nil realm args))
 		   realm args)))))
 
-;;###autoload
+;;;###autoload
 (defun url-register-auth-scheme (type &optional function rating)
   "Register an HTTP authentication method.
 
