@@ -1,7 +1,7 @@
 ;;; url-ns.el --- Various netscape-ish functions for proxy definitions
 ;; Author: wmperry
-;; Created: 1997/04/08 21:18:23
-;; Version: 1.2
+;; Created: 1997/10/03 15:19:07
+;; Version: 1.6
 ;; Keywords: comm, data, processes, hypermedia
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -25,21 +25,27 @@
 ;;; Boston, MA 02111-1307, USA.
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(require 'url-gw)
+
+;;;###autoload
 (defun isPlainHostName (host)
   (not (string-match "\\." host)))
 
+;;;###autoload
 (defun dnsDomainIs (host dom)
-  (setq host (url-gateway-nslookup-host host))
   (string-match (concat (regexp-quote dom) "$") host))
 
+;;;###autoload
 (defun dnsResolve (host)
   (url-gateway-nslookup-host host))
 
+;;;###autoload
 (defun isResolvable (host)
   (if (string-match "^[0-9.]+$" host)
       t
     (not (string= host (url-gateway-nslookup-host host)))))
 
+;;;###autoload
 (defun isInNet (ip net mask)
   (let ((netc (split-string ip "\\."))
 	(ipc  (split-string net "\\."))
@@ -60,4 +66,41 @@
        (= (logand (nth 3 netc) (nth 3 maskc))
 	  (logand (nth 3 ipc)  (nth 3 maskc)))))))
 
+;; Netscape configuration file parsing
+(defvar url-ns-user-prefs nil
+  "Internal, do not use.")
+
+;;;###autoload
+(defun url-ns-prefs (&optional file)
+  (if (not file)
+      (setq file (expand-file-name "~/.netscape/preferences.js")))
+  (if (not (and (file-exists-p file)
+		(file-readable-p file)))
+      (message "Could not open %s for reading" file)
+    (save-excursion
+      (let ((false nil)
+	    (true t))
+	(setq url-ns-user-prefs (make-hash-table :size 13 :test 'equal))
+	(set-buffer (get-buffer-create " *ns-parse*"))
+	(erase-buffer)
+	(insert-file-contents file)
+	(goto-char (point-min))
+	(while (re-search-forward "^//" nil t)
+	  (replace-match ";;"))
+	(goto-char (point-min))
+	(while (re-search-forward "^user_pref(" nil t)
+	  (replace-match "(url-ns-set-user-pref "))
+	(goto-char (point-min))
+	(while (re-search-forward "\"," nil t)
+	  (replace-match "\""))
+	(goto-char (point-min))
+	(eval-buffer)))))
+
+(defun url-ns-set-user-pref (key val)
+  (cl-puthash key val url-ns-user-prefs))
+
+;;;###autoload
+(defun url-ns-user-pref (key &optional default)
+  (cl-gethash key url-ns-user-prefs default))
+
 (provide 'url-ns)
