@@ -1,17 +1,17 @@
 ;;; url.el --- Uniform Resource Locator retrieval tool
 ;; Author: wmperry
-;; Created: 1997/05/08 22:17:40
-;; Version: 1.78
+;; Created: 1997/12/25 23:53:39
+;; Version: 1.88
 ;; Keywords: comm, data, processes, hypermedia
 
 ;;; LCD Archive Entry:
 ;;; url|William M. Perry|wmperry@cs.indiana.edu|
 ;;; Functions for retrieving/manipulating URLs|
-;;; 1997/05/08 22:17:40|1.78|Location Undetermined
+;;; 1997/12/25 23:53:39|1.88|Location Undetermined
 ;;;
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;; Copyright (c) 1993-1996 by William M. Perry (wmperry@cs.indiana.edu)
+;;; Copyright (c) 1993-1996 by William M. Perry <wmperry@cs.indiana.edu>
 ;;; Copyright (c) 1996, 1997 Free Software Foundation, Inc.
 ;;;
 ;;; This file is not part of GNU Emacs, but the same permissions apply.
@@ -613,20 +613,22 @@ accessible."
   "Insert the contents of the URL in this buffer."
   (interactive "sURL: ")
   (save-excursion
-    (let ((old-asynch url-be-asynchronous))
-      (setq-default url-be-asynchronous nil)
-      (let ((buf (current-buffer))
-	    (url-working-buffer (cdr (url-retrieve url))))
-	(setq-default url-be-asynchronous old-asynch)
-	(set-buffer url-working-buffer)
-	(url-uncompress)
-	(set-buffer buf)
-	(insert-buffer url-working-buffer)
-	(setq buffer-file-name url)
-	(save-excursion
-	  (set-buffer url-working-buffer)
-	  (set-buffer-modified-p nil))
-	(kill-buffer url-working-buffer)))))
+    (let ((old-asynch (default-value 'url-be-asynchronous)))
+      (unwind-protect
+	  (progn
+	    (setq-default url-be-asynchronous nil)
+	    (let ((buf (current-buffer))
+		  (url-working-buffer (cdr (url-retrieve url))))
+	      (set-buffer url-working-buffer)
+	      (url-uncompress)
+	      (set-buffer buf)
+	      (insert-buffer url-working-buffer)
+	      (setq buffer-file-name url)
+	      (save-excursion
+		(set-buffer url-working-buffer)
+		(set-buffer-modified-p nil))
+	      (kill-buffer url-working-buffer)))
+	(setq-default url-be-asynchronous old-asynch)))))
 
 (defun url-file-directory-p (url &rest args)
   "Return t iff a url points to a directory"
@@ -763,6 +765,7 @@ numbers, etc."
   "Just like `message', but is a no-op if called more than once a second.
 Will not do anything if url-show-status is nil."
   (if (or (null url-show-status)
+	  (active-minibuffer-window)
 	  (= url-lazy-message-time
 	     (setq url-lazy-message-time (nth 1 (current-time)))))
       nil
@@ -804,7 +807,7 @@ or 1 second has elapsed."
 	 ((eq system-type 'windows-nt) "Windows-NT; 32bit")
 	 ((eq system-type 'ms-windows) "Windows; 16bit")
 	 ((eq system-type 'ms-dos) "MS-DOS; 32bit")
-	 ((eq (device-type) 'win32) "Windows; 32bit")
+	 ((memq (device-type) '(win32 w32)) "Windows; 32bit")
 	 ((eq (device-type) 'pm) "OS/2; 32bit")
 	 (t
 	  (case (device-type)
@@ -858,6 +861,7 @@ or 1 second has elapsed."
      (t
       (url-retrieve (concat "http://www." url ".com"))))))
 
+;;;###autoload
 (defun url-setup-save-timer ()
   "Reset the history list timer."
   (interactive)
@@ -883,6 +887,7 @@ or 1 second has elapsed."
   (if (<= url-download-minor-mode 0)
       (setq url-download-minor-mode nil)))
 
+;;;###autoload
 (defun url-do-setup ()
   "Do setup - this is to avoid conflict with user settings when URL is
 dumped with emacs."
@@ -1025,7 +1030,7 @@ dumped with emacs."
     (run-hooks 'url-load-hook)
     (setq url-setup-done t)))
 
-(defvar url-get-url-filename-chars "%.?@a-zA-Z0-9---()_/:~=&"
+(defvar url-get-url-filename-chars "-%.?@a-zA-Z0-9()_/:~=&"
   "Valid characters in a URL")
 
 ;;;###autoload
@@ -1055,6 +1060,8 @@ directly on a symbol."
 	      (skip-chars-forward url-get-url-filename-chars))
 	  (setq start (point)))
 	(setq url (buffer-substring-no-properties start (point))))
+      (if (string-match "^(.*)$" url)
+	  (setq url (substring url 1 -1)))
       (if (string-match "^URL:" url)
 	  (setq url (substring url 4 nil)))
       (if (string-match "\\.$" url)
@@ -1902,7 +1909,9 @@ retrieve a URL by its HTML source."
     ;; Not sure how I should handle gracefully degrading from one proxy to
     ;; another, so for now just deal with the first one
     ;; (while proxies
-    (setq proxy (pop proxies))
+    (if (listp proxies)
+	(setq proxy (pop proxies))
+      (setq proxy proxies))
     (cond
      ((string-match "^direct" proxy) nil)
      ((string-match "^proxy +" proxy)
@@ -1942,6 +1951,8 @@ retrieve a URL by its HTML source."
 	  url (if cached (url-cache-create-filename url) url))
     (save-excursion
       (set-buffer (get-buffer-create url-working-buffer))
+      (if (boundp 'buffer-file-coding-system)
+	  (setq buffer-file-coding-system nil))
       (setq url-current-can-be-cached (not no-cache)
 	    url-current-object urlobj))
     (if (and handler (fboundp handler))
