@@ -1,7 +1,7 @@
 ;;; w3-forms.el --- Emacs-w3 forms parsing code for new display engine
 ;; Author: wmperry
-;; Created: 1997/10/17 14:08:15
-;; Version: 1.88
+;; Created: 1997/12/24 18:51:10
+;; Version: 1.89
 ;; Keywords: faces, help, comm, data, languages
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -111,6 +111,8 @@
   "An assoc list of available key sizes and meaningful descriptions.")
    
 (defun w3-form-determine-size (el size)
+  (if (equal size 0)
+      (setq size nil))
   (case (w3-form-element-type el)
     (checkbox 3)
     (radio 4)
