@@ -4,6 +4,7 @@
 ;; Copyright (C) 1995 Tinker Systems and INS Engineering Corp.
 ;; Copyright (C) 1995 Sun Microsystems.
 ;; Copyright (C) 1995, 1996 Ben Wing.
+;; Copyright (C) 1997 MORIOKA Tomohiko
 
 ;; This file is part of XEmacs.
 
@@ -28,7 +29,8 @@
 
 ;;; Warning-free compile
 (eval-when-compile
-  (defvar language-environment-list))
+  (defvar language-environment-list)
+  (require 'pending-del))
 
 (defconst default-menubar
   (purecopy-menubar
@@ -81,14 +83,16 @@
       ["Replace (Regexp)..."	query-replace-regexp	t]
       "----"
       ("Bookmarks"
-       ["Jump to bookmark"  	bookmark-menu-jump	t]
+       ("Jump to bookmark"
+	:filter bookmark-menu-filter)
        ["Set bookmark"  	bookmark-set		t]
        "---"
        ["Insert contents"  	bookmark-menu-insert	t]
        ["Insert location"  	bookmark-menu-locate	t]
        "---"
        ["Rename bookmark"  	bookmark-menu-rename	t]
-       ["Delete bookmark"  	bookmark-menu-delete	t]
+       ("Delete bookmark"
+  	:filter bookmark-delete-filter)
        ["Edit Bookmark List"    bookmark-bmenu-list	t]
        "---"
        ["Save bookmarks"        bookmark-save		t]
@@ -101,19 +105,45 @@
       ["Start Macro Recording"	start-kbd-macro	      (not defining-kbd-macro)]
       ["End Macro Recording"	end-kbd-macro		defining-kbd-macro]
       ["Execute Last Macro"	call-last-kbd-macro	last-kbd-macro]
+      "----"
+      ["Show Message Log"	show-message-log	t]
       )
-
+     
+     ,@(if (featurep 'mule)
+	   '(("Mule"
+	      ("Describe language support")
+	      ("Set language environment")
+	      "--"
+	      ["Toggle input method" toggle-input-method t]
+	      ["Select input method" select-input-method t]
+	      ["Describe input method" describe-input-method t]
+	      "--"
+	      ["Describe current coding systems"
+	       describe-current-coding-system t]
+	      ["Set coding system of buffer file"
+	       set-buffer-file-coding-system t]
+	      ["Set coding system of terminal"
+	       set-terminal-coding-system nil] ; not implemented yet
+	      ["Set coding system of keyboard"
+	       set-keyboard-coding-system nil] ; not implemented yet
+	      ["Set coding system of process"
+	       set-current-process-coding-system nil] ; not implemented yet
+	      "--"
+	      ["Show character table" view-charset-by-menu t]
+	      ["Show diagnosis for MULE" mule-diag nil] ; not implemented yet
+	      ["Show many languages" view-hello-file t]
+	      )))
+     
      ("Apps"
       ["Read Mail (VM)..."	vm			t]
       ["Read Mail (MH)..."	(mh-rmail t)		t]
       ["Send mail..."		mail			t]
-      ["Usenet News"		gnus			t]
+      ["Usenet News"		gnus			(fboundp 'gnus)]
       ["Browse the Web"		w3			t]
       ["Gopher"			gopher			t]
-      ["Hyperbole..."		hyperbole		t]
       "----"
       ["Spell-Check Buffer"	ispell-buffer		t]
-      ["Emulate VI"		viper-mode		t]
+      ["Toggle VI emulation"		toggle-viper-mode		t]
       "----"
       ("Calendar"
        ["3-Month Calendar"	calendar		t]
@@ -124,6 +154,8 @@
        ["Sunrise/Sunset"	sunrise-sunset		t]
        )
       ("Games"
+       ["Mine Game"		xmine			t]
+       ["Tetris"		tetris			t]
        ["Quote from Zippy"	yow			t]
        ["Psychoanalyst"		doctor			t]
        ["Psychoanalyze Zippy!"	psychoanalyze-pinhead	t]
@@ -132,7 +164,6 @@
        ["Towers of Hanoi"	hanoi			t]
        ["Game of Life"		life			t]
        ["Multiplication Puzzle"	mpuz			t]
-       ["Mine Game"		xmine			t]
        )
       )
 
@@ -140,10 +171,11 @@
       ("Customize"
        ("Emacs" :filter (lambda (&rest junk)
 			   (cdr (custom-menu-create 'emacs))))
-       ["Group..." customize t]
+       ["Group..." customize-group t]
        ["Variable..." customize-variable t]
        ["Face..." customize-face t]
-       ["Saved..." customize-customized t]
+       ["Saved..." customize-saved t]
+       ["Set..." customize-customized t]
        ["Apropos..." customize-apropos t])
       ["Read Only" (toggle-read-only)
        :style toggle :selected buffer-read-only]
@@ -159,12 +191,10 @@
 	:style toggle :selected (not case-fold-search)]
        ["Case Matching Replace" (setq case-replace (not case-replace))
 	:style toggle :selected case-replace]
-       ["Auto Delete Selection" (if (memq 'pending-delete-pre-hook
-					  pre-command-hook)
-				    (pending-delete-off nil)
-				  (pending-delete-on nil))
+       ["Auto Delete Selection" (pending-delete-mode
+				 (if pending-delete-mode 0 1))
 	:style toggle
-	:selected (memq 'pending-delete-pre-hook pre-command-hook)]
+	:selected (and (boundp 'pending-delete-mode) pending-delete-mode)]
        ["Active Regions" (setq zmacs-regions (not zmacs-regions))
 	:style toggle :selected zmacs-regions]
        ["Mouse Paste At Text Cursor" (setq mouse-yank-at-point
@@ -288,7 +318,7 @@
        ["Make current frame gnuserv target"
 	(setq gnuserv-frame
 	      (if (equal gnuserv-frame (selected-frame))
-		  nil
+		  'new
 		(selected-frame)))
 	:style radio
 	:selected (equal gnuserv-frame (selected-frame))]
@@ -371,25 +401,25 @@
 			   (and (integerp font-lock-maximum-decoration)
 				(>= font-lock-maximum-decoration 3))))]
        "-----"
-       ["Lazy" (progn (require 'lazy-lock)
-		      (if (and (boundp 'lazy-lock-mode) lazy-lock-mode)
+       ["Lazy" (progn (require 'lazy-shot)
+		      (if (and (boundp 'lazy-shot-mode) lazy-shot-mode)
 			  (progn
-			    (lazy-lock-mode 0)
+			    (lazy-shot-mode 0)
 			    ;; this shouldn't be necessary so there has to
 			    ;; be a redisplay bug lurking somewhere (or
 			    ;; possibly another event handler bug)
 			    (redraw-modeline)
 			    (remove-hook 'font-lock-mode-hook
-					 'turn-on-lazy-lock))
+					 'turn-on-lazy-shot))
 			(if font-lock-mode
 			    (progn
-			      (lazy-lock-mode 1)
+			      (lazy-shot-mode 1)
 			      (redraw-modeline)
 			      (add-hook 'font-lock-mode-hook
-					'turn-on-lazy-lock)))))
+					'turn-on-lazy-shot)))))
 	:active font-lock-mode
 	:style toggle
-	:selected (and (boundp 'lazy-lock-mode) lazy-lock-mode)]
+	:selected (and (boundp 'lazy-shot-mode) lazy-shot-mode)]
        ["Caching" (progn (require 'fast-lock)
 			 (if fast-lock-mode
 			     (progn
@@ -536,7 +566,13 @@
 	      (mouse-avoidance-mode 'banish))
 	  (beep)
 	  (message "This option requires a window system."))
-	:style toggle :selected (and mouse-avoidance-mode window-system)])
+	:style toggle :selected (and mouse-avoidance-mode window-system)]
+       ["strokes-mode"
+	(if (equal (device-type) 'x)
+	    (strokes-mode)
+	  (beep)
+	  (message "This option requires a window system."))
+	:style toggle :selected (and strokes-mode window-system)])
       ("Open URLs With"
        ["Emacs-W3" (setq browse-url-browser-function 'browse-url-w3)
 	:style radio
@@ -568,10 +604,6 @@
       ("Font"   :filter font-menu-family-constructor)
       ("Size"	:filter font-menu-size-constructor)
       ("Weight"	:filter font-menu-weight-constructor)
-      ,@(if (featurep 'mule)
-	    '("-----"
-	      ("Language Environment"
-	       :filter language-environment-menu-filter)))
       "-----"
       ["Save Options" save-options-menu-settings t]
       )
@@ -591,7 +623,6 @@
       ["Debug (GDB)..."		gdb			t]
       ["Debug (DBX)..."		dbx			t]
       "-----"
-      ["OO-Browser..."		oobr			t]
       ("Tags"
        ["Find Tag..."		find-tag		t]
        ["Find Other Window..."	find-tag-other-window	t]
@@ -734,7 +765,7 @@ The name of the current buffer is only appended to the menu items if
 	    (and put-buffer-names-in-file-menu
 		 (member name '("Save" "Revert Buffer" "Print Buffer"
 				"Pretty-Print Buffer" "Delete Buffer"))
-		 (>= 4 (length item))
+		 (>= (length item) 4)
 		 (aset item 3 bufname))
 	    (and (string= "Save" name)
 		 (aset item 2 (buffer-modified-p)))
@@ -742,7 +773,9 @@ The name of the current buffer is only appended to the menu items if
 		 (aset item 2 (not (not (or buffer-file-name
 					    revert-buffer-function)))))
 	    (and (string= "Delete Frame" name)
-		 (aset item 2 (not (eq (next-frame) (selected-frame)))))
+		 (aset item 2 (not (eq (next-frame (selected-frame)
+						   'nomini 'window-system)
+				       (selected-frame)))))
 	    )))
     result))
 
@@ -802,52 +835,95 @@ This function changes the sensitivity of these Edit menu items:
     result))
 
 
+;;; The Bookmarks menu
+
+(defun bookmark-menu-filter (menu-items)
+  "*Build the bookmark jump submenu dynamically from all defined bookmarks."
+  (if (bookmark-all-names)
+      (mapcar
+       #'(lambda (bmk)
+	   (vector bmk `(bookmark-jump ',bmk) t)) (bookmark-all-names))
+    '(["No Bookmarks Set" nil nil])))
+
+(defun bookmark-delete-filter (menu-items)
+  "*Build the bookmark delete submenu dynamically from all defined bookmarks."
+  (if (bookmark-all-names)
+      (mapcar
+       #'(lambda (bmk)
+	   (vector bmk `(bookmark-delete ',bmk) t)) (bookmark-all-names))
+    '(["No Bookmarks Set" nil nil])))
+
 ;;; The Buffers menu
 
-(defvar buffers-menu-max-size 25
+(defgroup buffers-menu nil
+  "Customization of `Buffers' menu."
+  :group 'menu)
+
+(defcustom buffers-menu-max-size 25
   "*Maximum number of entries which may appear on the \"Buffers\" menu.
 If this is 10, then only the ten most-recently-selected buffers will be
 shown.  If this is nil, then all buffers will be shown.  Setting this to
-a large number or nil will slow down menu responsiveness.")
+a large number or nil will slow down menu responsiveness."
+  :type '(choice (const :tag "Show all" nil)
+		 (integer 10))
+  :group 'buffers-menu)
 
-(defvar complex-buffers-menu-p nil
-  "*If true, the buffers menu will contain several commands, as submenus
-of each buffer line.  If this is false, then there will be only one command:
-select that buffer.")
+(defcustom complex-buffers-menu-p nil
+  "*If non-nil, the buffers menu will contain several commands.
+Commands will be presented as submenus of each buffer line.  If this
+is false, then there will be only one command: select that buffer."
+  :type 'boolean
+  :group 'buffers-menu)
 
-(defvar buffers-menu-submenus-for-groups-p nil
-  "*If true, the buffers menu will contain one submenu per group of buffers,
-if a grouping function is specified in `buffers-menu-grouping-function'.
+(defcustom buffers-menu-submenus-for-groups-p nil
+  "*If non-nil, the buffers menu will contain one submenu per group of buffers.
+The grouping function is specified in `buffers-menu-grouping-function'.
 If this is an integer, do not build submenus if the number of buffers
-is not larger than this value.")
+is not larger than this value."
+  :type '(choice (const :tag "No Subgroups" nil)
+		 (integer :tag "Max. submenus" 10)
+		 (sexp :format "%t\n" :tag "Allow Subgroups"))
+  :group 'buffers-menu)
 
-(defvar buffers-menu-switch-to-buffer-function 'switch-to-buffer
+(defcustom buffers-menu-switch-to-buffer-function 'switch-to-buffer
   "*The function to call to select a buffer from the buffers menu.
-`switch-to-buffer' is a good choice, as is `pop-to-buffer'.")
+`switch-to-buffer' is a good choice, as is `pop-to-buffer'."
+  :type '(radio (function-item switch-to-buffer)
+		(function-item pop-to-buffer)
+		(function :tag "Other"))
+  :group 'buffers-menu)
 
-(defvar buffers-menu-omit-function 'buffers-menu-omit-invisible-buffers
-"*If non-nil, a function specifying the buffers to omit from the buffers menu.
+(defcustom buffers-menu-omit-function 'buffers-menu-omit-invisible-buffers
+  "*If non-nil, a function specifying the buffers to omit from the buffers menu.
 This is passed a buffer and should return non-nil if the buffer should be
 omitted.  The default value `buffers-menu-omit-invisible-buffers' omits
 buffers that are normally considered \"invisible\" (those whose name
-begins with a space).")
+begins with a space)."
+  :type '(choice (const :tag "None" nil)
+		 function)
+  :group 'buffers-menu)
 
-(defvar buffers-menu-format-buffer-line-function 'format-buffers-menu-line
+(defcustom buffers-menu-format-buffer-line-function 'format-buffers-menu-line
   "*The function to call to return a string to represent a buffer in the
 buffers menu.  The function is passed a buffer and should return a string.
 The default value `format-buffers-menu-line' just returns the name of
 the buffer.  Also check out `slow-format-buffers-menu-line' which
-returns a whole bunch of info about a buffer.")
+returns a whole bunch of info about a buffer."
+  :type 'function
+  :group 'buffers-menu)
 
-(defvar buffers-menu-sort-function
+(defcustom buffers-menu-sort-function
   'sort-buffers-menu-by-mode-then-alphabetically
   "*If non-nil, a function to sort the list of buffers in the buffers menu.
 It will be passed two arguments (two buffers to compare) and should return
 T if the first is \"less\" than the second.  One possible value is
 `sort-buffers-menu-alphabetically'; another is
-`sort-buffers-menu-by-mode-then-alphabetically'.")
+`sort-buffers-menu-by-mode-then-alphabetically'."
+  :type '(choice (const :tag "None" nil)
+		 function)
+  :group 'buffers-menu)
 
-(defvar buffers-menu-grouping-function
+(defcustom buffers-menu-grouping-function
   'group-buffers-menu-by-mode-then-alphabetically
   "*If non-nil, a function to group buffers in the buffers menu together.
 It will be passed two arguments, successive members of the sorted buffers
@@ -858,7 +934,10 @@ buffers menus.  The last invocation of the function contains nil as the
 second argument, so that the name of the last group can be determined.
 
 The sensible values of this function are dependent on the value specified
-for `buffers-menu-sort-function'.")
+for `buffers-menu-sort-function'."
+  :type '(choice (const :tag "None" nil)
+		 function)
+  :group 'buffers-menu)
 
 (defun buffers-menu-omit-invisible-buffers (buf)
   "For use as a value of `buffers-menu-omit-function'.
@@ -1071,10 +1150,9 @@ of changing and saving faces via cu-edit-faces.el & custom.el.")
 	 '(overwrite-mode 1))
      `(setq-default case-fold-search ,(default-value 'case-fold-search))
      case-replace
-     (if (memq 'pending-delete-pre-hook pre-command-hook)
-	 '(progn
-	    (require 'pending-del)
-	    (pending-delete-on nil)))
+     (if (and (boundp 'pending-delete-mode)
+	      pending-delete-mode)
+	 '(pending-delete-mode 1))
      zmacs-regions
      mouse-yank-at-point
      require-final-newline
@@ -1116,9 +1194,9 @@ of changing and saving faces via cu-edit-faces.el & custom.el.")
 	 '(add-hook 'font-lock-mode-hook 'turn-on-fast-lock)
        '(remove-hook 'font-lock-mode-hook 'turn-on-fast-lock))
      (if (and (boundp 'font-lock-mode-hook)
-	      (memq 'turn-on-lazy-lock font-lock-mode-hook))
-	 '(add-hook 'font-lock-mode-hook 'turn-on-lazy-lock)
-       '(remove-hook 'font-lock-mode-hook 'turn-on-lazy-lock))
+	      (memq 'turn-on-lazy-shot font-lock-mode-hook))
+	 '(add-hook 'font-lock-mode-hook 'turn-on-lazy-shot)
+       '(remove-hook 'font-lock-mode-hook 'turn-on-lazy-shot))
 
      ;; Paren Highlighting
      (if paren-mode
@@ -1207,7 +1285,7 @@ of changing and saving faces via cu-edit-faces.el & custom.el.")
      ;; Mule-specific:
      (if (featurep 'mule)
 	 `(if (featurep 'mule)
-	      (set-language-environment ',(current-language-environment))))
+	      (set-language-environment ',current-language-environment)))
      ))
   "The variables to save; or forms to evaluate to get forms to write out.
 This is used by `save-options-menu-settings' and should mirror the
@@ -1476,7 +1554,8 @@ The menu is computed by combining `global-popup-menu' and `mode-popup-menu'."
 (global-set-key 'button3 'popup-mode-menu)
 ;; shift button3 and shift button2 are reserved for Hyperbole
 (global-set-key '(meta control button3) 'popup-buffer-menu)
-(global-set-key '(meta shift button3) 'popup-menubar-menu)
+;; The following command is way too dangerous with Custom.
+;; (global-set-key '(meta shift button3) 'popup-menubar-menu)
 
 ;; Here's a test of the cool new menu features (from Stig).
 
